@@ -21,6 +21,7 @@
  * Boston, MA 02110-1301, USA.
  */
 
+#include "../config-fontinst.h"
 #include "DuplicatesDialog.h"
 #include "Misc.h"
 #include "Fc.h"
@@ -48,6 +49,9 @@
 #include <QtGui/QApplication>
 #include <QtGui/QDesktopWidget>
 #include <QtCore/QProcess>
+#if defined USE_POLICYKIT && USE_POLICYKIT==1
+#include <QtDBus/QDBusInterface>
+#endif
 
 using namespace KDESu;
 
@@ -121,7 +125,7 @@ void CDuplicatesDialog::scanFinished()
         {
             QSize sizeB4(size());
 
-            itsLabel->setText(i18n("%1 duplicate fonts found.", duplicates.count()));
+            itsLabel->setText(i18np("%1 duplicate font found.", "%1 duplicate fonts found.", duplicates.count()));
             itsView->show();
 
             CFontFileList::TFontMap::ConstIterator it(duplicates.begin()),
@@ -163,6 +167,7 @@ void CDuplicatesDialog::scanFinished()
                              QVariant(SmallIcon(t1>tt ? "application-x-font-type1" : "application-x-font-ttf")));
                 top->setFont(COL_FILE, boldFont);
             }
+
             QTreeWidgetItem *item=0L;
             for(int i=0; item=itsView->topLevelItem(i); ++i)
                 item->setExpanded(true);
@@ -334,6 +339,22 @@ QSet<QString> CDuplicatesDialog::deleteSysFiles(const QStringList &files)
 
     if(files.count())
     {
+#if defined USE_POLICYKIT && USE_POLICYKIT==1
+        QStringList::ConstIterator it(files.begin()),
+                                   end(files.end());
+        QDBusInterface             iface(KFI_IFACE,
+                                         "/FontInst",
+                                         KFI_IFACE,
+                                         QDBusConnection::systemBus(), this);
+
+        for(; it!=end; ++it)
+        {
+            iface.call("deleteFont", (unsigned int)getpid(), *it);
+
+            if(!Misc::fExists(*it))
+                removed.insert(*it);
+        }
+#else
         QByteArray cmd("rm -f");
         QStringList::ConstIterator it(files.begin()),
                                    end(files.end());
@@ -352,6 +373,7 @@ QSet<QString> CDuplicatesDialog::deleteSysFiles(const QStringList &files)
         for(it=files.begin(); it!=end; ++it)
             if(!Misc::fExists(*it))
                 removed.insert(*it);
+#endif
     }
 
     if(removed.count())
@@ -438,7 +460,7 @@ void CFontFileList::run()
         for(int n=0; it!=end && !itsTerminated; ++it)
         {
             QStringList           add;
-            QStringList::Iterator fIt((*it).begin()),
+            QStringList::const_iterator fIt((*it).begin()),
                                   fEnd((*it).end());
 
             for(; fIt!=fEnd && !itsTerminated; ++fIt, ++n)
@@ -480,7 +502,7 @@ void CFontFileList::fileDuplicates(const QString &folder, const QSet<TFile> &fil
         // Check if this file is already know about - this will do a case-sensitive comparison
         if(!files.contains(TFile(fileInfo.fileName())))
         {
-            // OK, not found - this means its a duplicate, but different case. So, find the
+            // OK, not found - this means it is a duplicate, but different case. So, find the
             // FontMap iterator, and update its list of files.
             QSet<TFile>::ConstIterator entry=files.find(TFile(fileInfo.fileName(), true));
 
@@ -530,9 +552,9 @@ CFontFileListView::CFontFileListView(QWidget *parent)
     itsMenu->addAction(KIcon("document-properties"), i18n("Properties"),
                        this, SLOT(properties()));
     itsMenu->addSeparator();
-    itsUnMarkAct=itsMenu->addAction(i18n("Unmark for Deletion..."),
+    itsUnMarkAct=itsMenu->addAction(i18n("Unmark for Deletion"),
                                     this, SLOT(unmark()));
-    itsMarkAct=itsMenu->addAction(KIcon("edit-delete"), i18n("Mark for Deletion..."),
+    itsMarkAct=itsMenu->addAction(KIcon("edit-delete"), i18n("Mark for Deletion"),
                                   this, SLOT(mark()));
 
     connect(this, SIGNAL(itemSelectionChanged()), SLOT(selectionChanged()));
@@ -604,7 +626,7 @@ void CFontFileListView::openViewer()
     QSet<QString>            files;
 
     foreach(item, items)
-        if(item->parent()) // Then its a file, not font name :-)
+        if(item->parent()) // Then it is a file, not font name :-)
             files.insert(item->text(0));
 
     if(files.count() &&
