@@ -702,7 +702,7 @@ QImage CFcEngine::draw(const KUrl &url, int w, int h, const QColor &txt, const Q
 
         //
         // We allow kio_thumbnail to cache our thumbs. Normal is 128x128, and large is 256x256
-        // ...if kio_thumbnail asks us for a bigger size, then its probably the file info dialog, in
+        // ...if kio_thumbnail asks us for a bigger size, then it is probably the file info dialog, in
         // which case treat it as a normal preview...
         if(thumb && (h>256 || w!=h))
             thumb=false;
@@ -1111,10 +1111,10 @@ bool CFcEngine::parseUrl(const KUrl &url, int faceNo)
     }
     else if(url.isLocalFile() || url.protocol().isEmpty())
     {
-        // Now lets see if its from the thumbnail job! if so, then file should contain either:
+        // Now lets see if it is from the thumbnail job! if so, then file should contain either:
         //    a. fonts:/ Url
         //    b. FontName followed by style info
-        QFile file(url.path());
+        QFile file(url.toLocalFile());
         bool  isThumbnailUrl=false;
 
         if(file.size()<2048 && file.open(QIODevice::ReadOnly)) // Data should be less than 2k, and fonts usually above!
@@ -1271,12 +1271,12 @@ XftFont * CFcEngine::queryFont()
 
     XftFont *f=getFont(constQuerySize);
 
-    if(f && !isCorrect(f, true))                                                           
+    if(f && !isCorrect(f, true))
         closeFont(f);
 
     if(itsInstalled && !f)
     {
-        // Perhaps its a newly installed font? If so try re-initialising fontconfig...
+        // Perhaps it is a newly installed font? If so try re-initialising fontconfig...
         theirFcDirty=true;
         reinit();
 
