@@ -174,7 +174,7 @@ void getAssociatedFiles(const QString &path, QStringList &files, bool afmAndPfm)
 
     if(-1==dotPos) // Hmm, no extension - check anyway...
         check=true;
-    else           // Cool, got an extension - see if its a Type1 font...
+    else           // Cool, got an extension - see if it is a Type1 font...
     {
         ext=ext.mid(dotPos+1);
         check=0==ext.compare("pfa", Qt::CaseInsensitive) ||
@@ -183,8 +183,8 @@ void getAssociatedFiles(const QString &path, QStringList &files, bool afmAndPfm)
 
     if(check)
     {
-        const char *afm[]={"afm", "AFM", "Afm", NULL},
-                   *pfm[]={"pfm", "PFM", "Pfm", NULL};
+        static const char * const afm[]={"afm", "AFM", "Afm", NULL};
+        static const char * const pfm[]={"pfm", "PFM", "Pfm", NULL};
         bool       gotAfm(false);
         int        e;
 
