@@ -218,11 +218,13 @@ static void applyQtSettings( KConfigGroup kglobals, QSettings& settings )
     settings.setValue("/qt/KDE/kdeAddedLibraryPaths", kdeAdded);
     settings.setValue(libPathKey, libraryPath.join(QLatin1String(":")));
 
+#if (QT_VERSION < QT_VERSION_CHECK(4, 5, 0))    //Qt 4.5 will read kde config files if running on kde
   /* export widget style */
   kglobals.changeGroup("General");
   QString style = kglobals.readEntry("widgetStyle", KStyle::defaultStyle());
   if (!style.isEmpty())
     settings.setValue("/qt/style", style);
+#endif
 
   /* export font settings */
   settings.setValue("/qt/font", KGlobalSettings::generalFont().toString());
@@ -312,8 +314,8 @@ static void createGtkrc( bool exportColors, const QPalette& cg, int version )
             "# created by KDE, %1\n"
             "#\n"
             "# If you do not want KDE to override your GTK settings, select\n"
-            "# Appearance & Themes -> Colors in the Control Center and disable the checkbox\n"
-            "# \"Apply colors to non-KDE applications\"\n"
+            "# Appearance -> Colors in the Control Center and disable the checkbox\n"
+            "# \"Apply colors to non-KDE4 applications\"\n"
             "#\n"
             "#\n", QDateTime::currentDateTime().toString());
 
@@ -433,8 +435,8 @@ void runRdb( uint flags )
 
     QStringList list;
 
-    QStringList adPaths = KGlobal::dirs()->findDirs("appdefaults", "");
-    for (QStringList::ConstIterator it = adPaths.begin(); it != adPaths.end(); ++it) {
+    const QStringList adPaths = KGlobal::dirs()->findDirs("appdefaults", "");
+    for (QStringList::ConstIterator it = adPaths.constBegin(); it != adPaths.constEnd(); ++it) {
       QDir dSys( *it );
 
       if ( dSys.exists() ) {
@@ -445,7 +447,7 @@ void runRdb( uint flags )
       }
     }
 
-    for (QStringList::ConstIterator it = list.begin(); it != list.end(); ++it)
+    for (QStringList::ConstIterator it = list.constBegin(); it != list.constEnd(); ++it)
       copyFile(tmpFile, KStandardDirs::locate("appdefaults", *it ), true);
   }
 
