@@ -35,30 +35,25 @@
 #include <kstringhandler.h>
 #include <kmacroexpander.h>
 #include <kglobal.h>
+#include <kmimetypetrader.h>
+#include <kmimetype.h>
 
+#include "klippersettings.h"
 #include "urlgrabber.h"
 
 // TODO:
 // - script-interface?
 
-URLGrabber::URLGrabber(const KSharedConfigPtr &config)
- : m_config( config )
+URLGrabber::URLGrabber()
 {
-    if(!m_config) {
-        m_config = KGlobal::config();
-    }
-    myCurrentAction = 0L;
-    myMenu = 0L;
-    myPopupKillTimeout = 8;
+    m_myCurrentAction = 0L;
+    m_myMenu = 0L;
+    m_myPopupKillTimeout = 8;
     m_trimmed = true;
 
-    myActions = new ActionList();
-
-    readConfiguration( m_config.data() );
-
-    myPopupKillTimer = new QTimer( this );
-    myPopupKillTimer->setSingleShot( true );
-    connect( myPopupKillTimer, SIGNAL( timeout() ),
+    m_myPopupKillTimer = new QTimer( this );
+    m_myPopupKillTimer->setSingleShot( true );
+    connect( m_myPopupKillTimer, SIGNAL( timeout() ),
              SLOT( slotKillPopupMenu() ));
 
     // testing
@@ -67,27 +62,25 @@ URLGrabber::URLGrabber(const KSharedConfigPtr &config)
     action = new ClipAction( "^http:\\/\\/", "Web-URL" );
     action->addCommand("kfmclient exec %s", "Open with Konqi", true);
     action->addCommand("netscape -no-about-splash -remote \"openURL(%s, new-window)\"", "Open with Netscape", true);
-    myActions->append( action );
+    m_myActions->append( action );
 
     action = new ClipAction( "^mailto:", "Mail-URL" );
     action->addCommand("kmail --composer %s", "Launch kmail", true);
-    myActions->append( action );
+    m_myActions->append( action );
 
     action = new ClipAction( "^\\/.+\\.jpg$", "Jpeg-Image" );
     action->addCommand("kuickshow %s", "Launch KuickShow", true);
     action->addCommand("kview %s", "Launch KView", true);
-    myActions->append( action );
+    m_myActions->append( action );
     */
 }
 
 
 URLGrabber::~URLGrabber()
 {
-    delete myMenu;
-    ActionListIterator it( *myActions );
-    while (it.hasNext())
-        delete it.next();
-    delete myActions;
+    qDeleteAll(m_myActions);
+    m_myActions.clear();
+    delete m_myMenu;
 }
 
 //
@@ -97,103 +90,124 @@ URLGrabber::~URLGrabber()
 void URLGrabber::invokeAction( const QString& clip )
 {
     if ( !clip.isEmpty() )
-        myClipData = clip;
+        m_myClipData = clip;
     if ( m_trimmed )
-        myClipData = myClipData.trimmed();
+        m_myClipData = m_myClipData.trimmed();
 
     actionMenu( false );
 }
 
 
-void URLGrabber::setActionList( ActionList *list )
+void URLGrabber::setActionList( const ActionList& list )
 {
-    ActionListIterator it( *myActions );
-    while (it.hasNext())
-        delete it.next();
-    delete myActions;
-    myActions = list;
+    qDeleteAll(m_myActions);
+    m_myActions.clear();
+    m_myActions = list;
 }
 
 
 const ActionList& URLGrabber::matchingActions( const QString& clipData )
 {
-    myMatches.clear();
-    ClipAction *action = 0L;
+    m_myMatches.clear();
 
-    ActionListIterator it( *myActions );
-    while (it.hasNext()) {
-        action = it.next();
-        if ( action->matches( clipData ) )
-            myMatches.append( action );
+    // try to figure out if clipData contains a filename
+    KMimeType::Ptr mimetype = KMimeType::findByUrl( KUrl( clipData ), 0,
+                                                    false,
+                                                    true /*fast mode*/ );
+
+    // let's see if we found some reasonable mimetype.
+    // If we do we'll populate menu with actions for apps
+    // that can handle that mimetype
+
+    // first: if clipboard contents starts with http, let's assume it's "text/html".
+    // That is even if we've url like "http://www.kde.org/somescript.pl", we'll
+    // still treat that as html page, because determining a mimetype using kio
+    // might take a long time, and i want this function to be quick!
+    if ( ( clipData.startsWith( "http://" ) || clipData.startsWith( "https://"))
+         && mimetype->name() != "text/html" )
+    {
+        // use a fake path to create a mimetype that corresponds to "text/html"
+        mimetype = KMimeType::findByPath( "/tmp/klipper.html", 0, true /*fast mode*/ );
     }
 
-    return myMatches;
+    if ( mimetype->name() != "application/octet-stream" ) {
+        ClipAction* action = new ClipAction( QString(), mimetype->comment() );
+        KService::List lst = KMimeTypeTrader::self()->query( mimetype->name(), "Application" );
+        foreach( const KService::Ptr &service, lst ) {
+            action->addCommand( service->exec(), service->name(), true, service->icon() );
+        }
+        if ( !lst.isEmpty() )
+            m_myMatches.append( action );
+    }
+
+    // now look for matches in custom user actions
+    foreach (ClipAction* action, m_myActions) {
+        if ( action->matches( clipData ) )
+            m_myMatches.append( action );
+    }
+
+    return m_myMatches;
 }
 
 
 bool URLGrabber::checkNewData( const QString& clipData )
 {
     // kDebug() << "** checking new data: " << clipData;
-    myClipData = clipData;
+    m_myClipData = clipData;
     if ( m_trimmed )
-        myClipData = myClipData.trimmed();
+        m_myClipData = m_myClipData.trimmed();
 
-    if ( myActions->isEmpty() )
+    if ( m_myActions.isEmpty() )
         return false;
 
-    actionMenu( true ); // also creates myMatches
+    actionMenu( true ); // also creates m_myMatches
 
-    return ( !myMatches.isEmpty() &&
-             (!m_config->group("General").readEntry("Put Matching URLs in history", true))); //XXX i am not sure this entry exists anymore
+    return !m_myMatches.isEmpty();
 }
 
 
 void URLGrabber::actionMenu( bool wm_class_check )
 {
-    if ( myClipData.isEmpty() )
+    if ( m_myClipData.isEmpty() )
         return;
 
-    ActionListIterator it( matchingActions( myClipData ) );
-    ClipAction *action = 0L;
-    ClipCommand *command = 0L;
+    ActionList matchingActionsList = matchingActions( m_myClipData );
 
-    if (it.hasNext()) {
+    if (!matchingActionsList.isEmpty()) {
         // don't react on konqi's/netscape's urls...
         if ( wm_class_check && isAvoidedWindow() )
             return;
 
-        QString item;
-        myCommandMapper.clear();
+        m_myCommandMapper.clear();
 
-        myPopupKillTimer->stop();
+        m_myPopupKillTimer->stop();
 
-        myMenu = new KMenu;
+        m_myMenu = new KMenu;
 
-        connect(myMenu, SIGNAL(triggered(QAction*)), SLOT(slotItemSelected(QAction*)));
+        connect(m_myMenu, SIGNAL(triggered(QAction*)), SLOT(slotItemSelected(QAction*)));
 
-        while (it.hasNext()) {
-            action = it.next();
-            QListIterator<ClipCommand*> it2( action->commands() );
-            if ( it2.hasNext() )
-                myMenu->addTitle(KIcon( "klipper" ), action->description() +
-                                 i18n(" - Actions For: ") +
-                                 KStringHandler::csqueeze(myClipData, 45));
-            while (it2.hasNext()) {
-                command = it2.next();
-                item = command->description;
+        foreach (ClipAction* clipAct, matchingActionsList) {
+            m_myMenu->addTitle(KIcon( "klipper" ),
+                               i18n("%1 - Actions For: %2", clipAct->description(), KStringHandler::csqueeze(m_myClipData, 45)));
+            QList<ClipCommand> cmdList = clipAct->commands();
+            int listSize = cmdList.count();
+            for (int i=0; i<listSize;++i) {
+                ClipCommand command = cmdList.at(i);
+
+                QString item = command.description;
                 if ( item.isEmpty() )
-                    item = command->command;
+                    item = command.command;
 
                 QString id = QUuid::createUuid().toString();
                 QAction * action = new QAction(this);
                 action->setData(id);
                 action->setText(item);
 
-                if (!command->pixmap.isEmpty())
-                    action->setIcon(KIcon(command->pixmap));
+                if (!command.pixmap.isEmpty())
+                    action->setIcon(KIcon(command.pixmap));
 
-                myCommandMapper.insert(id, command);
-                myMenu->addAction(action);
+                m_myCommandMapper.insert(id, qMakePair(clipAct,i));
+                m_myMenu->addAction(action);
             }
         }
 
@@ -201,33 +215,33 @@ void URLGrabber::actionMenu( bool wm_class_check )
         // explicit Ctrl-Alt-R
         if ( wm_class_check )
         {
-            myMenu->addSeparator();
+            m_myMenu->addSeparator();
             QAction *disableAction = new QAction(i18n("Disable This Popup"), this);
             connect(disableAction, SIGNAL(triggered()), SIGNAL(sigDisablePopup()));
-            myMenu->addAction(disableAction);
+            m_myMenu->addAction(disableAction);
         }
-        myMenu->addSeparator();
+        m_myMenu->addSeparator();
         // add an edit-possibility
         QAction *editAction = new QAction(KIcon("document-properties"), i18n("&Edit Contents..."), this);
         connect(editAction, SIGNAL(triggered()), SLOT(editData()));
-        myMenu->addAction(editAction);
+        m_myMenu->addAction(editAction);
 
         QAction *cancelAction = new QAction(KIcon("dialog-cancel"), i18n("&Cancel"), this);
-        connect(cancelAction, SIGNAL(triggered()), myMenu, SLOT(hide()));
-        myMenu->addAction(cancelAction);
+        connect(cancelAction, SIGNAL(triggered()), m_myMenu, SLOT(hide()));
+        m_myMenu->addAction(cancelAction);
 
-        if ( myPopupKillTimeout > 0 )
-            myPopupKillTimer->start( 1000 * myPopupKillTimeout );
+        if ( m_myPopupKillTimeout > 0 )
+            m_myPopupKillTimer->start( 1000 * m_myPopupKillTimeout );
 
-        emit sigPopup( myMenu );
+        emit sigPopup( m_myMenu );
     }
 }
 
 
 void URLGrabber::slotItemSelected(QAction *action)
 {
-    if (myMenu)
-        myMenu->hide(); // deleted by the timer or the next action
+    if (m_myMenu)
+        m_myMenu->hide(); // deleted by the timer or the next action
 
     QString id = action->data().toString();
 
@@ -236,36 +250,42 @@ void URLGrabber::slotItemSelected(QAction *action)
         return;
     }
 
-    QHash<QString, ClipCommand*>::iterator i = myCommandMapper.find(id);
-    ClipCommand *command = i.value();
+    // first is action ptr, second is command index
+    QPair<ClipAction*, int> actionCommand = m_myCommandMapper.value(id);
 
-    if (command)
-        execute(command);
+    if (actionCommand.first)
+        execute(actionCommand.first, actionCommand.second);
     else
         kDebug() << "Klipper: cannot find associated action";
 }
 
 
-void URLGrabber::execute( const struct ClipCommand *command ) const
+void URLGrabber::execute( const ClipAction *action, int cmdIdx ) const
 {
-    if ( command->isEnabled ) {
+    if (!action) {
+        kDebug() << "Action object is null";
+        return;
+    }
+
+    ClipCommand command = action->command(cmdIdx);
+
+    if ( command.isEnabled ) {
         QHash<QChar,QString> map;
-        map.insert( 's', myClipData );
-        // commands executed should always have a parent,
-        // but a simple check won't hurt...
-        if ( command->parent )
-        {
-            const QStringList matches = command->parent->regExpMatches();
-            // support only %0 and the first 9 matches...
-            const int numMatches = qMin(10, matches.count());
-            for ( int i = 0; i < numMatches; ++i )
-                map.insert( QChar( '0' + i ), matches.at( i ) );
-        }
-        else
-        {
-            kDebug() << "No parent for" << command->description << "(" << command->command << ")";
-        }
-        QString cmdLine = KMacroExpander::expandMacrosShellQuote( command->command, map );
+        map.insert( 's', m_myClipData );
+
+        // support %u, %U (indicates url param(s)) and %f, %F (file param(s))
+        map.insert( 'u', m_myClipData );
+        map.insert( 'U', m_myClipData );
+        map.insert( 'f', m_myClipData );
+        map.insert( 'F', m_myClipData );
+
+        const QStringList matches = action->regExpMatches();
+        // support only %0 and the first 9 matches...
+        const int numMatches = qMin(10, matches.count());
+        for ( int i = 0; i < numMatches; ++i )
+            map.insert( QChar( '0' + i ), matches.at( i ) );
+
+        QString cmdLine = KMacroExpander::expandMacrosShellQuote( command.command, map );
 
         if ( cmdLine.isEmpty() )
             return;
@@ -280,70 +300,64 @@ void URLGrabber::execute( const struct ClipCommand *command ) const
 
 void URLGrabber::editData()
 {
-    myPopupKillTimer->stop();
+    m_myPopupKillTimer->stop();
     KDialog *dlg = new KDialog( 0 );
     dlg->setModal( true );
     dlg->setCaption( i18n("Edit Contents") );
     dlg->setButtons( KDialog::Ok | KDialog::Cancel );
 
     KTextEdit *edit = new KTextEdit( dlg );
-    edit->setText( myClipData );
+    edit->setText( m_myClipData );
     edit->setFocus();
     edit->setMinimumSize( 300, 40 );
     dlg->setMainWidget( edit );
     dlg->adjustSize();
 
     if ( dlg->exec() == KDialog::Accepted ) {
-        myClipData = edit->toPlainText();
+        m_myClipData = edit->toPlainText();
         QTimer::singleShot( 0, this, SLOT( slotActionMenu() ) );
     }
     else
     {
-        myMenu->deleteLater();
-        myMenu = 0;
+        m_myMenu->deleteLater();
+        m_myMenu = 0;
     }
     delete dlg;
 }
 
 
-void URLGrabber::readConfiguration( KConfig *kc )
+void URLGrabber::loadSettings()
 {
-    ActionListIterator it( *myActions );
-    while (it.hasNext())
-        delete it.next();
-    myActions->clear();
-    KConfigGroup cg(kc, "General");
+    m_trimmed = KlipperSettings::stripWhiteSpace();
+    m_myAvoidWindows = KlipperSettings::noActionsForWM_CLASS();
+    m_myPopupKillTimeout = KlipperSettings::timeoutForActionPopups();
+
+    qDeleteAll(m_myActions);
+    m_myActions.clear();
+
+    KConfigGroup cg(KGlobal::config(), "General");
     int num = cg.readEntry("Number of Actions", 0);
-    myAvoidWindows = cg.readEntry("No Actions for WM_CLASS",QStringList());
-    myPopupKillTimeout = cg.readEntry( "Timeout for Action popups (seconds)", 8 );
-    m_trimmed = cg.readEntry("Strip Whitespace before exec", true);
     QString group;
     for ( int i = 0; i < num; i++ ) {
         group = QString("Action_%1").arg( i );
-        myActions->append( new ClipAction( kc, group ) );
+        m_myActions.append( new ClipAction( KGlobal::config(), group ) );
     }
 }
 
-
-void URLGrabber::writeConfiguration( KConfig *kc )
+void URLGrabber::saveSettings() const
 {
-    KConfigGroup cg(kc, "General");
-    cg.writeEntry( "Number of Actions", myActions->count() );
-    cg.writeEntry( "Timeout for Action popups (seconds)", myPopupKillTimeout);
-    cg.writeEntry( "No Actions for WM_CLASS", myAvoidWindows );
-    cg.writeEntry( "Strip Whitespace before exec", m_trimmed );
-
-    ActionListIterator it( *myActions );
-    ClipAction *action;
+    KConfigGroup cg(KGlobal::config(), "General");
+    cg.writeEntry( "Number of Actions", m_myActions.count() );
 
     int i = 0;
     QString group;
-    while (it.hasNext()) {
-        action = it.next();
+    foreach (ClipAction* action, m_myActions) {
         group = QString("Action_%1").arg( i );
-        action->save( kc, group );
+        action->save( KGlobal::config(), group );
         ++i;
     }
+
+    KlipperSettings::setNoActionsForWM_CLASS(m_myAvoidWindows);
 }
 
 // find out whether the active window's WM_CLASS is in our avoid-list
@@ -381,7 +395,7 @@ bool URLGrabber::isAvoidedWindow() const
                             &unused, &data_ret ) == Success) {
         if ( type_ret == XA_STRING && format_ret == 8 && nitems_ret > 0 ) {
             wmClass = QString::fromUtf8( (const char *) data_ret );
-            ret = (myAvoidWindows.indexOf( wmClass ) != -1);
+            ret = (m_myAvoidWindows.indexOf( wmClass ) != -1);
         }
 
         XFree( data_ret );
@@ -393,72 +407,61 @@ bool URLGrabber::isAvoidedWindow() const
 
 void URLGrabber::slotKillPopupMenu()
 {
-    if ( myMenu && myMenu->isVisible() )
+    if ( m_myMenu && m_myMenu->isVisible() )
     {
-        if ( myMenu->geometry().contains( QCursor::pos() ) &&
-             myPopupKillTimeout > 0 )
+        if ( m_myMenu->geometry().contains( QCursor::pos() ) &&
+             m_myPopupKillTimeout > 0 )
         {
-            myPopupKillTimer->start( 1000 * myPopupKillTimeout );
+            m_myPopupKillTimer->start( 1000 * m_myPopupKillTimeout );
             return;
         }
     }
 
-    if ( myMenu ) {
-        myMenu->deleteLater();
-        myMenu = 0;
+    if ( m_myMenu ) {
+        m_myMenu->deleteLater();
+        m_myMenu = 0;
     }
 }
 
 ///////////////////////////////////////////////////////////////////////////
 ////////
 
-ClipCommand::ClipCommand(ClipAction *_parent, const QString &_command, const QString &_description,
+ClipCommand::ClipCommand(const QString &_command, const QString &_description,
                          bool _isEnabled, const QString &_icon)
-    : parent(_parent),
-      command(_command),
+    : command(_command),
       description(_description),
       isEnabled(_isEnabled)
 {
-    int len = command.indexOf(" ");
-    if (len == -1)
-        len = command.length();
 
     if (!_icon.isEmpty())
         pixmap = _icon;
     else
     {
-    KService::Ptr service= KService::serviceByDesktopName(command.left(len));
-    if (service)
-        pixmap = service->icon();
-    else
-        pixmap.clear();
+        // try to find suitable icon
+        QString appName = command.section( ' ', 0, 0 );
+        if ( !appName.isEmpty() )
+        {
+            QPixmap iconPix = KIconLoader::global()->loadIcon(
+                                         appName, KIconLoader::Small, 0,
+                                         KIconLoader::DefaultState,
+                                         QStringList(), 0, true /* canReturnNull */ );
+            if ( !iconPix.isNull() )
+                pixmap = appName;
+            else
+                pixmap.clear();
+        }
     }
 }
 
 
 ClipAction::ClipAction( const QString& regExp, const QString& description )
-    : myRegExp( regExp ), myDescription( description )
+    : m_myRegExp( regExp ), m_myDescription( description )
 {
 }
 
-
-ClipAction::ClipAction( const ClipAction& action )
-{
-    myRegExp      = action.myRegExp;
-    myDescription = action.myDescription;
-
-    ClipCommand *command = 0L;
-    QListIterator<ClipCommand*> it( myCommands );
-    while (it.hasNext()) {
-        command = it.next();
-        addCommand(command->command, command->description, command->isEnabled);
-    }
-}
-
-
-ClipAction::ClipAction( KConfig *kc, const QString& group )
-    : myRegExp( kc->group(group).readEntry("Regexp") ),
-      myDescription (kc->group(group).readEntry("Description") )
+ClipAction::ClipAction( KSharedConfigPtr kc, const QString& group )
+    : m_myRegExp( kc->group(group).readEntry("Regexp") ),
+      m_myDescription (kc->group(group).readEntry("Description") )
 {
     KConfigGroup cg(kc, group);
 
@@ -479,7 +482,7 @@ ClipAction::ClipAction( KConfig *kc, const QString& group )
 
 ClipAction::~ClipAction()
 {
-    qDeleteAll(myCommands);
+    m_myCommands.clear();
 }
 
 
@@ -489,33 +492,38 @@ void ClipAction::addCommand( const QString& command,
     if ( command.isEmpty() )
         return;
 
-    struct ClipCommand *cmd = new ClipCommand( this, command, description, enabled, icon );
-    //    cmd->id = myCommands.count(); // superfluous, I think...
-    myCommands.append( cmd );
+    m_myCommands.append( ClipCommand(command, description, enabled, icon) );
+}
+
+void ClipAction::replaceCommand( int idx, const ClipCommand& cmd )
+{
+    if ( idx < 0 || idx >= m_myCommands.count() ) {
+        kDebug() << "wrong command index given";
+        return;
+    }
+
+    m_myCommands[idx] = cmd;
 }
 
 
 // precondition: we're in the correct action's group of the KConfig object
-void ClipAction::save( KConfig *kc, const QString& group ) const
+void ClipAction::save( KSharedConfigPtr kc, const QString& group ) const
 {
     KConfigGroup cg(kc, group);
     cg.writeEntry( "Description", description() );
     cg.writeEntry( "Regexp", regExp() );
-    cg.writeEntry( "Number of commands", myCommands.count() );
-
-    struct ClipCommand *cmd;
-    QListIterator<struct ClipCommand*> it( myCommands );
+    cg.writeEntry( "Number of commands", m_myCommands.count() );
 
+    int i=0;
     // now iterate over all commands of this action
-    int i = 0;
-    while (it.hasNext()) {
-        cmd = it.next();
+    foreach (const ClipCommand& cmd, m_myCommands) {
         QString _group = group + "/Command_%1";
         KConfigGroup cg(kc, _group.arg(i));
 
-        cg.writePathEntry( "Commandline", cmd->command );
-        cg.writeEntry( "Description", cmd->description );
-        cg.writeEntry( "Enabled", cmd->isEnabled );
+        cg.writePathEntry( "Commandline", cmd.command );
+        cg.writeEntry( "Description", cmd.description );
+        cg.writeEntry( "Enabled", cmd.isEnabled );
+        cg.writeEntry( "Icon", cmd.pixmap );
 
         ++i;
     }
