@@ -20,8 +20,8 @@
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/
 
-
 #include "resultscene.h"
+
 #include <QtCore/QDebug>
 #include <QtGui/QKeyEvent>
 #include <QtCore/QMutexLocker>
@@ -33,22 +33,19 @@
 #include <QtGui/QGraphicsProxyWidget>
 
 #include <KDE/KDebug>
+#include <KDE/KIconLoader>
 #include <KDE/KLineEdit>
 
 #include <Plasma/AbstractRunner>
-#include <Plasma/FrameSvg>
 #include <Plasma/RunnerManager>
 
 #include "resultitem.h"
+#include "selectionbar.h"
 
-ResultScene::ResultScene(Plasma::RunnerManager *manager, QWidget * focusBase, QObject *parent)
+ResultScene::ResultScene(Plasma::RunnerManager *manager, QWidget *focusBase, QObject *parent)
     : QGraphicsScene(parent),
       m_runnerManager(manager),
-      m_cIndex(0),
-      m_rowStride(0),
-      m_pageStride(0),
-      m_pageCount(0),
-      m_currentPage(0),
+      m_currentIndex(0),
       m_focusBase(focusBase)
 {
     setItemIndexMethod(NoIndex);
@@ -56,92 +53,105 @@ ResultScene::ResultScene(Plasma::RunnerManager *manager, QWidget * focusBase, QO
     connect(m_runnerManager, SIGNAL(matchesChanged(const QList<Plasma::QueryMatch>&)),
             this, SLOT(setQueryMatches(const QList<Plasma::QueryMatch>&)));
 
-    m_resizeTimer.setSingleShot(true);
-    connect(&m_resizeTimer, SIGNAL(timeout()), this, SLOT(layoutIcons()));
-
     m_clearTimer.setSingleShot(true);
     connect(&m_clearTimer, SIGNAL(timeout()), this, SLOT(clearMatches()));
 
-    m_frame = new Plasma::FrameSvg(this);
+    m_hoverTimer.setSingleShot(true);
+    connect(&m_hoverTimer, SIGNAL(timeout()), this, SLOT(initItemsHoverEvents()));
 
-    {
-        // lock because setImagePath uses KSycoca
-        QMutexLocker lock(Plasma::AbstractRunner::bigLock());
-        m_frame->setImagePath("widgets/viewitem");
-    }
+    m_selectionBar = new SelectionBar(0);
+    addItem(m_selectionBar);
 
-    m_frame->setCacheAllRenderedFrames(true);
-    m_frame->setElementPrefix("normal");
+    connect(m_selectionBar, SIGNAL(ensureVisibility(QGraphicsItem *)), this, SIGNAL(ensureVisibility(QGraphicsItem *)));
+
+    m_selectionBar->hide();
+    updateItemMargins();
+
+    connect(m_selectionBar, SIGNAL(graphicsChanged()), this, SLOT(updateItemMargins()));
     //QColor bg(255, 255, 255, 126);
     //setBackgroundBrush(bg);
 }
 
 ResultScene::~ResultScene()
 {
+    clearMatches();
+    delete m_selectionBar;
 }
 
 QSize ResultScene::minimumSizeHint() const
 {
     QFontMetrics fm(font());
-    return QSize(ResultItem::BOUNDING_WIDTH * 4 + 6, (ResultItem::BOUNDING_HEIGHT + fm.height()) * 2 + 6);
+    return QSize(KIconLoader::SizeMedium * 4, (fm.height() * 5) * 3);
 }
 
 void ResultScene::resize(int width, int height)
 {
-    // optimize
-    if (m_size.width() == width && m_size.height() == height) {
-        return;
-    }
+    bool resizeItems = width != sceneRect().width();
 
-    m_size = QSize(width, height);
-    m_rowStride = width / (ResultItem::BOUNDING_WIDTH);
-    m_pageStride = height / (ResultItem::BOUNDING_WIDTH) * m_rowStride;
-    setSceneRect(0.0, 0.0, (qreal)width, (qreal)height);
-    m_resizeTimer.start(150);
-}
-
-void ResultScene::layoutIcons()
-{
-    QListIterator<ResultItem *> it(m_items);
-
-    while (it.hasNext()) {
-        ResultItem *item = it.next();
-        item->setRowStride(m_rowStride);
+    if (resizeItems) {
+        foreach (ResultItem *item, m_items) {
+            item->calculateSize(width, height);
+        }
     }
+    setSceneRect(itemsBoundingRect());
 }
 
 void ResultScene::clearMatches()
 {
     foreach (ResultItem *item, m_items) {
-        item->remove();
+        item->deleteLater();
     }
 
     m_itemsById.clear();
     m_items.clear();
-    m_pageCount = 0;
-    setPage(0);
     emit matchCountChanged(0);
 }
 
 bool ResultScene::canMoveItemFocus() const
 {
     // We prevent a late query result from stealing the item focus from the user
-    // The item focus can be moved only if the focusBase widget (the khistorycombobox)
-    // has focus (i.e. the user is still typing or waiting) or if there is no item currently focused
+    // The item focus can be moved only if
+    // 1) there is no item currently focused
+    // 2) the currently focused item is not visible anymore
+    // 3) the focusBase widget (the khistorycombobox) has focus (i.e. the user is still typing or waiting) AND the currently focused item has not been hovered
 
-    return m_focusBase->hasFocus() || (!focusItem());
+    return !(focusItem()) || (!m_items.contains(static_cast<ResultItem*>(focusItem()))) || (m_focusBase->hasFocus() && !static_cast<ResultItem*>(focusItem())->mouseHovered()) ;
+
+}
+
+void ResultScene::setItemsAcceptHoverEvents(bool enable)
+{
+    foreach (QGraphicsItem* tmpItem, items()) {
+        tmpItem->setAcceptHoverEvents(enable);
+    }
+}
+
+void ResultScene::initItemsHoverEvents()
+{
+    setItemsAcceptHoverEvents(true);
 }
 
 void ResultScene::setQueryMatches(const QList<Plasma::QueryMatch> &m)
 {
-    // kDebug() << "============================" << endl << "matches retrieved: " << m.count();
+    //kDebug() << "============================" << endl << "matches retrieved: " << m.count();
+    /*
+    foreach (const Plasma::QueryMatch &match, m) {
+        kDebug() << "    " << match.id() << match.text();
+    }
+    */
+
     if (m.isEmpty()) {
-        //kDebug() << "clearing";
+        kDebug() << "clearing";
+        resize(width(), 0);
         emit itemHoverEnter(0);
         m_clearTimer.start(200);
         return;
     }
 
+    m_hoverTimer.stop();
+    setItemsAcceptHoverEvents(false);
+
+    //resize(width(), m.count() * ResultItem::BOUNDING_HEIGHT);
     m_clearTimer.stop();
     m_items.clear();
 
@@ -167,42 +177,59 @@ void ResultScene::setQueryMatches(const QList<Plasma::QueryMatch> &m)
     // delete the stragglers
     QMapIterator<QString, ResultItem *> it(m_itemsById);
     while (it.hasNext()) {
-        it.next().value()->remove();
+        it.next().value()->deleteLater();
     }
 
     // organize the remainders
-    int i = 0;
     m_itemsById.clear();
 
     // this will leave them in *reverse* order
     qSort(m_items.begin(), m_items.end(), ResultItem::compare);
 
-    m_pageCount = m.count();
-    m_pageCount = m_pageCount / m_pageStride + (m_pageCount % m_pageStride != 0 ? 1 : 0);
-    setPage(0);
-    //kDebug() << "gots us" << m_pageCount << "m_pageCount of items";
-
     emit matchCountChanged(m.count());
+    arrangeItems(0);
 
+    m_hoverTimer.start(200);
+
+}
+
+void ResultScene::arrangeItems(ResultItem *itemChanged)
+{
     QListIterator<ResultItem*> matchIt(m_items);
     QGraphicsWidget *tab = 0;
+    int y = 0;
+    int i = 0;
     while (matchIt.hasNext()) {
         ResultItem *item = matchIt.next();
         //kDebug()  << item->name() << item->id() << item->priority() << i;
-        QGraphicsWidget::setTabOrder(tab, item);
-        m_itemsById.insert(item->id(), item);
-        item->setIndex(i);
+
+        if (!itemChanged) {
+            // first time set up
+            QGraphicsWidget::setTabOrder(tab, item);
+            m_itemsById.insert(item->id(), item);
+            item->setIndex(i);
+        }
+
+        item->setPos(0, y);
+        item->show();
+        //kDebug() << item->pos();
+        y += item->geometry().height();
 
         // it is vital that focus is set *after* the index
-        if (i == 0 && canMoveItemFocus()) {
+        if (!itemChanged && i == 0 && canMoveItemFocus()) {
+            clearSelection();
             setFocusItem(item);
-            emit itemHoverEnter(item);
+            item->setSelected(true);
+            emit ensureVisibility(item);
         }
 
         ++i;
         tab = item;
     }
 
+    // Here we cannot use itemsBoundingRect().height() because old items will be deleted on the next event cycle
+    // However we use itemsBoundingRect().width() to take care of width changes.
+    setSceneRect(QRect(0,0,itemsBoundingRect().width(),y));
 }
 
 ResultItem* ResultScene::addQueryMatch(const Plasma::QueryMatch &match, bool useAnyId)
@@ -215,11 +242,12 @@ ResultItem* ResultScene::addQueryMatch(const Plasma::QueryMatch &match, bool use
         //kDebug() << "did not find for" << match.id();
         if (useAnyId) {
             //kDebug() << "creating for" << match.id();
-            item = new ResultItem(match, 0, m_frame);
+            item = new ResultItem(match, 0);
+            item->setContentsMargins(m_itemMarginLeft, m_itemMarginTop,
+                                     m_itemMarginRight, m_itemMarginBottom);
             addItem(item);
             item->hide();
-            int rowStride = sceneRect().width() / (ResultItem::BOUNDING_WIDTH);
-            item->setRowStride(rowStride);
+            connect(item, SIGNAL(sizeChanged(ResultItem*)), this, SLOT(arrangeItems(ResultItem*)));
             connect(item, SIGNAL(activated(ResultItem*)), this, SIGNAL(itemActivated(ResultItem*)));
             connect(item, SIGNAL(hoverEnter(ResultItem*)), this, SIGNAL(itemHoverEnter(ResultItem*)));
             connect(item, SIGNAL(hoverLeave(ResultItem*)), this, SIGNAL(itemHoverLeave(ResultItem*)));
@@ -237,6 +265,32 @@ ResultItem* ResultScene::addQueryMatch(const Plasma::QueryMatch &match, bool use
     return item;
 }
 
+
+void ResultScene::focusInEvent(QFocusEvent *focusEvent)
+{
+
+    // The default implementation of focusInEvent assumes that if the scene has no focus
+    // then it has no focused item; thus, when a scene gains focus, focusInEvent gives
+    // focus to the last focused item.
+    // In our case this assumption is not true, as an item can be focused before the scene,
+    // therefore we revert the behaviour by re-selecting the previously selected item
+
+    ResultItem *currentFocus = dynamic_cast<ResultItem*>(focusItem());
+
+    QGraphicsScene::focusInEvent(focusEvent);
+
+    switch (focusEvent->reason()) {
+    case Qt::TabFocusReason:
+    case Qt::BacktabFocusReason:
+        break;
+    default:
+        if (currentFocus) {
+            setFocusItem(currentFocus);
+        }
+        break;
+    }
+}
+
 void ResultScene::focusOutEvent(QFocusEvent *focusEvent)
 {
     QGraphicsScene::focusOutEvent(focusEvent);
@@ -248,87 +302,55 @@ void ResultScene::focusOutEvent(QFocusEvent *focusEvent)
 void ResultScene::keyPressEvent(QKeyEvent * keyEvent)
 {
     //kDebug() << "m_items (size): " << m_items.size() << "\n";
-    ResultItem *currentFocus = dynamic_cast<ResultItem*>(focusItem());
-    int m_cIndex = currentFocus ? currentFocus->index() : 0;
     switch (keyEvent->key()) {
-        case Qt::Key_Up:{
-            if (m_cIndex < m_rowStride) {
-                if (m_items.size() < m_rowStride) {
-                    // we have less than one row of items, so lets just move to the next item
-                    m_cIndex = (m_cIndex + 1) % m_items.size();
-                } else {
-                    m_cIndex = m_items.size() - (m_items.size() % m_rowStride) - 1 + (m_cIndex % m_items.size());
-                    if (m_cIndex >= m_items.size()) {
-                        // we should be on the bottom row, but there is nothing there; move up one row
-                        m_cIndex -= m_rowStride % m_items.size();
-                    }
-                }
-            } else {
-                m_cIndex = m_cIndex - m_rowStride;
-            }
-            break;
-        }
-
-        case Qt::Key_Down:{
-            if (m_cIndex + m_rowStride >= m_items.size()) {
-                // warp to the top
-                m_cIndex = (m_cIndex + 1) % m_rowStride % m_items.size();
-            } else {
-                // next row!
-                m_cIndex += m_rowStride;
-            }
-
-            break;
-        }
-
+        case Qt::Key_Up:
         case Qt::Key_Left:
-            m_cIndex = (m_cIndex == 0) ? m_items.size() - 1 : m_cIndex - 1;
-        break;
+            selectNextItem();
+            break;
 
+        case Qt::Key_Down:
         case Qt::Key_Right:
-            m_cIndex = (m_cIndex + 1) % m_items.size();
+            selectPreviousItem();
         break;
 
-        case Qt::Key_Return:
-            //TODO: run the item
-        case Qt::Key_Space:
         default:
             // pass the event to the item
             QGraphicsScene::keyPressEvent(keyEvent);
             return;
         break;
     }
+}
 
-    if (m_items[m_cIndex]->pos().y() + m_items[m_cIndex]->size().height() > sceneRect().y() + sceneRect().height()){
-        setPage(m_currentPage + 1);
-    }else if (m_items[m_cIndex]->pos().y() < sceneRect().y()){
-        setPage(m_currentPage - 1);
+void ResultScene::selectNextItem()
+{
+    ResultItem *currentFocus = dynamic_cast<ResultItem*>(focusItem());
+    int m_currentIndex = currentFocus ? currentFocus->index() : 0;
+
+    if (m_currentIndex > 0) {
+        --m_currentIndex;
+    } else {
+        m_currentIndex = m_items.size() - 1;
     }
 
-    // If we arrive here, it was due to an arrow button.
-    Q_ASSERT(m_cIndex  >= 0);
-    Q_ASSERT(m_cIndex < m_items.count());
-    //kDebug() << "m_cIndex: " << m_cIndex << "\n";
-    setFocusItem(m_items.at(m_cIndex));
+    setFocusItem(m_items.at(m_currentIndex));
+    clearSelection();
+    m_items.at(m_currentIndex)->setSelected(true);
 }
 
-void ResultScene::wheelEvent(QGraphicsSceneWheelEvent *event)
+void ResultScene::selectPreviousItem()
 {
-    if (event->delta() > 0) {
-        setPage(m_currentPage - 1);
-    }else{
-        setPage(m_currentPage + 1);
+    ResultItem *currentFocus = dynamic_cast<ResultItem*>(focusItem());
+    int m_currentIndex = currentFocus ? currentFocus->index() : 0;
+
+    ++m_currentIndex;
+
+    if (m_currentIndex >= m_items.size()) {
+        m_currentIndex = 0;
     }
-}
-
-void ResultScene::slotArrowResultItemPressed()
-{
-
-}
-
-void ResultScene::slotArrowResultItemReleased()
-{
 
+    setFocusItem(m_items.at(m_currentIndex));
+    clearSelection();
+    m_items.at(m_currentIndex)->setSelected(true);
 }
 
 bool ResultScene::launchQuery(const QString &term)
@@ -370,35 +392,22 @@ void ResultScene::run(ResultItem *item) const
     item->run(m_runnerManager);
 }
 
+/*
 Plasma::RunnerManager* ResultScene::manager() const
 {
     return m_runnerManager;
 }
-
-uint ResultScene::pageCount() const
+*/
+void ResultScene::updateItemMargins()
 {
-    return m_pageCount;
-}
+    m_selectionBar->getMargins(m_itemMarginLeft, m_itemMarginTop,
+                               m_itemMarginRight, m_itemMarginBottom);
 
-void ResultScene::nextPage()
-{
-    setPage(m_currentPage + 1);
-}
-
-void ResultScene::previousPage()
-{
-    setPage(m_currentPage - 1);
-}
-
-void ResultScene::setPage(uint index)
-{
-    if (index > m_pageCount || index == m_currentPage) {
-        return;
+    foreach (ResultItem *item, m_items) {
+        item->setContentsMargins(m_itemMarginLeft, m_itemMarginTop,
+                                m_itemMarginRight, m_itemMarginBottom);
     }
-
-    m_currentPage = index;
-    setSceneRect(0.0, ((m_currentPage * (m_pageStride / m_rowStride))) * ResultItem::BOUNDING_HEIGHT,
-                 width(), height());
 }
+
 #include "resultscene.moc"
 
