@@ -22,14 +22,16 @@
 #include <stdlib.h>
 #include <unistd.h>
 
-#include <QProcess>
+#include <QClipboard>
 #include <QObject>
+#include <QProcess>
 #include <QTimer>
 #include <QtDBus/QtDBus>
 #include <QLineEdit>
 
 #include <KAction>
 #include <KActionCollection>
+#include <KCrash>
 #include <KDialog>
 #include <KAuthorized>
 #include <KGlobalAccel>
@@ -38,85 +40,58 @@
 #include <KMessageBox>
 #include <KWindowSystem>
 
+#include <Plasma/RunnerManager>
+
+#ifdef Q_WS_X11
 #include "processui/ksysguardprocesslist.h"
+#endif
 
 #include "appadaptor.h"
 #include "kworkspace.h"
-#include "interfaceadaptor.h"
-#include "interface.h"
+#include "interfaces/default/interface.h"
+#include "interfaces/quicksand/qs_dialog.h"
+#ifdef Q_WS_X11
 #include "startupid.h"
+#endif
 #include "klaunchsettings.h"
 #include "krunnersettings.h"
 
+#ifdef Q_WS_X11
 #include <X11/extensions/Xrender.h>
+#endif
 
-Display* dpy = 0;
-Colormap colormap = 0;
-Visual *visual = 0;
 
-void checkComposite()
-{
-    // thanks to zack rusin and frederik for pointing me in the right direction
-    // for the following bits of X11 code
-    dpy = XOpenDisplay(0); // open default display
-    if (!dpy)
-    {
-        kError() << "Cannot connect to the X server";
-        return;
-    }
-
-    int screen = DefaultScreen(dpy);
-    int eventBase, errorBase;
-
-    if (XRenderQueryExtension(dpy, &eventBase, &errorBase))
-    {
-        int nvi;
-        XVisualInfo templ;
-        templ.screen  = screen;
-        templ.depth   = 32;
-        templ.c_class = TrueColor;
-        XVisualInfo *xvi = XGetVisualInfo(dpy, VisualScreenMask |
-                                                VisualDepthMask |
-                                                VisualClassMask,
-                                            &templ, &nvi);
-        for (int i = 0; i < nvi; ++i)
-        {
-            XRenderPictFormat *format = XRenderFindVisualFormat(dpy,
-                                                                xvi[i].visual);
-            if (format->type == PictTypeDirect && format->direct.alphaMask)
-            {
-                visual = xvi[i].visual;
-                colormap = XCreateColormap(dpy, RootWindow(dpy, screen),
-                                            visual, AllocNone);
-                break;
-            }
-        }
-
-    }
-}
 
 KRunnerApp* KRunnerApp::self()
 {
     if (!kapp) {
-        checkComposite();
-        return new KRunnerApp(dpy, visual ? Qt::HANDLE(visual) : 0, colormap ? Qt::HANDLE(colormap) : 0);
+        return new KRunnerApp();
     }
 
     return qobject_cast<KRunnerApp*>(kapp);
 }
 
-KRunnerApp::KRunnerApp(Display *display, Qt::HANDLE visual, Qt::HANDLE colormap)
-    : RestartingApplication(display, visual, colormap),
+KRunnerApp::KRunnerApp()
+    : KUniqueApplication(),
       m_interface(0),
       m_tasks(0),
       m_startupId(NULL)
 {
     initialize();
+    connect(this, SIGNAL(aboutToQuit()), this, SLOT(cleanUp()));
 }
 
 KRunnerApp::~KRunnerApp()
 {
+}
+
+void KRunnerApp::cleanUp()
+{
+    disconnect(KRunnerSettings::self(), SIGNAL(configChanged()), this, SLOT(reloadConfig()));
     delete m_interface;
+    m_interface = 0;
+    delete m_runnerManager;
+    m_runnerManager = 0;
 }
 
 void KRunnerApp::initialize()
@@ -124,9 +99,27 @@ void KRunnerApp::initialize()
     setWindowIcon(KIcon("system-run"));
 
     setQuitOnLastWindowClosed(false);
+    KCrash::setFlags(KCrash::AutoRestart);
     initializeStartupNotification();
-    m_interface = new Interface;
 
+    connect(KRunnerSettings::self(), SIGNAL(configChanged()), this, SLOT(reloadConfig()));
+
+    m_runnerManager = new Plasma::RunnerManager;
+
+    switch (KRunnerSettings::interface()) {
+        default:
+        case KRunnerSettings::EnumInterface::CommandOriented:
+            m_interface = new Interface(m_runnerManager);
+            break;
+        case KRunnerSettings::EnumInterface::TaskOriented:
+            m_interface = new QsDialog(m_runnerManager);
+            break;
+    }
+
+    new AppAdaptor(this);
+    QDBusConnection::sessionBus().registerObject( "/App", this );
+
+#ifdef Q_WS_X11
     //FIXME: if argb visuals enabled Qt will always set WM_CLASS as "qt-subapplication" no matter what
     //the application name is we set the proper XClassHint here, hopefully won't be necessary anymore when
     //qapplication will manage apps with argvisuals in a better way
@@ -134,79 +127,81 @@ void KRunnerApp::initialize()
     classHint.res_name = const_cast<char*>("krunner");
     classHint.res_class = const_cast<char*>("krunner");
     XSetClassHint(QX11Info::display(), m_interface->winId(), &classHint);
-    
+#endif
+
     // Global keys
-    m_actionCollection = new KActionCollection( m_interface );
+    m_actionCollection = new KActionCollection(this);
     KAction* a = 0;
 
     if ( KAuthorized::authorizeKAction( "run_command" ) ) {
-        a = m_actionCollection->addAction( I18N_NOOP("Run Command") );
-        a->setText( i18n( I18N_NOOP( "Run Command" ) ) );
+        a = m_actionCollection->addAction( "Run Command" );
+        a->setText( i18n( "Run Command" ) );
         a->setGlobalShortcut(KShortcut(Qt::ALT+Qt::Key_F2));
-        connect( a, SIGNAL(triggered(bool)), m_interface, SLOT(display()) );
+        connect( a, SIGNAL(triggered(bool)), SLOT(display()) );
 
-        a = m_actionCollection->addAction( I18N_NOOP("Run Command on clipboard contents") );
-        a->setText( i18n( I18N_NOOP( "Run Command on clipboard contents" ) ) );
+        a = m_actionCollection->addAction( "Run Command on clipboard contents" );
+        a->setText( i18n( "Run Command on clipboard contents" ) );
         a->setGlobalShortcut(KShortcut(Qt::ALT+Qt::SHIFT+Qt::Key_F2));
-        connect( a, SIGNAL(triggered(bool)), m_interface, SLOT(displayWithClipboardContents()) );
+        connect( a, SIGNAL(triggered(bool)), SLOT(displayWithClipboardContents()) );
     }
 
-    a = m_actionCollection->addAction( I18N_NOOP( "Show System Activity" ) );
-    a->setText( i18n( I18N_NOOP( "Show System Activity" ) ) );
+    a = m_actionCollection->addAction( "Show System Activity" );
+    a->setText( i18n( "Show System Activity" ) );
     a->setGlobalShortcut( KShortcut( Qt::CTRL+Qt::Key_Escape ) );
     connect( a, SIGNAL(triggered(bool)), SLOT(showTaskManager()) );
 
 /*
  * TODO: doesn't this belong in the window manager?
-    a = m_actionCollection->addAction( I18N_NOOP( "Show Window List") );
-    a->setText( i18n( I18N_NOOP( "Show Window List") ) );
+    a = m_actionCollection->addAction( "Show Window List" );
+    a->setText( i18n( "Show Window List" ) );
     a->setGlobalShortcut( KShortcut( Qt::ALT+Qt::Key_F5 ) );
     connect( a, SIGNAL(triggered(bool)), SLOT(slotShowWindowList()) );
 */
-    a = m_actionCollection->addAction( I18N_NOOP("Switch User") );
-    a->setText( i18n( I18N_NOOP("Switch User") ) );
+    a = m_actionCollection->addAction( "Switch User" );
+    a->setText( i18n("Switch User") );
     a->setGlobalShortcut( KShortcut( Qt::ALT+Qt::CTRL+Qt::Key_Insert ) );
-    connect(a, SIGNAL(triggered(bool)), m_interface, SLOT(switchUser()));
+    connect(a, SIGNAL(triggered(bool)), SLOT(switchUser()));
 
+#ifdef Q_WS_X11
     if ( KAuthorized::authorizeKAction( "lock_screen" ) ) {
-        a = m_actionCollection->addAction( I18N_NOOP( "Lock Session" ) );
-        a->setText( i18n( I18N_NOOP( "Lock Session" ) ) );
+        a = m_actionCollection->addAction( "Lock Session" );
+        a->setText( i18n( "Lock Session" ) );
         a->setGlobalShortcut( KShortcut( Qt::ALT+Qt::CTRL+Qt::Key_L ) );
         connect( a, SIGNAL(triggered(bool)), &m_saver, SLOT(Lock()) );
     }
+#endif
 
     if ( KAuthorized::authorizeKAction( "logout" ) ) {
-        a = m_actionCollection->addAction( I18N_NOOP("Log Out") );
-        a->setText( i18n(I18N_NOOP("Log Out")) );
+        a = m_actionCollection->addAction( "Log Out" );
+        a->setText( i18n("Log Out") );
         a->setGlobalShortcut(KShortcut(Qt::ALT+Qt::CTRL+Qt::Key_Delete));
         connect(a, SIGNAL(triggered(bool)), SLOT(logout()));
 
-        a = m_actionCollection->addAction( I18N_NOOP("Log Out Without Confirmation") );
-        a->setText( i18n(I18N_NOOP("Log Out Without Confirmation")) );
+        a = m_actionCollection->addAction( "Log Out Without Confirmation" );
+        a->setText( i18n("Log Out Without Confirmation") );
         a->setGlobalShortcut(KShortcut(Qt::ALT+Qt::CTRL+Qt::SHIFT+Qt::Key_Delete));
         connect(a, SIGNAL(triggered(bool)), SLOT(logoutWithoutConfirmation()));
 
-        a = m_actionCollection->addAction( I18N_NOOP("Halt Without Confirmation") );
-        a->setText( i18n(I18N_NOOP("Halt Without Confirmation")) );
+        a = m_actionCollection->addAction( "Halt Without Confirmation" );
+        a->setText( i18n("Halt Without Confirmation") );
         a->setGlobalShortcut(KShortcut(Qt::ALT+Qt::CTRL+Qt::SHIFT+Qt::Key_PageDown));
         connect(a, SIGNAL(triggered(bool)), SLOT(haltWithoutConfirmation()));
 
-        a = m_actionCollection->addAction( I18N_NOOP("Reboot Without Confirmation") );
-        a->setText( i18n(I18N_NOOP("Reboot Without Confirmation")) );
+        a = m_actionCollection->addAction( "Reboot Without Confirmation" );
+        a->setText( i18n("Reboot Without Confirmation") );
         a->setGlobalShortcut(KShortcut(Qt::ALT+Qt::CTRL+Qt::SHIFT+Qt::Key_PageUp));
         connect(a, SIGNAL(triggered(bool)), SLOT(rebootWithoutConfirmation()));
     }
 
     m_actionCollection->readSettings();
-
-    new AppAdaptor(this);
-    QDBusConnection::sessionBus().registerObject( "/App", this );
-} // end void KRunnerApp::initializeBindings
+    m_runnerManager->reloadConfiguration(); // pre-load the runners
+}
 
 void KRunnerApp::initializeStartupNotification()
 {
     // Startup notification
     KLaunchSettings::self()->readConfig();
+#ifdef Q_WS_X11
     if( !KLaunchSettings::busyCursor() ) {
         delete m_startupId;
         m_startupId = NULL;
@@ -217,17 +212,16 @@ void KRunnerApp::initializeStartupNotification()
 
         m_startupId->configure();
     }
+#endif
 }
 
-/*TODO: fixme - move to kwin
-void KRunnerApp::showWindowList()
-{
-     //KRootWm::self()->slotWindowList();
-}
-*/
-
 void KRunnerApp::showTaskManager()
 {
+    showTaskManagerWithFilter(QString());
+}
+void KRunnerApp::showTaskManagerWithFilter(const QString &filterText)
+{
+#ifndef Q_WS_WIN
     //kDebug(1204) << "Launching KSysGuard...";
     KSysGuardProcessList* w = NULL;
     if (!m_tasks) {
@@ -253,7 +247,7 @@ void KRunnerApp::showTaskManager()
         m_tasks->restoreDialogSize(cg);
         w->loadSettings(cg);
         // Since we default to forcing the window to be KeepAbove, if the user turns this off, remember this
-        bool keepAbove = KRunnerSettings::keepTaskDialogAbove();
+        const bool keepAbove = KRunnerSettings::keepTaskDialogAbove();
         if (keepAbove) {
             KWindowSystem::setState( m_tasks->winId(), NET::KeepAbove );
         }
@@ -268,12 +262,41 @@ void KRunnerApp::showTaskManager()
     KWindowSystem::forceActiveWindow(m_tasks->winId());
 
     if (w) {
+        w->filterLineEdit()->setText(filterText);
         w->filterLineEdit()->setFocus();
     }
+#endif
+}
+
+void KRunnerApp::display()
+{
+    m_interface->display();
+}
+
+void KRunnerApp::query(const QString &term)
+{
+    m_interface->display(term);
+}
+
+void KRunnerApp::displayWithClipboardContents()
+{
+   QString clipboardData = QApplication::clipboard()->text(QClipboard::Selection);
+   m_interface->display(clipboardData);
+}
+
+void KRunnerApp::switchUser()
+{
+    m_interface->switchUser();
+}
+
+void KRunnerApp::clearHistory()
+{
+    m_interface->clearHistory();
 }
 
 void KRunnerApp::taskDialogFinished()
 {
+#ifndef Q_WS_WIN  
     KConfigGroup cg = KGlobal::config()->group("TaskDialog");
     m_tasks->saveDialogSize(cg);
     KSysGuardProcessList *w = static_cast<KSysGuardProcessList *> (m_tasks->mainWidget());
@@ -288,11 +311,12 @@ void KRunnerApp::taskDialogFinished()
 
     m_tasks->deleteLater();
     m_tasks = 0;
+#endif
 }
 
 void KRunnerApp::logout()
 {
-    logout( KWorkSpace::ShutdownConfirmDefault,
+    logout( KWorkSpace::ShutdownConfirmYes,
             KWorkSpace::ShutdownTypeDefault );
 }
 
@@ -317,6 +341,7 @@ void KRunnerApp::rebootWithoutConfirmation()
 void KRunnerApp::logout( KWorkSpace::ShutdownConfirm confirm,
                        KWorkSpace::ShutdownType sdtype )
 {
+#ifndef Q_WS_WIN
     if ( !KWorkSpace::requestShutDown( confirm, sdtype ) ) {
         // TODO: should we show these errors in Interface?
         KMessageBox::error( 0, i18n("Could not log out properly.\nThe session manager cannot "
@@ -324,6 +349,7 @@ void KRunnerApp::logout( KWorkSpace::ShutdownConfirm confirm,
                                     "Ctrl+Alt+Backspace; note, however, that your current session "
                                     "will not be saved with a forced shutdown." ) );
     }
+#endif
 }
 
 int KRunnerApp::newInstance()
@@ -335,13 +361,37 @@ int KRunnerApp::newInstance()
         m_interface->display();
     }
 
-    return RestartingApplication::newInstance();
+    return KUniqueApplication::newInstance();
     //return 0;
 }
 
 bool KRunnerApp::hasCompositeManager() const
 {
-    return colormap && KWindowSystem::compositingActive();
+#ifdef Q_WS_X11
+    return KWindowSystem::compositingActive();
+#else
+    return false;
+#endif
+}
+
+void KRunnerApp::reloadConfig()
+{
+    //Prevent Interface destructor from triggering this method
+    disconnect(KRunnerSettings::self(), SIGNAL(configChanged()), this, SLOT(reloadConfig()));
+
+    const int interface = KRunnerSettings::interface();
+    if (qobject_cast<Interface*>(m_interface) &&
+        interface == KRunnerSettings::EnumInterface::TaskOriented) {
+        delete m_interface;
+        m_interface = new QsDialog(m_runnerManager);
+        m_interface->display();
+    } else if (interface == KRunnerSettings::EnumInterface::CommandOriented) {
+        delete m_interface;
+        m_interface = new Interface(m_runnerManager);
+        m_interface->display();
+    }
+
+    connect(KRunnerSettings::self(), SIGNAL(configChanged()), this, SLOT(reloadConfig()));
 }
 
 #include "krunnerapp.moc"
