@@ -9,7 +9,7 @@
 
 //krunner keeps running and checks user inactivity
 //when it should show screensaver (and maybe lock the session),
-//it starts krunner_lock, who does all the locking and who
+//it starts kscreenlocker, who does all the locking and who
 //actually starts the screensaver
 
 //It's done this way to prevent screen unlocking when krunner
@@ -25,6 +25,8 @@
 #include "autologout.h"
 #include "kscreensaversettings.h"
 
+#include <kephal/screens.h>
+
 #include <kdisplaymanager.h>
 
 #include <KStandardDirs>
@@ -121,7 +123,6 @@ int XSelectInput( Display* dpy, Window w, long e )
 }
 #endif
 
-const int TIMEOUT_CODE = 2; //from PasswordDlg
 
 //===========================================================================
 //
@@ -148,7 +149,7 @@ LockProcess::LockProcess(bool child, bool useBlankOnly)
     setupSignals();
 
     new LockProcessAdaptor(this);
-    QDBusConnection::sessionBus().registerService("org.kde.krunner_lock");
+    QDBusConnection::sessionBus().registerService("org.kde.screenlocker");
     QDBusConnection::sessionBus().registerObject("/LockProcess", this);
 
     kapp->installX11EventFilter(this);
@@ -176,7 +177,7 @@ LockProcess::LockProcess(bool child, bool useBlankOnly)
     mSuspendTimer.setSingleShot(true);
     connect(&mSuspendTimer, SIGNAL(timeout()), SLOT(suspend()));
 
-    QStringList dmopt =
+    const QStringList dmopt =
         QString::fromLatin1( ::getenv( "XDM_MANAGED" )).split(QChar(','), QString::SkipEmptyParts);
     for (QStringList::ConstIterator it = dmopt.constBegin(); it != dmopt.constEnd(); ++it)
         if ((*it).startsWith("method="))
@@ -236,7 +237,6 @@ void LockProcess::timerEvent(QTimerEvent *ev)
 {
     if (ev->timerId() == mAutoLogoutTimerId)
     {
-        killTimer(mAutoLogoutTimerId);
         AutoLogout autologout(this);
         execDialog(&autologout);
     }
@@ -365,10 +365,8 @@ void LockProcess::configure()
         mLockGrace = -1;
     }
 
-    if ( KScreenSaverSettings::autoLogout() ) {
-        mAutoLogoutTimeout = KScreenSaverSettings::autoLogoutTimeout();
-        mAutoLogoutTimerId = startTimer(mAutoLogoutTimeout * 1000); // in milliseconds
-    }
+    mAutoLogoutTimeout = KScreenSaverSettings::autoLogout() ?
+                         KScreenSaverSettings::autoLogoutTimeout() : 0;
 
 #ifdef HAVE_DPMS
     mDPMSDepend = KScreenSaverSettings::suspendWhenInvisible();
@@ -408,24 +406,25 @@ void LockProcess::readSaver()
         QString entryName = mSaver;
         if( entryName.endsWith( ".desktop" ))
             entryName = entryName.left( entryName.length() - 8 ); // strip it
-        KService::List offers = KServiceTypeTrader::self()->query( "ScreenSaver",
+        const KService::List offers = KServiceTypeTrader::self()->query( "ScreenSaver",
             "DesktopEntryName == '" + entryName.toLower() + '\'' );
-        if( offers.count() == 0 )
+        if( offers.isEmpty() )
         {
             kDebug(1204) << "Cannot find screesaver: " << mSaver;
             return;
         }
-        QString file = KStandardDirs::locate("services", offers.first()->entryPath());
+        const QString file = KStandardDirs::locate("services", offers.first()->entryPath());
 
-        bool opengl = KAuthorized::authorizeKAction("opengl_screensavers");
-        bool manipulatescreen = KAuthorized::authorizeKAction("manipulatescreen_screensavers");
+        const bool opengl = KAuthorized::authorizeKAction("opengl_screensavers");
+        const bool manipulatescreen = KAuthorized::authorizeKAction("manipulatescreen_screensavers");
         KDesktopFile config( file );
         KConfigGroup desktopGroup = config.desktopGroup();
         if (!desktopGroup.readEntry("X-KDE-Type").toUtf8().isEmpty())
         {
-            QString saverType = desktopGroup.readEntry("X-KDE-Type").toUtf8();
-            QStringList saverTypes = saverType.split( ";");
-            for (int i = 0; i < saverTypes.count(); i++)
+            const QString saverType = desktopGroup.readEntry("X-KDE-Type").toUtf8();
+            const QStringList saverTypes = saverType.split( ";");
+            const int nbSaverTypes = saverTypes.count();
+            for (int i = 0; i < nbSaverTypes ; ++i)
             {
                 if ((saverTypes[i] == "ManipulateScreen") && !manipulatescreen)
                 {
@@ -467,7 +466,7 @@ void LockProcess::createSaverWindow()
 // this code is (partially) duplicated in kdebase/workspace/kcontrol/screensaver
     if( mOpenGLVisual )
     {
-        static int attribs[][ 15 ] =
+        static const int attribs[][ 15 ] =
         {
         #define R GLX_RED_SIZE
         #define G GLX_GREEN_SIZE
@@ -492,7 +491,7 @@ void LockProcess::createSaverWindow()
              i < sizeof( attribs ) / sizeof( attribs[ 0 ] );
              ++i )
         {
-            if( XVisualInfo* info = glXChooseVisual( x11Info().display(), x11Info().screen(), attribs[ i ] ))
+            if( XVisualInfo* info = glXChooseVisual( x11Info().display(), x11Info().screen(), const_cast<int*>(attribs[ i ]) ))
             {
                 visual = info->visual;
                 depth = info->depth;
@@ -622,7 +621,7 @@ void LockProcess::setVRoot(Window win, Window vr)
     if (gVRoot)
         removeVRoot(gVRoot);
 
-	QX11Info info;
+        QX11Info info;
     unsigned long rw = RootWindowOfScreen(ScreenOfDisplay(QX11Info::display(), info.screen()));
     unsigned long vroot_data[1] = { vr };
 
@@ -666,7 +665,7 @@ bool LockProcess::grabKeyboard()
 }
 
 #define GRABEVENTS ButtonPressMask | ButtonReleaseMask | PointerMotionMask | \
-		   EnterWindowMask | LeaveWindowMask
+                   EnterWindowMask | LeaveWindowMask
 
 //---------------------------------------------------------------------------
 //
@@ -743,6 +742,8 @@ bool LockProcess::startSaver()
         QSocketNotifier *notifier = new QSocketNotifier(mParent, QSocketNotifier::Read, this);
         connect(notifier, SIGNAL( activated (int)), SLOT( quitSaver()));
     }
+    if (mAutoLogoutTimeout && !mSetupMode)
+        mAutoLogoutTimerId = startTimer(mAutoLogoutTimeout * 1000); // in milliseconds
     createSaverWindow();
     move(0, 0);
     show();
@@ -913,8 +914,8 @@ void LockProcess::stopHack()
 //
 void LockProcess::hackExited()
 {
-	// Hack exited while we're supposed to be saving the screen.
-	// Make sure the saver window is black.
+    // Hack exited while we're supposed to be saving the screen.
+    // Make sure the saver window is black.
     XSetWindowBackground(QX11Info::display(), winId(), 0);
     XClearWindow(QX11Info::display(), winId());
 }
@@ -1104,7 +1105,6 @@ void LockProcess::resume( bool force )
         return; // no resuming with dialog visible or when not visible
     if( mSuspended && mHackProc.state() == QProcess::Running )
     {
-        XForceScreenSaver(QX11Info::display(), ScreenSaverReset );
         QPainter p( this );
         p.drawPixmap( 0, 0, mSavedScreen );
         p.end();
@@ -1122,14 +1122,12 @@ void LockProcess::resume( bool force )
 //
 bool LockProcess::checkPass()
 {
-    killTimer(mAutoLogoutTimerId);
-
     if (isPlasmaValid()) {
         mPlasmaDBus->call(QDBus::NoBlock, "setActive", true);
     }
 
     PasswordDlg passDlg( this, &greetPlugin);
-    int ret = execDialog( &passDlg );
+    const int ret = execDialog( &passDlg );
 
     if (isPlasmaValid()) {
         if (ret == QDialog::Rejected) {
@@ -1160,7 +1158,7 @@ bool LockProcess::checkPass(const QString &reason)
         return false;
     }
     PasswordDlg passDlg(this, &greetPlugin, reason);
-    int ret = execDialog( &passDlg );
+    const int ret = execDialog( &passDlg );
     kDebug() << ret;
 
     //FIXME do we need to copy&paste that SubstructureNotifyMask code above?
@@ -1210,6 +1208,7 @@ bool LockProcess::eventFilter(QObject *o, QEvent *e)
 
 int LockProcess::execDialog( QDialog *dlg )
 {
+
     QFrame *winFrame = new QFrame( dlg );
     winFrame->setFrameStyle( QFrame::WinPanel | QFrame::Raised );
     winFrame->setLineWidth( 2 );
@@ -1219,24 +1218,34 @@ int LockProcess::execDialog( QDialog *dlg )
 
     dlg->adjustSize();
 
+    int screen = Kephal::ScreenUtils::primaryScreenId();
+    if (Kephal::ScreenUtils::numScreens() > 1) {
+        screen = Kephal::ScreenUtils::screenId(QCursor::pos());
+    }
+
+    const QRect screenRect = Kephal::ScreenUtils::screenGeometry(screen);
     QRect rect = dlg->geometry();
-    rect.moveCenter(KGlobalSettings::desktopGeometry(QCursor::pos()).center());
-    dlg->move( rect.topLeft() );
+    rect.moveCenter(screenRect.center());
+    dlg->move(rect.topLeft());
 
     if (mDialogs.isEmpty())
     {
+        if (mAutoLogoutTimerId)
+            killTimer(mAutoLogoutTimerId);
         suspend();
         XChangeActivePointerGrab( QX11Info::display(), GRABEVENTS,
                 QCursor(Qt::ArrowCursor).handle(), CurrentTime);
     }
     mDialogs.prepend( dlg );
     fakeFocusIn( dlg->winId());
-    int rt = dlg->exec();
-    int pos = mDialogs.indexOf( dlg );
+    const int rt = dlg->exec();
+    const int pos = mDialogs.indexOf( dlg );
     if (pos != -1)
         mDialogs.remove( pos );
     if( mDialogs.isEmpty() ) {
         resume( false );
+        if (mAutoLogoutTimerId)
+            mAutoLogoutTimerId = startTimer(mAutoLogoutTimeout * 1000);
     }
     updateFocus();
 
@@ -1246,22 +1255,6 @@ int LockProcess::execDialog( QDialog *dlg )
     return rt;
 }
 
-void LockProcess::preparePopup()
-{
-    QWidget *dlg = (QWidget *)sender();
-    mDialogs.prepend( dlg );
-    fakeFocusIn( dlg->winId() );
-}
-
-void LockProcess::cleanupPopup()
-{
-    QWidget *dlg = (QWidget *)sender();
-
-    int pos = mDialogs.indexOf( dlg );
-    mDialogs.remove( pos );
-    updateFocus();
-}
-
 void LockProcess::updateFocus()
 {
     if (mDialogs.isEmpty()) {
@@ -1311,16 +1304,15 @@ bool LockProcess::x11Event(XEvent *event)
             //note: mSetupMode should end when we either get a winid or hit the checkPlasma timeout
             if (mSuppressUnlock.isActive() && (mSetupMode || !mForeignInputWindows.isEmpty())) {
                 mSuppressUnlock.start(); //help, help, I'm being suppressed!
+                if (mAutoLogoutTimerId) {
+                    killTimer(mAutoLogoutTimerId);
+                    mAutoLogoutTimerId = startTimer(mAutoLogoutTimeout * 1000);
+                }
             } else if (!mLocked || checkPass()) {
                 quitSaver();
                 mBusy = false;
                 return true; //it's better not to forward any input while quitting, right?
             }
-            if (mAutoLogoutTimerId) // we need to restart the auto logout countdown
-            {
-                killTimer(mAutoLogoutTimerId);
-                mAutoLogoutTimerId = startTimer(mAutoLogoutTimeout * 1000);
-            }
             mBusy = false;
             ret = true;
             break;
@@ -1355,7 +1347,7 @@ bool LockProcess::x11Event(XEvent *event)
 
         case ConfigureNotify: // from SubstructureNotifyMask on the root window
             if(event->xconfigure.event == QX11Info::appRootWindow()) {
-                //kDebug() << "ConfigureNotify:"; 
+                //kDebug() << "ConfigureNotify:";
                 //the stacking order changed, so let's change the stacking order!
                 stayOnTop();
             }
@@ -1472,62 +1464,65 @@ LockProcess::WindowType LockProcess::windowType(WId id)
         XFree(data);
     }
     return type;
-/*    if (result != Success) {
-        return false;
-    }
-    if (actualType == tag) {
-        return true;
-    }
-    //managed windows will have a pesky frame we have to bypass
-    XWindowAttributes attr;
-    XGetWindowAttributes(display, id, &attr);
-    if (!attr.override_redirect) {
-        //check the real client window
-        if (Window client = XmuClientWindow(display, id)) {
-            result = XGetWindowProperty(display, client, tag, 0, 0, False, tag, &actualType,
-                    &actualFormat, &nitems, &remaining, &data);
-            kDebug() << (result == Success) << (actualType == tag);
-            if (data) {
-                XFree(data);
-            }
-            return (result == Success) && (actualType == tag);
-*        }
-    }
-    return false;*/
 }
 
 void LockProcess::stayOnTop()
 {
-    if(!(mDialogs.isEmpty() && mForeignWindows.isEmpty()))
-    {
-        // this restacking is written in a way so that
-        // if the stacking positions actually don't change,
-        // all restacking operations will be no-op,
-        // and no ConfigureNotify will be generated,
-        // thus avoiding possible infinite loops
-        Window* stack = new Window[ mDialogs.count() + mForeignWindows.count() + 1 ];
-        int count = 0;
-        if (!mDialogs.isEmpty()) {
-            XRaiseWindow( QX11Info::display(), mDialogs.first()->winId()); // raise topmost
-            // and stack others below it
-            for( QVector< QWidget* >::ConstIterator it = mDialogs.constBegin();
-                    it != mDialogs.constEnd();
-                    ++it )
-                stack[ count++ ] = (*it)->winId();
-        } else {
-            XRaiseWindow( QX11Info::display(), mForeignWindows.first()); // raise topmost
+    // this restacking is written in a way so that
+    // if the stacking positions actually don't change,
+    // all restacking operations will be no-op,
+    // and no ConfigureNotify will be generated,
+    // thus avoiding possible infinite loops
+    QVector< Window > stack( mDialogs.count() + mForeignWindows.count() + 1 );
+    int count = 0;
+    // dialogs first
+    foreach( QWidget* w, mDialogs )
+        stack[ count++ ] = w->winId();
+    // now the plasma stuff below the dialogs
+    foreach( WId w, mForeignWindows )
+        stack[ count++ ] = w;
+    // finally, the saver window
+    stack[ count++ ] = winId();
+    // We actually have to check the current stacking order. When an override-redirect
+    // window is shown or raised, it can get above the screensaver window and there's not
+    // much to do to prevent it (only the compositing manager can prevent that). This
+    // is detected by the screenlocker and handled here, but the contents of the window
+    // may remain visible, since some screensavers don't react to Expose events and
+    // don't repaint as necessary. Therefore, if a window is detected above any of the windows
+    // related to screenlocking, I don't see any better possibility than to completely
+    // erase the screenlocker window.
+    // It is important to first detect, than restack and then erase.
+    bool needs_erase = false;
+    Window r, p;
+    Window* real;
+    unsigned int nreal;
+    if( XQueryTree( x11Info().display(), x11Info().appRootWindow(), &r, &p, &real, &nreal )
+        && real != NULL ) {
+        bool found_ours = false;
+        for( unsigned int i = 0;
+             i < nreal;
+             ++i ) {
+            if( stack.contains( real[ i ] ))
+                found_ours = true;
+            else {
+                if( found_ours ) {
+                    needs_erase = true;
+                    break;
+                }
+            }
         }
-        //now the plasma stuff below the dialogs
-        foreach (const WId w, mForeignWindows) {
-            stack[count++] = w;
-        }
-        //finally, the saver window
-        stack[ count++ ] = winId();
-        XRestackWindows( x11Info().display(), stack, count );
-        //kDebug() << "restacked" << count;
-        delete[] stack;
-    } else {
-        XRaiseWindow(QX11Info::display(), winId());
+        XFree( real );
+    }
+    // do the actual restacking if needed
+    XRaiseWindow( x11Info().display(), stack[ 0 ] );
+    if( count > 1 )
+        XRestackWindows( x11Info().display(), stack.data(), count );
+    if( needs_erase ) {
+        kDebug( 1024 ) << "Window above screensaver, raising, erasing";
+        QPainter p( this );
+        p.fillRect( rect(), Qt::black );
+        p.end();
+        QApplication::syncX();
     }
 }
 
@@ -1551,7 +1546,7 @@ void LockProcess::checkDPMSActive()
 #if defined(HAVE_XF86MISC) && defined(HAVE_XF86MISCSETGRABKEYSSTATE)
 // see http://cvsweb.xfree86.org/cvsweb/xc/programs/Xserver/hw/xfree86/common/xf86Events.c#rev3.113
 // This allows enabling the "Allow{Deactivate/Closedown}Grabs" options in XF86Config,
-// and krunner_lock will still lock the session.
+// and kscreenlocker will still lock the session.
 static enum { Unknown, Yes, No } can_do_xf86_lock = Unknown;
 void LockProcess::lockXF86()
 {
@@ -1596,7 +1591,6 @@ void LockProcess::unlockXF86()
 void LockProcess::msgBox( QWidget *parent, QMessageBox::Icon type, const QString &txt )
 {
     QDialog box( parent, Qt::X11BypassWindowManagerHint );
-    box.setModal( true );
 
     QLabel *label1 = new QLabel( &box );
     label1->setPixmap( QMessageBox::standardIcon( type ) );
