@@ -46,26 +46,26 @@
 #include <kmessagebox.h>
 #include <ksgrd/SensorAgent.h>
 #include <ksgrd/SensorManager.h>
-#include <kstandarddirs.h>
 #include <kstatusbar.h>
 #include <kstandardaction.h>
 #include <ktoggleaction.h>
-#include <ktoolbar.h>
 #include <kurl.h>
 #include <kwindowsystem.h>
 #include <QSplitter>
 
-#include "../version.h"
+#include <kdeversion.h>
 #include "SensorBrowser.h"
 #include "Workspace.h"
 #include "WorkSheet.h"
 #include "StyleEngine.h"
 #include "HostConnector.h"
 #include "ProcessController.h"
+#include "ProcessTable.h"
 
 #include "ksysguard.h"
 
 
+ProcessController *sLocalProcessController = NULL;
 
 //Comment out to stop ksysguard from forking.  Good for debugging
 //#define FORK_KSYSGUARD
@@ -96,6 +96,8 @@ TopLevel::TopLevel()
   connect( mWorkSpace, SIGNAL( currentChanged( int ) ),
            SLOT( currentTabChanged( int ) ) );
 
+  sLocalProcessController = new ProcessController( this);
+
   /* Create the status bar. It displays some information about the
    * number of processes and the memory consumption of the local
    * host. */
@@ -122,38 +124,44 @@ TopLevel::TopLevel()
   mInsertWorksheetAction = actionCollection()->addAction("import_worksheet");
   mInsertWorksheetAction->setIcon(KIcon("document-open") );
   connect(mInsertWorksheetAction, SIGNAL(triggered(bool)), mWorkSpace, SLOT( importWorkSheet() ));
-  mTabRemoveAction = actionCollection()->addAction( "remove_worksheet" );
-  mTabRemoveAction->setIcon( KIcon("tab-close") );
-  connect(mTabRemoveAction, SIGNAL(triggered(bool)), mWorkSpace, SLOT( removeWorkSheet() ));
   mTabExportAction = actionCollection()->addAction( "export_worksheet" );
   mTabExportAction->setIcon( KIcon("document-save-as") );
   connect(mTabExportAction, SIGNAL(triggered(bool)), mWorkSpace, SLOT( exportWorkSheet() ));
-
-  mQuitAction = NULL;
-
+  mTabRemoveAction = actionCollection()->addAction( "remove_worksheet" );
+  mTabRemoveAction->setIcon( KIcon("tab-close") );
+  connect(mTabRemoveAction, SIGNAL(triggered(bool)), mWorkSpace, SLOT( removeWorkSheet() ));
   mMonitorRemoteAction = actionCollection()->addAction( "connect_host" );
   mMonitorRemoteAction->setIcon( KIcon("network-connect") );
   connect(mMonitorRemoteAction, SIGNAL(triggered(bool)), SLOT( connectHost() ));
+  //knewstuff2 action
+  mHotNewWorksheetAction = actionCollection()->addAction( "get_new_worksheet" );
+  mHotNewWorksheetAction->setIcon( KIcon("network-server") );
+  connect(mHotNewWorksheetAction, SIGNAL(triggered(bool)), mWorkSpace, SLOT( getHotNewWorksheet() ));
+  mHotNewWorksheetUploadAction = actionCollection()->addAction( "upload_worksheet" );
+  mHotNewWorksheetUploadAction->setIcon( KIcon("network-server") );
+  connect(mHotNewWorksheetUploadAction, SIGNAL(triggered(bool)), mWorkSpace, SLOT( uploadHotNewWorksheet() ));
+
+  mQuitAction = NULL;
 
   mConfigureSheetAction = actionCollection()->addAction( "configure_sheet" );
   mConfigureSheetAction->setIcon( KIcon("configure") );
   connect(mConfigureSheetAction, SIGNAL(triggered(bool)), mWorkSpace, SLOT( configure() ));
 
-  if (!initialGeometrySet())
-    resize( QSize(700, 480).expandedTo(minimumSizeHint()));
-
   retranslateUi();
 }
 
 void TopLevel::retranslateUi()
 {
   setPlainCaption( i18n( "System Monitor" ) );
-  mNewWorksheetAction->setText(i18n( "&New Worksheet..." ));
-  mInsertWorksheetAction->setText(i18n( "Import Worksheet..." ));
-  mTabExportAction->setText( i18n( "&Export Worksheet..." ) );
-  mTabRemoveAction->setText( i18n( "&Remove Worksheet" ) );
-  mMonitorRemoteAction->setText( i18n( "Monitor remote machine..." ) );
-  mConfigureSheetAction->setText( i18n( "&Worksheet Properties" ) );
+  mNewWorksheetAction->setText(i18n( "&New Tab..." ));
+  mInsertWorksheetAction->setText(i18n( "Import Tab Fr&om File..." ));
+  mTabExportAction->setText( i18n( "Save Tab &As..." ) );
+  mTabRemoveAction->setText( i18n( "&Close Tab" ) );
+  mMonitorRemoteAction->setText( i18n( "Monitor &Remote Machine..." ) );
+  mHotNewWorksheetAction->setText( i18n( "&Download New Tabs..." ) );
+  mHotNewWorksheetUploadAction->setText( i18n( "&Upload Current Tab..." ) );
+
+  mConfigureSheetAction->setText( i18n( "Tab &Properties" ) );
   if(mQuitAction) {
     KAction *tmpQuitAction = KStandardAction::quit( NULL, NULL, NULL );
     mQuitAction->setText(tmpQuitAction->text());
@@ -173,6 +181,7 @@ void TopLevel::currentTabChanged(int index)
   bool locked = !sheet || sheet->isLocked();
   mTabRemoveAction->setVisible(!locked);
   mTabExportAction->setVisible(!locked);
+  mHotNewWorksheetUploadAction->setVisible(!locked);
   mMonitorRemoteAction->setVisible(!locked);
 
   if(!locked && !mSensorBrowser) {
@@ -214,6 +223,11 @@ void TopLevel::removeWorkSheet( const QString &fileName )
   mWorkSpace->removeWorkSheet( fileName );
 }
 
+void TopLevel::getHotNewWorksheet()
+{
+  mWorkSpace->getHotNewWorksheet( );
+}
+
 QStringList TopLevel::listSensors( const QString &hostName )
 {
   if(!mSensorBrowser) {
@@ -248,7 +262,9 @@ void TopLevel::initStatusBar()
   KToggleAction *sb = dynamic_cast<KToggleAction*>(action("options_show_statusbar"));
   if (sb)
      connect(sb, SIGNAL(toggled(bool)), this, SLOT(updateStatusBar()));
-  setupGUI(ToolBar | Keys | StatusBar | Save | Create);
+  setupGUI(QSize(800,600), ToolBar | Keys | StatusBar | Save | Create);
+
+  updateStatusBar();
 }
 
 void TopLevel::updateStatusBar()
@@ -299,22 +315,6 @@ void TopLevel::disconnectHost()
     mSensorBrowser->disconnect();
 }
 
-void TopLevel::editToolbars()
-{
-  saveMainWindowSettings( KConfigGroup( KGlobal::config(), "MainWindow" ) );
-  KEditToolBar dlg( actionCollection() );
-  connect( &dlg, SIGNAL( newToolBarConfig() ), this,
-           SLOT( slotNewToolbarConfig() ) );
-
-  dlg.exec();
-}
-
-void TopLevel::slotNewToolbarConfig()
-{
-  createGUI();
-  applyMainWindowSettings( KConfigGroup( KGlobal::config(), "MainWindow" ) );
-}
-
 bool TopLevel::event( QEvent *e )
 {
   if ( e->type() == QEvent::User ) {
@@ -396,17 +396,9 @@ void TopLevel::readProperties( const KConfigGroup& cfg )
   mWorkSpace->readProperties( cfg );
 
   QList<WorkSheet *> workSheets = mWorkSpace->getWorkSheets();
-  ProcessController *processController = NULL;
-  foreach(WorkSheet *sheet, workSheets) {
-    processController = sheet->getLocalProcessController();
-    if(processController != NULL) {
-      for(int i = 0; i < processController->actions().size(); i++) {
-        actionCollection()->addAction("processAction" + QString::number(i), processController->actions().at(i));
-      }
-    }
+  for(int i = 0; i < sLocalProcessController->actions().size(); i++) {
+    actionCollection()->addAction("processAction" + QString::number(i), sLocalProcessController->actions().at(i));
   }
-
-  updateStatusBar();
 }
 
 void TopLevel::saveProperties( KConfigGroup& cfg )
@@ -432,15 +424,15 @@ void TopLevel::answerReceived( int id, const QList<QByteArray> &answerList )
   if(!answerList.isEmpty()) answer = answerList[0];
   QString s;
   static QString unit;
-  static long mFree = 0;
-  static long mUsedApplication = 0;
-  static long mUsedTotal = 0;
-  static long sUsed = 0;
-  static long sFree = 0;
+  static qlonglong mFree = 0;
+  static qlonglong mUsedApplication = 0;
+  static qlonglong mUsedTotal = 0;
+  static qlonglong sUsed = 0;
+  static qlonglong sFree = 0;
 
   switch ( id ) {
     case 0:
-      s = i18n( " %1 processes ", answer.toInt() );
+      s = i18np( " 1 process ", " %1 processes ", answer.toInt() );
       sbProcessCount->setText( s );
 
       break;
@@ -451,27 +443,27 @@ void TopLevel::answerReceived( int id, const QList<QByteArray> &answerList )
       break;
 
     case 2:
-      mFree = answer.toLong();
+      mFree = answer.toLongLong();
       break;
 
     case 3:
-      mUsedTotal = answer.toLong();
+      mUsedTotal = answer.toLongLong();
       break;
 
     case 4:
-      mUsedApplication = answer.toLong();
-      s = i18n( " Memory: %1 / %2 " ,
-                KGlobal::locale()->formatByteSize( mUsedApplication*1024),
-                KGlobal::locale()->formatByteSize( (mFree+mUsedTotal)*1024 ) );
+      mUsedApplication = answer.toLongLong();
+      s = i18nc( "Arguments are formatted byte sizes (used/total)", " Memory: %1 / %2 " ,
+                 KGlobal::locale()->formatByteSize( mUsedApplication*1024),
+                 KGlobal::locale()->formatByteSize( (mFree+mUsedTotal)*1024 ) );
       sbMemTotal->setText( s );
       break;
 
     case 5:
-      sFree = answer.toLong();
+      sFree = answer.toLongLong();
       break;
 
     case 6:
-      sUsed = answer.toLong();
+      sUsed = answer.toLongLong();
       setSwapInfo( sUsed, sFree, unit );
       break;
 
@@ -483,15 +475,15 @@ void TopLevel::answerReceived( int id, const QList<QByteArray> &answerList )
   }
 }
 
-void TopLevel::setSwapInfo( long used, long free, const QString & )
+void TopLevel::setSwapInfo( qlonglong used, qlonglong free, const QString & )
 {
   QString msg;
   if ( used == 0 && free == 0 ) // no swap available
     msg = i18n( " No swap space available " );
   else {
-    msg = i18n( " Swap: %1 / %2 " ,
-                KGlobal::locale()->formatByteSize( used*1024 ),
-                KGlobal::locale()->formatByteSize( (free+used)*1024) );
+    msg = i18nc( "Arguments are formatted byte sizes (used/total)", " Swap: %1 / %2 " ,
+                 KGlobal::locale()->formatByteSize( used*1024 ),
+                 KGlobal::locale()->formatByteSize( (free+used)*1024) );
   }
 
   sbSwapTotal->setText( msg );
@@ -507,13 +499,11 @@ extern "C" KDE_EXPORT int kdemain( int argc, char** argv )
 #ifdef FORK_KSYSGUARD
   int initpipe[ 2 ];
   pipe( initpipe );
-#endif
   /* This forking will put ksysguard in it's own session not having a
    * controlling terminal attached to it. This prevents ssh from
    * using this terminal for password requests. Thus, you
    * need a ssh with ssh-askpass support to popup an X dialog to
    * enter the password. */
-#ifdef FORK_KSYSGUARD
   pid_t pid;
   if ( ( pid = fork() ) < 0 )
     return -1;
@@ -535,7 +525,7 @@ extern "C" KDE_EXPORT int kdemain( int argc, char** argv )
 #endif
 
   KAboutData aboutData( "ksysguard", 0, ki18n( "System Monitor" ),
-                        KSYSGUARD_VERSION, ki18n(Description), KAboutData::License_GPL,
+                        KDE_VERSION_STRING, ki18n(Description), KAboutData::License_GPL,
                         ki18n( "(c) 1996-2008 The KDE System Monitor Developers" ) );
   aboutData.addAuthor( ki18n("John Tapsell"), ki18n("Current Maintainer"), "john.tapsell@kde.org" );
   aboutData.addAuthor( ki18n("Chris Schlaeger"), ki18n("Previous Maintainer"), "cs@kde.org" );
@@ -550,6 +540,7 @@ extern "C" KDE_EXPORT int kdemain( int argc, char** argv )
                        "Parts derived (by permission) from the sunos5\n"
                        "module of William LeFebvre's \"top\" utility." ),
                        "tk@Genetik.Uni-Bielefeld.DE" );
+  aboutData.setProgramIconName("utilities-system-monitor");
 
   KCmdLineArgs::init( argc, argv, &aboutData );
 
@@ -558,9 +549,6 @@ extern "C" KDE_EXPORT int kdemain( int argc, char** argv )
   KCmdLineArgs::addCmdLineOptions( options );
   // initialize KDE application
   KApplication *app = new KApplication;
-  QApplication::setWindowIcon(KIcon("utilities-system-monitor"));
-  KGlobal::locale()->insertCatalog("processcore");
-  KGlobal::locale()->insertCatalog("processui");
 
   KSGRD::SensorMgr = new KSGRD::SensorManager();
   KSGRD::Style = new KSGRD::StyleEngine();
@@ -572,10 +560,13 @@ extern "C" KDE_EXPORT int kdemain( int argc, char** argv )
 #endif
   topLevel = new TopLevel();
 
-  topLevel->initStatusBar();
 
   // create top-level widget
   topLevel->readProperties( KConfigGroup( KGlobal::config(), "MainWindow" ) );
+  // setup the statusbar, toolbar etc.
+  // Note that this comes after creating the top-level widgets whcih also 
+  // sets up the various QActions that the user may have added to the toolbar
+  topLevel->initStatusBar();
 
   //There seems to be some serious bugs with the session restore code.  Disabling
 //  if ( app->isSessionRestored() )
