@@ -77,6 +77,10 @@ static void oxkwincleanupBefore()
     h->invalidateCaches();
 }
 
+void OxygenClient::invalidateCaches()
+{
+}
+
 void renderDot(QPainter *p, const QPointF &point, qreal diameter)
 {
     p->drawEllipse(QRectF(point.x()-diameter/2, point.y()-diameter/2, diameter, diameter));
@@ -86,6 +90,8 @@ void renderDot(QPainter *p, const QPointF &point, qreal diameter)
 OxygenClient::OxygenClient(KDecorationBridge *b, KDecorationFactory *f)
     : KCommonDecorationUnstable(b, f)
     , colorCacheInvalid_(true)
+    , shadowTiles_(NULL)
+    , glowTiles_(NULL)
     , helper_(*globalHelper)
 {
     qAddPostRoutine(oxkwincleanupBefore);
@@ -93,6 +99,8 @@ OxygenClient::OxygenClient(KDecorationBridge *b, KDecorationFactory *f)
 
 OxygenClient::~OxygenClient()
 {
+    delete glowTiles_;
+    delete shadowTiles_;
 }
 QString OxygenClient::visibleName() const
 {
@@ -123,7 +131,8 @@ bool OxygenClient::decorationBehaviour(DecorationBehaviour behaviour) const
 
 int OxygenClient::layoutMetric(LayoutMetric lm, bool respectWindowState, const KCommonDecorationButton *btn) const
 {
-    bool maximized = maximizeMode()==MaximizeFull && !options()->moveResizeMaximizedWindows();
+    bool maximized = isMaximized();
+    int frameWidth = OxygenFactory::borderSize();
 
     switch (lm) {
         case LM_BorderLeft:
@@ -133,7 +142,13 @@ int OxygenClient::layoutMetric(LayoutMetric lm, bool respectWindowState, const K
             if (respectWindowState && maximized) {
                 return 0;
             } else {
-                return BFRAMESIZE;
+                // Even for thin borders (2px wide) we want to preserve
+                // the rounded corners having a minimum height of 7px
+                if (lm == LM_BorderBottom) {
+                    return qMax(frameWidth, 7);
+                } else {
+                    return frameWidth;
+                }
             }
         }
 
@@ -182,6 +197,13 @@ int OxygenClient::layoutMetric(LayoutMetric lm, bool respectWindowState, const K
         case LM_ButtonMarginTop:
             return 0;
 
+        // outer margin for shadow/glow
+        case LM_OuterPaddingLeft:
+        case LM_OuterPaddingRight:
+        case LM_OuterPaddingTop:
+        case LM_OuterPaddingBottom:
+            return SHADOW_WIDTH;
+
         default:
             return KCommonDecoration::layoutMetric(lm, respectWindowState, btn);
     }
@@ -271,14 +293,16 @@ QColor OxygenClient::titlebarTextColor(const QPalette &palette)
     }
 }
 
+bool OxygenClient::isMaximized() const
+{
+    return maximizeMode()==MaximizeFull && !options()->moveResizeMaximizedWindows();
+}
 
 void OxygenClient::paintEvent(QPaintEvent *e)
 {
     Q_UNUSED(e)
     if (!OxygenFactory::initialized()) return;
 
-    updateButtons();
-
     QPalette palette = widget()->palette();
     QPainter painter(widget());
 
@@ -290,7 +314,9 @@ void OxygenClient::paintEvent(QPaintEvent *e)
         palette.setCurrentColorGroup(QPalette::Inactive);
 
     int x,y,w,h;
-    QRect frame = widget()->frameGeometry();
+    QRect frame = widget()->rect();
+    frame.adjust(SHADOW_WIDTH, SHADOW_WIDTH, -SHADOW_WIDTH, -SHADOW_WIDTH);
+
     QColor color = OxygenFactory::blendTitlebarColors() ? palette.window().color() : options()->color( ColorTitleBar, isActive());
     QColor light = helper_.calcLightColor( color );
     QColor dark = helper_.calcDarkColor( color );
@@ -312,8 +338,35 @@ void OxygenClient::paintEvent(QPaintEvent *e)
         pal2.setColor( QPalette::Window, options()->color(
             KDecorationDefines::ColorTitleBar, isActive()));
     }
+
+    // draw shadow
+
+    if (compositingActive() && !isMaximized())
+        shadowTiles(color,KDecoration::options()->color(ColorTitleBar),
+                SHADOW_WIDTH, isActive())->render( frame.adjusted(-SHADOW_WIDTH+4,
+                    -SHADOW_WIDTH+4, SHADOW_WIDTH-4, SHADOW_WIDTH-4),
+                &painter, TileSet::Ring);
+
     // draw window background
-    helper_.renderWindowBackground(&painter, frame, this->widget(), pal2, 0);
+    bool isCompositingActive = compositingActive();
+
+    if (isCompositingActive) {
+        frame.getRect(&x, &y, &w, &h);
+
+        QRegion mask(x+5, y+0, w-10, h-0);
+        mask += QRegion(x+0, y+5, w-0, h-10);
+        mask += QRegion(x+1, y+3, w-2, h-6);
+        mask += QRegion(x+2, y+2, w-4, h-4);
+        mask += QRegion(x+3, y+1, w-6, h-2);
+
+        painter.setClipRegion(mask);
+    } 
+
+    helper_.renderWindowBackground(&painter, frame, this->widget(), pal2, SHADOW_WIDTH);
+
+    if (isCompositingActive) {
+        painter.setClipping(false);
+    }
 
     // draw title text
     painter.setFont(options()->font(isActive(), false));
@@ -324,8 +377,7 @@ void OxygenClient::paintEvent(QPaintEvent *e)
     painter.setRenderHint(QPainter::Antialiasing);
 
     // Draw dividing line
-    frame = widget()->rect();
-    if (shadowsActive()) {
+    if (compositingActive()) {
         frame.adjust(-1,-1,1,1);
     }
     frame.getRect(&x, &y, &w, &h);
@@ -348,12 +400,14 @@ void OxygenClient::paintEvent(QPaintEvent *e)
         if (align & Qt::AlignLeft) {
             int left = titleLeft + QFontMetrics(options()->font(isActive(), false)).width(caption()) + 4;
             int right = titleLeft + titleWidth;
-            drawScratch(&painter, palette, left, right, titleTop+6);
+            if (left < right)
+                drawStripes(&painter, palette, left, right, titleTop+6);
         }
         if (align & Qt::AlignRight) {
             int left = titleLeft;
             int right = titleLeft + titleWidth - QFontMetrics(options()->font(isActive(), false)).width(caption()) - 4;
-            drawScratch(&painter, palette, right, left, titleTop+6);
+            if (left < right)
+                drawStripes(&painter, palette, right, left, titleTop+6);
         }
         if (align & Qt::AlignHCenter) {
             int textWidth = QFontMetrics(options()->font(isActive(), false)).width(caption());
@@ -361,18 +415,18 @@ void OxygenClient::paintEvent(QPaintEvent *e)
             int centerLeft = titleLeft + titleWidth/2 - textWidth/2 - 4;
             int centerRight = titleLeft + titleWidth/2 + textWidth/2 + 4;
             int right = titleLeft + titleWidth;
-            drawScratch(&painter, palette, centerLeft, left, titleTop+6);
-            drawScratch(&painter, palette, centerRight, right, titleTop+6);
+            if (left < centerLeft && right > centerRight) {
+                drawStripes(&painter, palette, centerLeft, left, titleTop+6);
+                drawStripes(&painter, palette, centerRight, right, titleTop+6);
+            }
         }
     }
 
     // Draw shadows of the frame
-    bool maximized = maximizeMode()==MaximizeFull && !options()->moveResizeMaximizedWindows();
-
-    if(maximized)
+    if(isMaximized())
         return;
 
-    helper_.drawFloatFrame(&painter, frame, color, !shadowsActive(), isActive(),
+    helper_.drawFloatFrame(&painter, frame, color, !compositingActive(), isActive(),
                                                                 KDecoration::options()->color(ColorTitleBar));
 
     if(!isResizable())
@@ -393,19 +447,19 @@ void OxygenClient::paintEvent(QPaintEvent *e)
     renderDot(&painter, QPointF(6.5, 2.5), 1.8);
 }
 
-void OxygenClient::drawScratch(QPainter *p, QPalette &palette, const int start, const int end, const int topMargin)
+void OxygenClient::drawStripes(QPainter *p, QPalette &palette, const int start, const int end, const int topMargin)
 {
-    QLinearGradient scratchlg(QPoint(start,0), QPoint(end,0));
-    scratchlg.setColorAt(0.0, Qt::transparent);
-    scratchlg.setColorAt(0.05, KDecoration::options()->color(ColorTitleBar));
-    scratchlg.setColorAt(1.0, Qt::transparent);
-    QPen pen1(scratchlg, 0.5);
+    QLinearGradient stripeGradient(QPoint(start,0), QPoint(end,0));
+    stripeGradient.setColorAt(0.0, Qt::transparent);
+    stripeGradient.setColorAt(0.05, KDecoration::options()->color(ColorTitleBar));
+    stripeGradient.setColorAt(1.0, Qt::transparent);
+    QPen pen1(stripeGradient, 0.5);
 
-    QLinearGradient scratchlg2(QPoint(start,0), QPoint(end,0));
-    scratchlg2.setColorAt(0.0, Qt::transparent);
-    scratchlg2.setColorAt(0.05, helper_.calcLightColor(palette.color(QPalette::Window)));
-    scratchlg2.setColorAt(1.0, Qt::transparent);
-    QPen pen2(scratchlg2, 0.5);
+    QLinearGradient stripeGradient2(QPoint(start,0), QPoint(end,0));
+    stripeGradient2.setColorAt(0.0, Qt::transparent);
+    stripeGradient2.setColorAt(0.05, helper_.calcLightColor(palette.color(QPalette::Window)));
+    stripeGradient2.setColorAt(1.0, Qt::transparent);
+    QPen pen2(stripeGradient2, 0.5);
 
     bool antialiasing = p->testRenderHint(QPainter::Antialiasing);
     p->setRenderHint(QPainter::Antialiasing, false);
@@ -422,69 +476,178 @@ void OxygenClient::drawScratch(QPainter *p, QPalette &palette, const int start,
 
 void OxygenClient::updateWindowShape()
 {
-    bool maximized = maximizeMode()==MaximizeFull && !options()->moveResizeMaximizedWindows();
-    int w=widget()->width();
-    int h=widget()->height();
-
-    if(maximized) {
-        QRegion mask(0,0,w,h);
-        setMask(mask);
+    if(isMaximized() || compositingActive()) {
+        clearMask();
         return;
     }
 
-    if (!shadowsActive()) {
-        QRegion mask(4, 0, w-8, h);
-        mask += QRegion(0, 4, w, h-8);
-        mask += QRegion(2, 1, w-4, h-2);
-        mask += QRegion(1, 2, w-2, h-4);
+    int w=widget()->width() - (int)SHADOW_WIDTH;
+    int h=widget()->height() - (int)SHADOW_WIDTH;
 
-        setMask(mask);
-    }
-   else {
-        QRegion mask(5, 0, w-10, h-0);
-        mask += QRegion(0, 5, w-0, h-10);
-        mask += QRegion(2, 2, w-4, h-4);
-        mask += QRegion(3, 1, w-6, h-2);
-        mask += QRegion(1, 3, w-2, h-6);
+    // set a mask if compositing is not active
+    int sw = SHADOW_WIDTH;
+    QRegion mask(sw+4, sw+0, -sw+w-8, -sw+h);
+    mask += QRegion(sw+0, sw+4, -sw+w, -sw+h-8);
+    mask += QRegion(sw+2, sw+1, -sw+w-4, -sw+h-2);
+    mask += QRegion(sw+1, sw+2, -sw+w-2, -sw+h-4);
 
-        setMask(mask);
-    }
+    setMask(mask);
 }
 
-QList<QRect> OxygenClient::shadowQuads( ShadowType type ) const
+TileSet *OxygenClient::shadowTiles(const QColor& color, const QColor& glow, qreal size, bool active)
 {
-    Q_UNUSED(type)
+    ShadowTilesOption opt;
+    opt.active      = active;
+    opt.width       = size;
+    opt.windowColor = color;
+    opt.glowColor   = glow;
 
-    QSize size = widget()->size();
-    int outside=21, underlap=4, cornersize=25;
-    // These are underlap under the decoration so the corners look nicer 10px on the outside
-    QList<QRect> quads;
-    quads.append(QRect(-outside, size.height()-underlap, cornersize, cornersize));
-    quads.append(QRect(underlap, size.height()-underlap, size.width()-2*underlap, cornersize));
-    quads.append(QRect(size.width()-underlap, size.height()-underlap, cornersize, cornersize));
-    quads.append(QRect(-outside, underlap, cornersize, size.height()-2*underlap));
-    quads.append(QRect(size.width()-underlap, underlap, cornersize, size.height()-2*underlap));
-    quads.append(QRect(-outside, -outside, cornersize, cornersize));
-    quads.append(QRect(underlap, -outside, size.width()-2*underlap, cornersize));
-    quads.append(QRect(size.width()-underlap,     -outside, cornersize, cornersize));
-    return quads;
-}
+    ShadowTilesOption currentOpt = active ? shadowTilesOption_ : glowTilesOption_;
 
-double OxygenClient::shadowOpacity( ShadowType type ) const
-{
-    switch( type ) {
-        case ShadowBorderedActive:
-            if( isActive() )
-                return 1.0;
-            return 0.0;
-        case ShadowBorderedInactive:
-            if( isActive() )
-                return 0.0;
-            return 1.0;
-        default:
-            abort(); // Should never be reached
+    bool optionChanged = true;
+    if (currentOpt.active == opt.active
+            && currentOpt.width == opt.width
+            && opt.windowColor == opt.windowColor
+            && opt.glowColor == opt.glowColor)
+        optionChanged = false;
+
+    if (active && glowTiles_ && !optionChanged)
+        return glowTiles_;
+    else if (!active && shadowTiles_ && !optionChanged)
+        return shadowTiles_;
+        
+    TileSet *tileSet = 0;
+
+    //---------------------------------------------------------------
+    // Create new glow/shadow tiles
+
+    QColor light = oxygenHelper()->calcLightColor(oxygenHelper()->backgroundTopColor(color));
+    QColor dark = oxygenHelper()->calcDarkColor(oxygenHelper()->backgroundBottomColor(color));
+
+    QPixmap shadow = QPixmap( size*2, size*2 );
+    shadow.fill( Qt::transparent );
+
+    // draw the corner of the window - actually all 4 corners as one circle
+    QLinearGradient lg = QLinearGradient(0.0, size-4.5, 0.0, size+4.5);
+    lg.setColorAt(0.52, light);
+    lg.setColorAt(1.0, dark);
+
+    QPainter p( &shadow );
+    p.setRenderHint( QPainter::Antialiasing );
+    p.setPen( Qt::NoPen );
+
+    if (active)
+    {
+        //---------------------------------------------------------------
+        // Active shadow texture
+
+        QRadialGradient rg( size, size, size );
+        QColor c = color;
+        c.setAlpha( 255 );  rg.setColorAt( 4.4/size, c );
+        c.setAlpha( 220 );  rg.setColorAt( 4.5/size, c );
+        c.setAlpha( 180 );  rg.setColorAt( 5/size, c );
+        c.setAlpha( 25 );  rg.setColorAt( 5.5/size, c );
+        c.setAlpha( 0 );  rg.setColorAt( 6.5/size, c );
+
+        p.setBrush( rg );
+        p.drawRect( shadow.rect() );
+
+        rg = QRadialGradient( size, size, size );
+        c = color;
+        c.setAlpha( 255 );  rg.setColorAt( 4.4/size, c );
+        c = glow;
+        c.setAlpha( 0.58*255 );  rg.setColorAt( 4.5/size, c );
+        c.setAlpha( 0.43*255 );  rg.setColorAt( 5.5/size, c );
+        c.setAlpha( 0.30*255 );  rg.setColorAt( 6.5/size, c );
+        c.setAlpha( 0.22*255 );  rg.setColorAt( 7.5/size, c );
+        c.setAlpha( 0.15*255 );  rg.setColorAt( 8.5/size, c );
+        c.setAlpha( 0.08*255 );  rg.setColorAt( 11.5/size, c );
+        c.setAlpha( 0);  rg.setColorAt( 14.5/size, c );
+        p.setRenderHint( QPainter::Antialiasing );
+        p.setBrush( rg );
+        p.drawRect( shadow.rect() );
+        
+        p.setBrush( Qt::NoBrush );
+        p.setPen(QPen(lg, 0.8));
+        p.drawEllipse(QRectF(size-4, size-4, 8, 8));
+
+        p.end();
+
+        tileSet = new TileSet(shadow, size, size, 1, 1);
+        glowTilesOption_ = opt;
+        if (glowTiles_) {
+            delete glowTiles_;
+        }
+        glowTiles_       = tileSet;
+    } else {
+        //---------------------------------------------------------------
+        // Inactive shadow texture
+
+        QRadialGradient rg = QRadialGradient( size, size+4, size );
+        QColor c = QColor( Qt::black );
+        c.setAlpha( 0.12*255 );  rg.setColorAt( 4.5/size, c );
+        c.setAlpha( 0.11*255 );  rg.setColorAt( 6.6/size, c );
+        c.setAlpha( 0.075*255 );  rg.setColorAt( 8.5/size, c );
+        c.setAlpha( 0.06*255 );  rg.setColorAt( 11.5/size, c );
+        c.setAlpha( 0.035*255 );  rg.setColorAt( 14.5/size, c );
+        c.setAlpha( 0.025*255 );  rg.setColorAt( 17.5/size, c );
+        c.setAlpha( 0.01*255 );  rg.setColorAt( 21.5/size, c );
+        c.setAlpha( 0.0*255 );  rg.setColorAt( 25.5/size, c );
+        p.setRenderHint( QPainter::Antialiasing );
+        p.setPen( Qt::NoPen );
+        p.setBrush( rg );
+        p.drawRect( shadow.rect() );
+
+        rg = QRadialGradient( size, size+2, size );
+        c = QColor( Qt::black );
+        c.setAlpha( 0.25*255 );  rg.setColorAt( 4.5/size, c );
+        c.setAlpha( 0.20*255 );  rg.setColorAt( 5.5/size, c );
+        c.setAlpha( 0.13*255 );  rg.setColorAt( 7.5/size, c );
+        c.setAlpha( 0.06*255 );  rg.setColorAt( 8.5/size, c );
+        c.setAlpha( 0.015*255 );  rg.setColorAt( 11.5/size, c );
+        c.setAlpha( 0.0*255 );  rg.setColorAt( 14.5/size, c );
+        p.setRenderHint( QPainter::Antialiasing );
+        p.setPen( Qt::NoPen );
+        p.setBrush( rg );
+        p.drawRect( shadow.rect() );
+
+        rg = QRadialGradient( size, size+0.2, size );
+        c = color;
+        c = QColor( Qt::black );
+        c.setAlpha( 0.35*255 );  rg.setColorAt( 0/size, c );
+        c.setAlpha( 0.32*255 );  rg.setColorAt( 4.5/size, c );
+        c.setAlpha( 0.22*255 );  rg.setColorAt( 5.0/size, c );
+        c.setAlpha( 0.03*255 );  rg.setColorAt( 5.5/size, c );
+        c.setAlpha( 0.0*255 );  rg.setColorAt( 6.5/size, c );
+        p.setRenderHint( QPainter::Antialiasing );
+        p.setPen( Qt::NoPen );
+        p.setBrush( rg );
+        p.drawRect( shadow.rect() );
+
+        rg = QRadialGradient( size, size, size );
+        c = color;
+        c.setAlpha( 255 );  rg.setColorAt( 4.0/size, c );
+        c.setAlpha( 0 );  rg.setColorAt( 4.01/size, c );
+        p.setRenderHint( QPainter::Antialiasing );
+        p.setPen( Qt::NoPen );
+        p.setBrush( rg );
+        p.drawRect( shadow.rect() );
+
+        // draw the corner of the window - actually all 4 corners as one circle
+        p.setBrush( Qt::NoBrush );
+        p.setPen(QPen(lg, 0.8));
+        p.drawEllipse(QRectF(size-4, size-4, 8, 8));
+
+        p.end();
+
+        tileSet = new TileSet(shadow, size, size, 1, 1);
+        shadowTilesOption_ = opt;
+        if (shadowTiles_) {
+            delete shadowTiles_;
+        }
+        shadowTiles_       = tileSet;
     }
-    return 0;
+    return tileSet;
 }
 
 } //namespace Oxygen
