@@ -65,87 +65,87 @@ bool performTransiencyCheck()
     {
     bool ret = true;
     ClientList clients = Workspace::self()->clients;
-    for( ClientList::ConstIterator it1 = clients.begin();
-         it1 != clients.end();
+    for( ClientList::ConstIterator it1 = clients.constBegin();
+         it1 != clients.constEnd();
          ++it1 )
         {
         if( (*it1)->deleting )
             continue;
         if( (*it1)->in_group == NULL )
             {
-            kdDebug() << "TC: " << *it1 << " in not in a group" << endl;
+            kDebug(1212) << "TC: " << *it1 << " in not in a group" << endl;
             ret = false;
             }
         else if( !(*it1)->in_group->members().contains( *it1 ))
             {
-            kdDebug() << "TC: " << *it1 << " has a group " << (*it1)->in_group << " but group does not contain it" << endl;
+            kDebug(1212) << "TC: " << *it1 << " has a group " << (*it1)->in_group << " but group does not contain it" << endl;
             ret = false;
             }
         if( !(*it1)->isTransient())
             {
             if( !(*it1)->mainClients().isEmpty())
                 {
-                kdDebug() << "TC: " << *it1 << " is not transient, has main clients:" << (*it1)->mainClients() << endl;
+                kDebug(1212) << "TC: " << *it1 << " is not transient, has main clients:" << (*it1)->mainClients() << endl;
                 ret = false;
                 }
             }
         else
             {
             ClientList mains = (*it1)->mainClients();
-            for( ClientList::ConstIterator it2 = mains.begin();
-                 it2 != mains.end();
+            for( ClientList::ConstIterator it2 = mains.constBegin();
+                 it2 != mains.constEnd();
                  ++it2 )
                 {
                 if( transiencyCheckNonExistent
                     && !Workspace::self()->clients.contains( *it2 )
                     && !Workspace::self()->desktops.contains( *it2 ))
                     {
-                    kDebug() << "TC:" << *it1 << " has non-existent main client ";
-                    kDebug() << "TC2:" << *it2; // this may crash
+                    kDebug(1212) << "TC:" << *it1 << " has non-existent main client ";
+                    kDebug(1212) << "TC2:" << *it2; // this may crash
                     ret = false;
                     continue;
                     }
                 if( !(*it2)->transients_list.contains( *it1 ))
                     {
-                    kdDebug() << "TC:" << *it1 << " has main client " << *it2 << " but main client does not have it as a transient" << endl;
+                    kdDebug(1212) << "TC:" << *it1 << " has main client " << *it2 << " but main client does not have it as a transient" << endl;
                     ret = false;
                     }
                 }
             }
         ClientList trans = (*it1)->transients_list;
-        for( ClientList::ConstIterator it2 = trans.begin();
-             it2 != trans.end();
+        for( ClientList::ConstIterator it2 = trans.constBegin();
+             it2 != trans.constEnd();
              ++it2 )
             {
             if( transiencyCheckNonExistent
                 && !Workspace::self()->clients.contains( *it2 )
                 && !Workspace::self()->desktops.contains( *it2 ))
                 {
-                kDebug() << "TC:" << *it1 << " has non-existent transient ";
-                kDebug() << "TC2:" << *it2; // this may crash
+                kDebug(1212) << "TC:" << *it1 << " has non-existent transient ";
+                kDebug(1212) << "TC2:" << *it2; // this may crash
                 ret = false;
                 continue;
                 }
             if( !(*it2)->mainClients().contains( *it1 ))
                 {
-                kdDebug() << "TC:" << *it1 << " has transient " << *it2 << " but transient does not have it as a main client" << endl;
+                kdDebug(1212) << "TC:" << *it1 << " has transient " << *it2 << " but transient does not have it as a main client" << endl;
                 ret = false;
                 }
             }
         }
     GroupList groups = Workspace::self()->groups;
-    for( GroupList::ConstIterator it1 = groups.begin();
-         it1 != groups.end();
+    for( GroupList::ConstIterator it1 = groups.constBegin();
+         it1 != groups.constEnd();
          ++it1 )
         {
         ClientList members = (*it1)->members();
-        for( ClientList::ConstIterator it2 = members.begin();
-             it2 != members.end();
+        for( ClientList::ConstIterator it2 = members.constBegin();
+             it2 != members.constEnd();
              ++it2 )
             {
             if( (*it2)->in_group != *it1 )
                 {
-                kdDebug() << "TC: Group " << *it1 << " contains client " << *it2 << " but client is not in that group" << endl;
+                kDebug(1212) << "TC: Group " << *it1 << " contains client " << *it2 << " but client is not in that group" << endl;
                 ret = false;
                 }
             }
@@ -173,9 +173,9 @@ static void checkTransiency()
         {
         if( !performTransiencyCheck())
             {
-            kdDebug() << "BT:" << transiencyCheckStartBt << endl;
-            kdDebug() << "CLIENT:" << transiencyCheckClient << endl;
-            assert( false );
+            kDebug(1212) << "BT:" << transiencyCheckStartBt << endl;
+            kDebug(1212) << "CLIENT:" << transiencyCheckClient << endl;
+            abort();
             }
         transiencyCheckNonExistent = false;
         }
@@ -221,7 +221,7 @@ Group::Group( Window leader_P, Workspace* workspace_P )
         {
         leader_client = workspace_P->findClient( WindowMatchPredicate( leader_P ));
         unsigned long properties[ 2 ] = { 0, NET::WM2StartupId };
-        leader_info = new NETWinInfo( display(), leader_P, rootWindow(),
+        leader_info = new NETWinInfo2( display(), leader_P, rootWindow(),
             properties, 2 );
         }
     effect_group = new EffectWindowGroupImpl( this );
@@ -264,15 +264,15 @@ void Group::addMember( Client* member_P )
     {
     TRANSIENCY_CHECK( member_P );
     _members.append( member_P );
-//    kDebug() << "GROUPADD:" << this << ":" << member_P;
-//    kDebug() << kBacktrace();
+//    kDebug(1212) << "GROUPADD:" << this << ":" << member_P;
+//    kDebug(1212) << kBacktrace();
     }
 
 void Group::removeMember( Client* member_P )
     {
     TRANSIENCY_CHECK( member_P );
-//    kDebug() << "GROUPREMOVE:" << this << ":" << member_P;
-//    kDebug() << kBacktrace();
+//    kDebug(1212) << "GROUPREMOVE:" << this << ":" << member_P;
+//    kDebug(1212) << kBacktrace();
     Q_ASSERT( _members.contains( member_P ));
     _members.removeAll( member_P );
 // there are cases when automatic deleting of groups must be delayed,
@@ -329,8 +329,8 @@ void Group::getIcons()
 Group* Workspace::findGroup( Window leader ) const
     {
     assert( leader != None );
-    for( GroupList::ConstIterator it = groups.begin();
-         it != groups.end();
+    for( GroupList::ConstIterator it = groups.constBegin();
+         it != groups.constEnd();
          ++it )
         if( (*it)->leader() == leader )
             return *it;
@@ -343,8 +343,8 @@ Group* Workspace::findClientLeaderGroup( const Client* c ) const
     {
     TRANSIENCY_CHECK( c );
     Group* ret = NULL;
-    for( ClientList::ConstIterator it = clients.begin();
-         it != clients.end();
+    for( ClientList::ConstIterator it = clients.constBegin();
+         it != clients.constEnd();
          ++it )
         {
         if( *it == c )
@@ -380,31 +380,41 @@ void Workspace::updateMinimizedOfTransients( Client* c )
     // if mainwindow is minimized or shaded, minimize transients too
     if ( c->isMinimized() || c->isShade() )
         {
-        for( ClientList::ConstIterator it = c->transients().begin();
-             it != c->transients().end();
+        for( ClientList::ConstIterator it = c->transients().constBegin();
+             it != c->transients().constEnd();
              ++it )
             {
             if( !(*it)->isMinimized()
                  && !(*it)->isTopMenu() ) // topmenus are not minimized, they're hidden
                 {
-                (*it)->minimize( true ); // avoid animation
+                (*it)->minimize();
                 updateMinimizedOfTransients( (*it) );
                 }
             }
+        if( c->isModal()) // if a modal dialog is minimized, minimize its mainwindow too
+            {
+            foreach( Client* c2, c->mainClients())
+                c2->minimize();
+            }
         }
     else
         { // else unmiminize the transients
-        for( ClientList::ConstIterator it = c->transients().begin();
-             it != c->transients().end();
+        for( ClientList::ConstIterator it = c->transients().constBegin();
+             it != c->transients().constEnd();
              ++it )
             {
             if( (*it)->isMinimized()
                 && !(*it)->isTopMenu())
                 {
-                (*it)->unminimize( true ); // avoid animation
+                (*it)->unminimize();
                 updateMinimizedOfTransients( (*it) );
                 }
             }
+        if( c->isModal())
+            {
+            foreach( Client* c2, c->mainClients())
+                c2->unminimize();
+            }
         }
     }
 
@@ -414,8 +424,8 @@ void Workspace::updateMinimizedOfTransients( Client* c )
  */
 void Workspace::updateOnAllDesktopsOfTransients( Client* c )
     {
-    for( ClientList::ConstIterator it = c->transients().begin();
-         it != c->transients().end();
+    for( ClientList::ConstIterator it = c->transients().constBegin();
+         it != c->transients().constEnd();
          ++it)
         {
         if( (*it)->isOnAllDesktops() != c->isOnAllDesktops())
@@ -427,8 +437,8 @@ void Workspace::updateOnAllDesktopsOfTransients( Client* c )
 void Workspace::checkTransients( Window w )
     {
     TRANSIENCY_CHECK( NULL );
-    for( ClientList::ConstIterator it = clients.begin();
-         it != clients.end();
+    for( ClientList::ConstIterator it = clients.constBegin();
+         it != clients.constEnd();
          ++it )
         (*it)->checkTransient( w );
     }
@@ -639,8 +649,8 @@ void Client::removeFromMainClients()
         transientFor()->removeTransient( this );
     if( groupTransient())
         {
-        for( ClientList::ConstIterator it = group()->members().begin();
-             it != group()->members().end();
+        for( ClientList::ConstIterator it = group()->members().constBegin();
+             it != group()->members().constEnd();
              ++it )
             (*it)->removeTransient( this );
         }
@@ -653,50 +663,50 @@ void Client::removeFromMainClients()
 void Client::cleanGrouping()
     {
     TRANSIENCY_CHECK( this );
-//    kDebug() << "CLEANGROUPING:" << this;
+//    kDebug(1212) << "CLEANGROUPING:" << this;
 //    for( ClientList::ConstIterator it = group()->members().begin();
 //         it != group()->members().end();
 //         ++it )
-//        kDebug() << "CL:" << *it;
+//        kDebug(1212) << "CL:" << *it;
 //    ClientList mains;
 //    mains = mainClients();
 //    for( ClientList::ConstIterator it = mains.begin();
 //         it != mains.end();
 //         ++it )
-//        kDebug() << "MN:" << *it;
+//        kDebug(1212) << "MN:" << *it;
     removeFromMainClients();
-//    kDebug() << "CLEANGROUPING2:" << this;
+//    kDebug(1212) << "CLEANGROUPING2:" << this;
 //    for( ClientList::ConstIterator it = group()->members().begin();
 //         it != group()->members().end();
 //         ++it )
-//        kDebug() << "CL2:" << *it;
+//        kDebug(1212) << "CL2:" << *it;
 //    mains = mainClients();
 //    for( ClientList::ConstIterator it = mains.begin();
 //         it != mains.end();
 //         ++it )
-//        kDebug() << "MN2:" << *it;
-    for( ClientList::ConstIterator it = transients_list.begin();
-         it != transients_list.end();
+//        kDebug(1212) << "MN2:" << *it;
+    for( ClientList::ConstIterator it = transients_list.constBegin();
+         it != transients_list.constEnd();
          )
         {
         if( (*it)->transientFor() == this )
             {
             removeTransient( *it );
-            it = transients_list.begin(); // restart, just in case something more has changed with the list
+            it = transients_list.constBegin(); // restart, just in case something more has changed with the list
             }
         else
             ++it;
         }
-//    kDebug() << "CLEANGROUPING3:" << this;
+//    kDebug(1212) << "CLEANGROUPING3:" << this;
 //    for( ClientList::ConstIterator it = group()->members().begin();
 //         it != group()->members().end();
 //         ++it )
-//        kDebug() << "CL3:" << *it;
+//        kDebug(1212) << "CL3:" << *it;
 //    mains = mainClients();
 //    for( ClientList::ConstIterator it = mains.begin();
 //         it != mains.end();
 //         ++it )
-//        kDebug() << "MN3:" << *it;
+//        kDebug(1212) << "MN3:" << *it;
     // HACK
     // removeFromMainClients() did remove 'this' from transient
     // lists of all group members, but then made windows that
@@ -705,15 +715,15 @@ void Client::cleanGrouping()
     ClientList group_members = group()->members();
     group()->removeMember( this );
     in_group = NULL;
-    for( ClientList::ConstIterator it = group_members.begin();
-         it != group_members.end();
+    for( ClientList::ConstIterator it = group_members.constBegin();
+         it != group_members.constEnd();
          ++it )
         (*it)->removeTransient( this );
-//    kDebug() << "CLEANGROUPING4:" << this;
+//    kDebug(1212) << "CLEANGROUPING4:" << this;
 //    for( ClientList::ConstIterator it = group_members.begin();
 //         it != group_members.end();
 //         ++it )
-//        kDebug() << "CL4:" << *it;
+//        kDebug(1212) << "CL4:" << *it;
     }
 
 // Make sure that no group transient is considered transient
@@ -723,14 +733,14 @@ void Client::cleanGrouping()
 void Client::checkGroupTransients()
     {
     TRANSIENCY_CHECK( this );
-    for( ClientList::ConstIterator it1 = group()->members().begin();
-         it1 != group()->members().end();
+    for( ClientList::ConstIterator it1 = group()->members().constBegin();
+         it1 != group()->members().constEnd();
          ++it1 )
         {
         if( !(*it1)->groupTransient()) // check all group transients in the group
             continue;                  // TODO optimize to check only the changed ones?
-        for( ClientList::ConstIterator it2 = group()->members().begin();
-             it2 != group()->members().end();
+        for( ClientList::ConstIterator it2 = group()->members().constBegin();
+             it2 != group()->members().constEnd();
              ++it2 ) // group transients can be transient only for others in the group,
             {        // so don't make them transient for the ones that are transient for it
             if( *it1 == *it2 )
@@ -756,8 +766,8 @@ void Client::checkGroupTransients()
             // transient for it - the indirect transiency actually shouldn't break anything,
             // but it can lead to exponentially expensive operations (#95231)
             // TODO this is pretty slow as well
-            for( ClientList::ConstIterator it3 = group()->members().begin();
-                 it3 != group()->members().end();
+            for( ClientList::ConstIterator it3 = group()->members().constBegin();
+                 it3 != group()->members().constEnd();
                  ++it3 )
                 {
                 if( *it1 == *it2 || *it2 == *it3 || *it1 == *it3 )
@@ -785,10 +795,12 @@ Window Client::verifyTransientFor( Window new_transient_for, bool defined )
     if( isSplash() && new_transient_for == None )
         new_transient_for = rootWindow();
     if( new_transient_for == None )
+        {
         if( defined ) // sometimes WM_TRANSIENT_FOR is set to None, instead of root window
             new_property_value = new_transient_for = rootWindow();
         else
             return None;
+        }
     if( new_transient_for == window()) // pointing to self
         { // also fix the property itself
         kWarning( 1216 ) << "Client " << this << " has WM_TRANSIENT_FOR poiting to itself." ;
@@ -859,19 +871,19 @@ void Client::addTransient( Client* cl )
     transients_list.append( cl );
     if( workspace()->mostRecentlyActivatedClient() == this && cl->isModal())        
         check_active_modal = true;
-//    kDebug() << "ADDTRANS:" << this << ":" << cl;
-//    kDebug() << kBacktrace();
+//    kDebug(1212) << "ADDTRANS:" << this << ":" << cl;
+//    kDebug(1212) << kBacktrace();
 //    for( ClientList::ConstIterator it = transients_list.begin();
 //         it != transients_list.end();
 //         ++it )
-//        kDebug() << "AT:" << (*it);
+//        kDebug(1212) << "AT:" << (*it);
     }
 
 void Client::removeTransient( Client* cl )
     {
     TRANSIENCY_CHECK( this );
-//    kDebug() << "REMOVETRANS:" << this << ":" << cl;
-//    kDebug() << kBacktrace();
+//    kDebug(1212) << "REMOVETRANS:" << this << ":" << cl;
+//    kDebug(1212) << kBacktrace();
     transients_list.removeAll( cl );
     // cl is transient for this, but this is going away
     // make cl group transient
@@ -928,8 +940,8 @@ bool Client::hasTransientInternal( const Client* cl, bool indirect, ConstClientL
     if( set.contains( this ))
         return false;
     set.append( this );
-    for( ClientList::ConstIterator it = transients().begin();
-         it != transients().end();
+    for( ClientList::ConstIterator it = transients().constBegin();
+         it != transients().constEnd();
          ++it )
         if( (*it)->hasTransientInternal( cl, indirect, set ))
             return true;
@@ -943,8 +955,8 @@ ClientList Client::mainClients() const
     if( transientFor() != NULL )
         return ClientList() << const_cast< Client* >( transientFor());
     ClientList result;
-    for( ClientList::ConstIterator it = group()->members().begin();
-         it != group()->members().end();
+    for( ClientList::ConstIterator it = group()->members().constBegin();
+         it != group()->members().constEnd();
          ++it )
         if((*it)->hasTransient( this, false ))
             result.append( *it );
@@ -961,8 +973,8 @@ ClientList Client::allMainClients() const
 
 Client* Client::findModal( bool allow_itself )
     {
-    for( ClientList::ConstIterator it = transients().begin();
-         it != transients().end();
+    for( ClientList::ConstIterator it = transients().constBegin();
+         it != transients().constEnd();
          ++it )
         if( Client* ret = (*it)->findModal( true ))
             return ret;
@@ -1038,7 +1050,7 @@ void Client::checkGroup( Group* set_group, bool force )
             }
         else // Not transient without a group, put it in its client leader group.
             { // This might be stupid if grouping was used for e.g. taskbar grouping
-              // or minimizing together the whole group, but as long as its used
+              // or minimizing together the whole group, but as long as it is used
               // only for dialogs it's better to keep windows from one app in one group.
             Group* new_group = workspace()->findClientLeaderGroup( this );
             if( in_group != NULL && in_group != new_group )
@@ -1071,14 +1083,14 @@ void Client::checkGroup( Group* set_group, bool force )
             // no longer transient for ones in the old group
             if( old_group != NULL )
                 {
-                for( ClientList::ConstIterator it = old_group->members().begin();
-                     it != old_group->members().end();
+                for( ClientList::ConstIterator it = old_group->members().constBegin();
+                     it != old_group->members().constEnd();
                      ++it )
                     (*it)->removeTransient( this );
                 }
             // and make transient for all in the new group
-            for( ClientList::ConstIterator it = group()->members().begin();
-                 it != group()->members().end();
+            for( ClientList::ConstIterator it = group()->members().constBegin();
+                 it != group()->members().constEnd();
                  ++it )
                 {
                 if( *it == this )
@@ -1088,8 +1100,8 @@ void Client::checkGroup( Group* set_group, bool force )
             }
         // group transient splashscreens should be transient even for windows
         // in group mapped later
-        for( ClientList::ConstIterator it = group()->members().begin();
-             it != group()->members().end();
+        for( ClientList::ConstIterator it = group()->members().constBegin();
+             it != group()->members().constEnd();
              ++it )
             {
             if( !(*it)->isSplash())
