@@ -30,9 +30,11 @@
 #include <QPixmap>
 #include <QVBoxLayout>
 #include <QHBoxLayout>
+#include <QFormLayout>
 
 #include <kconfig.h>
 #include <kdialog.h>
+#include <kdebug.h>
 #include <kglobalsettings.h>
 #include <kcolorscheme.h>
 #include <kseparator.h>
@@ -139,8 +141,8 @@ KTitleBarActionsConfig::KTitleBarActionsConfig (bool _standAlone, KConfig *_conf
   QGridLayout *grid;
   QGroupBox *box;
   QLabel *label;
-  QString strMouseButton1, strMouseButton3, strMouseWheel;
-  QString txtButton1, txtButton3, txtButton4;
+  QString strMouseButton1, strMouseButton3;
+  QString txtButton1, txtButton3;
   QStringList items;
   bool leftHandedMouse = ( KGlobalSettings::mouseSettings().handed == KGlobalSettings::KMouseSettings::LeftHanded);
 
@@ -150,13 +152,8 @@ KTitleBarActionsConfig::KTitleBarActionsConfig (bool _standAlone, KConfig *_conf
 
 /** Titlebar doubleclick ************/
 
-  QHBoxLayout *hlayout = new QHBoxLayout();
-  layout->addLayout( hlayout );
-
-  label = new QLabel(i18n("&Titlebar double-click:"), this);
-  hlayout->addWidget(label);
-  label->setWhatsThis( i18n("Here you can customize mouse click behavior when double clicking on the"
-    " titlebar of a window.") );
+  QGridLayout *gLayout = new QGridLayout();
+  layout->addLayout( gLayout );
 
   QComboBox* combo = new QComboBox(this);
   combo->addItem(i18n("Maximize"));
@@ -167,23 +164,19 @@ KTitleBarActionsConfig::KTitleBarActionsConfig (bool _standAlone, KConfig *_conf
   combo->addItem(i18n("Lower"));
   combo->addItem(i18n("On All Desktops"));
   combo->addItem(i18n("Nothing"));
-  combo->setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed));
+  combo->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
   connect(combo, SIGNAL(activated(int)), SLOT(changed()));
-  hlayout->addWidget(combo);
   coTiDbl = combo;
   combo->setWhatsThis( i18n("Behavior on <em>double</em> click into the titlebar."));
 
+  combo->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
+  label = new QLabel(i18n("&Titlebar double-click:"), this);
+  label->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
   label->setBuddy(combo);
+  gLayout->addWidget(label, 0, 0);
+  gLayout->addWidget(combo, 0, 1);
 
 /** Mouse Wheel Events  **************/
-  QHBoxLayout *hlayoutW = new QHBoxLayout();
-  layout->addLayout( hlayoutW );
-  strMouseWheel = i18n("Titlebar wheel event:");
-  label = new QLabel(strMouseWheel, this);
-  hlayoutW->addWidget(label);
-  txtButton4 = i18n("Handle mouse wheel events");
-  label->setWhatsThis( txtButton4);
-  
   // Titlebar and frame mouse Wheel  
   QComboBox* comboW = new QComboBox(this);
   comboW->addItem(i18n("Raise/Lower"));
@@ -193,13 +186,18 @@ KTitleBarActionsConfig::KTitleBarActionsConfig (bool _standAlone, KConfig *_conf
   comboW->addItem(i18n("Move to Previous/Next Desktop"));  
   comboW->addItem(i18n("Change Opacity"));  
   comboW->addItem(i18n("Nothing"));  
-  comboW->setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed));
+  comboW->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
   connect(comboW, SIGNAL(activated(int)), SLOT(changed()));
-  hlayoutW->addWidget(comboW);
   coTiAct4 = comboW;
-  comboW->setWhatsThis( txtButton4);
+  comboW->setWhatsThis( i18n("Handle mouse wheel events"));
+
+  comboW->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
+  label = new QLabel(i18n("Titlebar wheel event:"), this);
+  label->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
   label->setBuddy(comboW);
-  
+  gLayout->addWidget(label, 1, 0);
+  gLayout->addWidget(comboW, 1, 1);
+
 /** Titlebar and frame  **************/
 
   box = new QGroupBox(i18n("Titlebar && Frame"), this);
@@ -228,15 +226,18 @@ KTitleBarActionsConfig::KTitleBarActionsConfig (bool _standAlone, KConfig *_conf
   }
 
   label = new QLabel(strMouseButton1, box);
+  label->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
   grid->addWidget(label, 1, 0);
   label->setWhatsThis( txtButton1);
 
   label = new QLabel(i18n("Middle button:"), box);
+  label->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
   grid->addWidget(label, 2, 0);
   label->setWhatsThis( i18n("In this row you can customize middle click behavior when clicking into"
     " the titlebar or the frame.") );
 
   label = new QLabel(strMouseButton3, box);
+  label->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
   grid->addWidget(label, 3, 0);
   label->setWhatsThis( txtButton3);
 
@@ -249,6 +250,7 @@ KTitleBarActionsConfig::KTitleBarActionsConfig (bool _standAlone, KConfig *_conf
 
   // Titlebar and frame, active, mouse button 1
   combo = new QComboBox(box);
+  combo->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
   grid->addWidget(combo, 1, 1);
   combo->addItem(i18n("Raise"));
   combo->addItem(i18n("Lower"));
@@ -279,6 +281,7 @@ KTitleBarActionsConfig::KTitleBarActionsConfig (bool _standAlone, KConfig *_conf
         << i18n("Shade");
 
   combo = new QComboBox(box);
+  combo->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
   grid->addWidget(combo, 2, 1);
   combo->addItems(items);
   connect(combo, SIGNAL(activated(int)), SLOT(changed()));
@@ -287,6 +290,7 @@ KTitleBarActionsConfig::KTitleBarActionsConfig (bool _standAlone, KConfig *_conf
 
   // Titlebar and frame, active, mouse button 3
   combo = new QComboBox(box);
+  combo->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
   grid->addWidget(combo, 3, 1);
   combo->addItems(items);
   connect(combo, SIGNAL(activated(int)), SLOT(changed()));
@@ -319,6 +323,7 @@ KTitleBarActionsConfig::KTitleBarActionsConfig (bool _standAlone, KConfig *_conf
          << i18n("Nothing");
 
   combo = new QComboBox(box);
+  combo->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
   grid->addWidget(combo, 1, 2);
   combo->addItems(items);
   connect(combo, SIGNAL(activated(int)), SLOT(changed()));
@@ -326,6 +331,7 @@ KTitleBarActionsConfig::KTitleBarActionsConfig (bool _standAlone, KConfig *_conf
   combo->setWhatsThis( txtButton1);
 
   combo = new QComboBox(box);
+  combo->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
   grid->addWidget(combo, 2, 2);
   combo->addItems(items);
   connect(combo, SIGNAL(activated(int)), SLOT(changed()));
@@ -333,6 +339,7 @@ KTitleBarActionsConfig::KTitleBarActionsConfig (bool _standAlone, KConfig *_conf
   combo->setWhatsThis( i18n("Behavior on <em>middle</em> click into the titlebar or frame of an <em>inactive</em> window."));
 
   combo = new QComboBox(box);
+  combo->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
   grid->addWidget(combo, 3, 2);
   combo->addItems(items);
   connect(combo, SIGNAL(activated(int)), SLOT(changed()));
@@ -347,7 +354,7 @@ KTitleBarActionsConfig::KTitleBarActionsConfig (bool _standAlone, KConfig *_conf
   box->setWhatsThis(
     i18n("Here you can customize behavior when clicking on the maximize button.") );
 
-  hlayout = new QHBoxLayout(box);
+  QHBoxLayout* hlayout = new QHBoxLayout(box);
   hlayout->setMargin(KDialog::marginHint());
   hlayout->setSpacing(KDialog::spacingHint());
 
@@ -375,7 +382,7 @@ KTitleBarActionsConfig::KTitleBarActionsConfig (bool _standAlone, KConfig *_conf
     QLabel * label = new QLabel(strMouseButton[b], box);
     hlayout->addWidget(label);
     label->setWhatsThis(    txtButton[b] );
-    label   ->setSizePolicy( QSizePolicy( QSizePolicy::Fixed, QSizePolicy::Minimum ));
+    label->setSizePolicy( QSizePolicy::Fixed, QSizePolicy::Minimum );
 
     coMax[b] = new ToolTipComboBox(box, tbl_Max);
     hlayout->addWidget(coMax[b]);
@@ -383,7 +390,7 @@ KTitleBarActionsConfig::KTitleBarActionsConfig (bool _standAlone, KConfig *_conf
     connect(coMax[b], SIGNAL(activated(int)), SLOT(changed()));
     connect(coMax[b], SIGNAL(activated(int)), coMax[b], SLOT(changed()));
     coMax[b]->setWhatsThis( txtButton[b] );
-    coMax[b]->setSizePolicy( QSizePolicy( QSizePolicy::Fixed, QSizePolicy::Minimum ));
+    coMax[b]->setSizePolicy( QSizePolicy::Fixed, QSizePolicy::Minimum );
   }
 
   connect(KGlobalSettings::self(), SIGNAL(kdisplayPaletteChanged()), SLOT(paletteChanged()));
@@ -439,6 +446,12 @@ const char* const tbl_Win[] = {
     "Activate",
     "Activate and raise",
     "" };
+    
+const char* const tbl_WinWheel[] = {
+    "Scroll",
+    "Activate and scroll",
+    "Activate, raise and scroll",
+    "" };
 
 const char* const tbl_AllKey[] = {
     "Meta",
@@ -609,15 +622,13 @@ void KTitleBarActionsConfig::defaults()
 KWindowActionsConfig::KWindowActionsConfig (bool _standAlone, KConfig *_config, const KComponentData &inst, QWidget * parent)
   : KCModule(inst, parent), config(_config), standAlone(_standAlone)
 {
-  QString strWin1, strWin2, strWin3, strAllKey, strAll1, strAll2, strAll3, strAllW;
-  QGridLayout *grid;
+  QString strWin1, strWin2, strWin3, strWinWheel, strAllKey, strAll1, strAll2, strAll3, strAllW;
   QGroupBox *box;
-  QLabel *label;
-  QString strMouseButton1, strMouseButton3;
-  QString txtButton1, txtButton3;
+  QString strMouseButton1, strMouseButton2, strMouseButton3, strMouseWheel;
+  QString txtButton1, txtButton2, txtButton3, txtWheel;
   QStringList items;
   bool leftHandedMouse = ( KGlobalSettings::mouseSettings().handed == KGlobalSettings::KMouseSettings::LeftHanded);
-  
+
   QVBoxLayout *layout = new QVBoxLayout(this);
   layout->setMargin(0);
   layout->setSpacing(KDialog::spacingHint());
@@ -630,17 +641,23 @@ KWindowActionsConfig::KWindowActionsConfig (bool _standAlone, KConfig *_config,
   box->setWhatsThis( i18n("Here you can customize mouse click behavior when clicking on an inactive"
                              " inner window ('inner' means: not titlebar, not frame).") );
 
-  grid = new QGridLayout(box);
-  grid->setMargin(KDialog::marginHint());
-  grid->setSpacing(KDialog::spacingHint());
+  QGridLayout *gridLayout = new QGridLayout(box);
+  gridLayout->setMargin(KDialog::marginHint());
+  gridLayout->setSpacing(KDialog::spacingHint());
 
   strMouseButton1 = i18n("Left button:");
   txtButton1 = i18n("In this row you can customize left click behavior when clicking into"
      " the titlebar or the frame.");
 
+  strMouseButton2 = i18n("Middle button:");
+  txtButton2 = i18n("In this row you can customize middle click behavior when clicking into"
+     " the titlebar or the frame." );
+
   strMouseButton3 = i18n("Right button:");
   txtButton3 = i18n("In this row you can customize right click behavior when clicking into"
      " the titlebar or the frame." );
+     
+  strMouseWheel = i18n("Wheel");
 
   if ( leftHandedMouse )
   {
@@ -651,26 +668,18 @@ KWindowActionsConfig::KWindowActionsConfig (bool _standAlone, KConfig *_config,
   strWin1 = i18n("In this row you can customize left click behavior when clicking into"
      " an inactive inner window ('inner' means: not titlebar, not frame).");
 
+  strWin2 = i18n("In this row you can customize middle click behavior when clicking into"
+     " an inactive inner window ('inner' means: not titlebar, not frame).");
+
   strWin3 = i18n("In this row you can customize right click behavior when clicking into"
      " an inactive inner window ('inner' means: not titlebar, not frame).");
+     
+  strWinWheel = i18n("In this row you can customize behavior when scrolling into"
+     " an inactive inner window ('inner' means: not titlebar, not frame).");
 
   // Be nice to lefties
   if ( leftHandedMouse ) qSwap(strWin1, strWin3);
 
-  label = new QLabel(strMouseButton1, box);
-  grid->addWidget(label, 0, 0);
-  label->setWhatsThis( strWin1 );
-
-  label = new QLabel(i18n("Middle button:"), box);
-  grid->addWidget(label, 1, 0);
-  strWin2 = i18n("In this row you can customize middle click behavior when clicking into"
-     " an inactive inner window ('inner' means: not titlebar, not frame).");
-  label->setWhatsThis( strWin2 );
-
-  label = new QLabel(strMouseButton3, box);
-  grid->addWidget(label, 2, 0);
-  label->setWhatsThis( strWin3 );
-
   items.clear();
   items   << i18n("Activate, Raise & Pass Click")
           << i18n("Activate & Pass Click")
@@ -678,25 +687,59 @@ KWindowActionsConfig::KWindowActionsConfig (bool _standAlone, KConfig *_config,
           << i18n("Activate & Raise");
 
   QComboBox* combo = new QComboBox(box);
-  grid->addWidget(combo, 0, 1);
+  coWin1 = combo;
   combo->addItems(items);
   connect(combo, SIGNAL(activated(int)), SLOT(changed()));
-  coWin1 = combo;
   combo->setWhatsThis( strWin1 );
+  combo->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
+  QLabel* label = new QLabel(strMouseButton1, this);
+  label->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
+  label->setBuddy(combo);
+  gridLayout->addWidget(label, 0, 0);
+  gridLayout->addWidget(combo, 0, 1);
 
   combo = new QComboBox(box);
-  grid->addWidget(combo, 1, 1);
   combo->addItems(items);
   connect(combo, SIGNAL(activated(int)), SLOT(changed()));
   coWin2 = combo;
   combo->setWhatsThis( strWin2 );
+  combo->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
+  label = new QLabel(strMouseButton2, this);
+  label->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
+  label->setBuddy(combo);
+  gridLayout->addWidget(label, 1, 0);
+  gridLayout->addWidget(combo, 1, 1);
 
   combo = new QComboBox(box);
-  grid->addWidget(combo, 2, 1);
   combo->addItems(items);
   connect(combo, SIGNAL(activated(int)), SLOT(changed()));
   coWin3 = combo;
   combo->setWhatsThis( strWin3 );
+  combo->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
+  label = new QLabel(strMouseButton3, this);
+  label->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
+  label->setBuddy(combo);
+  gridLayout->addWidget(label, 2, 0);
+  gridLayout->addWidget(combo, 2, 1);
+  
+  items.clear();
+  items   << i18n("Scroll")
+          << i18n("Activate & Scroll")
+          << i18n("Activate, Raise & Scroll");
+  
+  combo = new QComboBox(box);
+  combo->addItems(items);
+  connect(combo, SIGNAL(activated(int)), SLOT(changed()));
+  coWinWheel = combo;
+  combo->setWhatsThis( strWinWheel );
+  combo->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
+  label = new QLabel(strMouseWheel, this);
+  label->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
+  label->setBuddy(combo);
+  gridLayout->addWidget(label, 3, 0);
+  gridLayout->addWidget(combo, 3, 1);
+  
+  
 
 
 /** Inner window, titlebar and frame **************/
@@ -707,24 +750,28 @@ KWindowActionsConfig::KWindowActionsConfig (bool _standAlone, KConfig *_config,
   box->setWhatsThis( i18n("Here you can customize KDE's behavior when clicking somewhere into"
                              " a window while pressing a modifier key."));
 
-  grid = new QGridLayout(box);
-  grid->setMargin(KDialog::marginHint());
-  grid->setSpacing(KDialog::spacingHint());
+  QHBoxLayout* innerLay=new QHBoxLayout(box);
+  innerLay->setContentsMargins(0,0,0,0);
+  innerLay->setSpacing(KDialog::spacingHint());
+  QHBoxLayout* fLay=new QHBoxLayout;
+  fLay->setMargin(KDialog::marginHint());
+  fLay->setSpacing(KDialog::spacingHint());
+  gridLayout = new QGridLayout;
+  gridLayout->setMargin(KDialog::marginHint());
+  gridLayout->setSpacing(KDialog::spacingHint());
+  innerLay->addLayout(fLay);
+  innerLay->addLayout(gridLayout);
 
   // Labels
-  label = new QLabel(i18n("Modifier key:"), box);
-  grid->addWidget(label, 0, 0);
-
-  strAllKey = i18n("Here you select whether holding the Meta key or Alt key "
-    "will allow you to perform the following actions.");
-  label->setWhatsThis( strAllKey );
-
-
-  strMouseButton1 = i18n("Modifier key + left button:");
+  strMouseButton1 = i18n("Left button:");
   strAll1 = i18n("In this row you can customize left click behavior when clicking into"
                  " the titlebar or the frame.");
 
-  strMouseButton3 = i18n("Modifier key + right button:");
+  strMouseButton2 = i18n("Middle button:");
+  strAll2 = i18n("In this row you can customize middle click behavior when clicking into"
+                 " the titlebar or the frame." );
+
+  strMouseButton3 = i18n("Right button:");
   strAll3 = i18n("In this row you can customize right click behavior when clicking into"
                  " the titlebar or the frame." );
 
@@ -734,34 +781,22 @@ KWindowActionsConfig::KWindowActionsConfig (bool _standAlone, KConfig *_config,
      qSwap(strAll1, strAll3);
   }
 
-  label = new QLabel(strMouseButton1, box);
-  grid->addWidget(label, 1, 0);
-  label->setWhatsThis( strAll1);
-
-  label = new QLabel(i18n("Modifier key + middle button:"), box);
-  grid->addWidget(label, 2, 0);
-  strAll2 = i18n("Here you can customize KDE's behavior when middle clicking into a window"
-                 " while pressing the modifier key.");
-  label->setWhatsThis( strAll2 );
-
-  label = new QLabel(strMouseButton3, box);
-  grid->addWidget(label, 3, 0);
-  label->setWhatsThis( strAll3);
-
-  label = new QLabel(i18n("Modifier key + mouse wheel:"), box);
-  grid->addWidget(label, 4, 0);
-  strAllW = i18n("Here you can customize KDE's behavior when scrolling with the mouse wheel"
-      "  in a window while pressing the modifier key.");
-  label->setWhatsThis( strAllW);
-
   // Combo's
   combo = new QComboBox(box);
-  grid->addWidget(combo, 0, 1);
   combo->addItem(i18n("Meta"));
   combo->addItem(i18n("Alt"));
   connect(combo, SIGNAL(activated(int)), SLOT(changed()));
   coAllKey = combo;
-  combo->setWhatsThis( strAllKey );
+  combo->setWhatsThis( i18n("Here you select whether holding the Meta key or Alt key "
+                            "will allow you to perform the following actions.") );
+  combo->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
+  label = new QLabel(i18n("Modifier key:"), this);
+  label->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
+  label->setBuddy(combo);
+  fLay->addWidget(label);
+  fLay->addWidget(combo);
+  fLay->addWidget(new QLabel("   + ", this));
+
 
   items.clear();
   items << i18n("Move")
@@ -774,28 +809,44 @@ KWindowActionsConfig::KWindowActionsConfig (bool _standAlone, KConfig *_config,
         << i18n("Nothing");
 
   combo = new QComboBox(box);
-  grid->addWidget(combo, 1, 1);
   combo->addItems(items);
   connect(combo, SIGNAL(activated(int)), SLOT(changed()));
   coAll1 = combo;
   combo->setWhatsThis( strAll1 );
+  combo->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
+  label = new QLabel(strMouseButton1, this);
+  label->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
+  label->setBuddy(combo);
+  gridLayout->addWidget(label, 0, 0);
+  gridLayout->addWidget(combo, 0, 1);
+
 
   combo = new QComboBox(box);
-  grid->addWidget(combo, 2, 1);
   combo->addItems(items);
   connect(combo, SIGNAL(activated(int)), SLOT(changed()));
   coAll2 = combo;
   combo->setWhatsThis( strAll2 );
+  combo->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
+  label = new QLabel(strMouseButton2, this);
+  label->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
+  label->setBuddy(combo);
+  gridLayout->addWidget(label, 1, 0);
+  gridLayout->addWidget(combo, 1, 1);
 
   combo = new QComboBox(box);
-  grid->addWidget(combo, 3, 1);
   combo->addItems(items);
   connect(combo, SIGNAL(activated(int)), SLOT(changed()));
   coAll3 =  combo;
   combo->setWhatsThis( strAll3 );
+  combo->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
+  label = new QLabel(strMouseButton3, this);
+  label->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
+  label->setBuddy(combo);
+  gridLayout->addWidget(label, 2, 0);
+  gridLayout->addWidget(combo, 2, 1);
+
 
   combo = new QComboBox(box);
-  grid->addWidget(combo, 4, 1);
   combo->addItem(i18n("Raise/Lower"));
   combo->addItem(i18n("Shade/Unshade"));
   combo->addItem(i18n("Maximize/Restore"));
@@ -805,10 +856,17 @@ KWindowActionsConfig::KWindowActionsConfig (bool _standAlone, KConfig *_config,
   combo->addItem(i18n("Nothing"));  
   connect(combo, SIGNAL(activated(int)), SLOT(changed()));
   coAllW =  combo;
-  combo->setWhatsThis( strAllW );
+  combo->setWhatsThis( i18n("Here you can customize KDE's behavior when scrolling with the mouse wheel"
+                            " in a window while pressing the modifier key.") );
+  combo->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
+  label = new QLabel(i18n("Mouse wheel:"), this);
+  label->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
+  label->setBuddy(combo);
+  gridLayout->addWidget(label, 3, 0);
+  gridLayout->addWidget(combo, 3, 1);
+
 
   layout->addStretch();
-
   load();
 }
 
@@ -822,6 +880,8 @@ void KWindowActionsConfig::setComboText( QComboBox* combo, const char*txt )
 {
     if( combo == coWin1 || combo == coWin2 || combo == coWin3 )
         combo->setCurrentIndex( tbl_txt_lookup( tbl_Win, txt ));
+    else if (combo == coWinWheel)
+        combo->setCurrentIndex( tbl_txt_lookup( tbl_WinWheel, txt ));
     else if( combo == coAllKey )
         combo->setCurrentIndex( tbl_txt_lookup( tbl_AllKey, txt ));
     else if( combo == coAll1 || combo == coAll2 || combo == coAll3 )
@@ -837,6 +897,11 @@ const char* KWindowActionsConfig::functionWin( int i )
     return tbl_num_lookup( tbl_Win, i );
 }
 
+const char* KWindowActionsConfig::functionWinWheel (int i)
+{
+    return tbl_num_lookup( tbl_WinWheel, i);
+}
+
 const char* KWindowActionsConfig::functionAllKey( int i )
 {
     return tbl_num_lookup( tbl_AllKey, i );
@@ -858,6 +923,7 @@ void KWindowActionsConfig::load()
   setComboText(coWin1,cg.readEntry("CommandWindow1","Activate, raise and pass click").toAscii());
   setComboText(coWin2,cg.readEntry("CommandWindow2","Activate and pass click").toAscii());
   setComboText(coWin3,cg.readEntry("CommandWindow3","Activate and pass click").toAscii());
+  setComboText(coWinWheel, cg.readEntry("CommandWindowWheel", "Scroll").toAscii());
   setComboText(coAllKey,cg.readEntry("CommandAllKey","Alt").toAscii());
   setComboText(coAll1,cg.readEntry("CommandAll1","Move").toAscii());
   setComboText(coAll2,cg.readEntry("CommandAll2","Toggle raise and lower").toAscii());
@@ -871,6 +937,7 @@ void KWindowActionsConfig::save()
   cg.writeEntry("CommandWindow1", functionWin(coWin1->currentIndex()));
   cg.writeEntry("CommandWindow2", functionWin(coWin2->currentIndex()));
   cg.writeEntry("CommandWindow3", functionWin(coWin3->currentIndex()));
+  cg.writeEntry("CommandWindowWheel", functionWinWheel(coWinWheel->currentIndex()));
   cg.writeEntry("CommandAllKey", functionAllKey(coAllKey->currentIndex()));
   cg.writeEntry("CommandAll1", functionAll(coAll1->currentIndex()));
   cg.writeEntry("CommandAll2", functionAll(coAll2->currentIndex()));
@@ -892,6 +959,7 @@ void KWindowActionsConfig::defaults()
   setComboText(coWin1,"Activate, raise and pass click");
   setComboText(coWin2,"Activate and pass click");
   setComboText(coWin3,"Activate and pass click");
+  setComboText(coWinWheel, "Scroll");
   setComboText(coAllKey,"Alt");
   setComboText (coAll1,"Move");
   setComboText(coAll2,"Toggle raise and lower");
