@@ -24,6 +24,7 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "programgroupingstrategy.h"
 
 #include <QAction>
+#include <QPointer>
 
 #include <KDebug>
 #include <KLocale>
@@ -43,9 +44,9 @@ public:
         :editableGroupProperties(AbstractGroupingStrategy::None)
     {
     }
-    GroupManager *groupManager;
+
     AbstractGroupingStrategy::EditableGroupProperties editableGroupProperties;
-    AbstractGroupableItem *tempItem;
+    QPointer<AbstractGroupableItem> tempItem;
     QStringList blackList; //Programs in this list should not be grouped
 };
 
@@ -54,7 +55,6 @@ ProgramGroupingStrategy::ProgramGroupingStrategy(GroupManager *groupManager)
     :AbstractGroupingStrategy(groupManager),
      d(new Private)
 {
-    d->groupManager = groupManager;
     setType(GroupManager::ProgramGrouping);
 
     KConfig groupBlacklist( "taskbargroupblacklistrc", KConfig::NoGlobals );
@@ -64,11 +64,11 @@ ProgramGroupingStrategy::ProgramGroupingStrategy(GroupManager *groupManager)
 
 ProgramGroupingStrategy::~ProgramGroupingStrategy()
 {
-    KConfig groupBlacklist( "taskbargroupblacklistrc", KConfig::NoGlobals );
-    KConfigGroup blackGroup( &groupBlacklist, "Blacklist" );
-    blackGroup.writeEntry( "Applications", d->blackList );
+    KConfig groupBlacklist("taskbargroupblacklistrc", KConfig::NoGlobals);
+    KConfigGroup blackGroup(&groupBlacklist, "Blacklist");
+    blackGroup.writeEntry("Applications", d->blackList);
     blackGroup.config()->sync();
-    
+
     delete d;
 }
 
@@ -103,6 +103,10 @@ QString ProgramGroupingStrategy::className(AbstractGroupableItem *item)
 
 void ProgramGroupingStrategy::toggleGrouping()
 {
+    if (!d->tempItem) {
+        return;
+    }
+
     QString name = className(d->tempItem);
 
     if (d->blackList.contains(name)) {
@@ -118,36 +122,45 @@ void ProgramGroupingStrategy::toggleGrouping()
         d->blackList.append(name);
         if (d->tempItem->isGroupItem()) {
             closeGroup(qobject_cast<TaskGroup*>(d->tempItem));
-        } else {
-            d->groupManager->rootGroup()->add(d->tempItem);
+        } else if (rootGroup()) {
+            rootGroup()->add(d->tempItem);
         }
     }
+
     d->tempItem = 0;
 }
 
-void ProgramGroupingStrategy::handleItem(AbstractItemPtr item)
+void ProgramGroupingStrategy::handleItem(AbstractGroupableItem *item)
 {
+    GroupPtr root = rootGroup();
+
+    if (!root) {
+        return;
+    }
+
     if (item->isGroupItem()) {
-        d->groupManager->rootGroup()->add(item);
+        //kDebug() << "item is groupitem";
+        root->add(item);
         return;
     } else if (d->blackList.contains((qobject_cast<TaskItem*>(item))->task()->classClass())) {
-        d->groupManager->rootGroup()->add(item);
+        //kDebug() << "item is in blacklist";
+        root->add(item);
         return;
     }
 
     TaskItem *task = dynamic_cast<TaskItem*>(item);
-    if (task && !programGrouping(task, d->groupManager->rootGroup())) {
+    if (task && !programGrouping(task, root)) {
         //kDebug() << "joined rootGroup ";
-        d->groupManager->rootGroup()->add(item);
+        root->add(item);
     }
 }
 
 bool ProgramGroupingStrategy::programGrouping(TaskItem* taskItem, TaskGroup* groupItem)
 {
     //kDebug();
-    QHash <QString,AbstractItemPtr> itemMap;
+    QHash <QString,AbstractGroupableItem *> itemMap;
 
-    foreach (AbstractItemPtr item, groupItem->members()) { //search for an existing group
+    foreach (AbstractGroupableItem *item, groupItem->members()) { //search for an existing group
         if (item->isGroupItem()) { //maybe add the condition that the subgroup was created by programGrouping
             if (programGrouping(taskItem, static_cast<TaskGroup*>(item))) {
                 //kDebug() << "joined subGroup";
@@ -167,14 +180,14 @@ bool ProgramGroupingStrategy::programGrouping(TaskItem* taskItem, TaskGroup* gro
     }
 
     QString name = taskItem->task()->classClass();
-    if (itemMap.count(name) >= groupItem->members().count()) { //join this group
+    if (itemMap.count(name) >= groupItem->members().count() && !groupItem->isRootGroup()) { //join this group if this is not the rootGroup, otherwise tasks may not be grouped if there arent tasks of any other type, typically on startup
         //kDebug() << "joined this Group";
         groupItem->add(taskItem);
         return true;
     } else if (itemMap.count(name) >= 2) { //create new subgroup with at least 2 other task
         //kDebug() << "create Group";
         QIcon icon = taskItem->task()->icon();
-        QList <AbstractItemPtr> list(itemMap.values(name));
+        QList <AbstractGroupableItem *> list(itemMap.values(name));
         TaskGroup* group = createGroup(list);
         group->setName(name);
         group->setColor(Qt::red);
@@ -187,10 +200,7 @@ bool ProgramGroupingStrategy::programGrouping(TaskItem* taskItem, TaskGroup* gro
 void ProgramGroupingStrategy::checkGroup()
 {
     TaskGroup *group = qobject_cast<TaskGroup*>(sender()); 
-    if (!group) {
-        return;
-    }
-    if (group->members().size() <= 1) {
+    if (group && group->members().size() <= 1) {
         closeGroup(group);
     }
 }
