@@ -24,6 +24,7 @@
 #include <QApplication>
 #include <QDBusConnection>
 #include <QDBusInterface>
+#include <QDBusPendingCall>
 #include <QPainter>
 #include <QStyleOptionGraphicsItem>
 #include <QFont>
@@ -67,29 +68,33 @@ Battery::Battery(QObject *parent, const QVariantList &args)
       m_extenderVisible(false),
       m_controlsLayout(0),
       m_batteryLayout(0),
+      m_brightnessLayout(0),
       m_statusLabel(0),
       m_batteryLabel(0),
       m_profileLabel(0),
       m_profileCombo(0),
+      m_brightnessSlider(0),
+      m_minutes(0),
+      m_hours(0),
       m_theme(0),
       m_availableProfiles(QStringList()),
       m_currentProfile(0),
       m_animId(-1),
-      m_alpha(1),
+      m_alpha(0),
       m_fadeIn(false),
       m_acAnimId(-1),
-      m_acAlpha(1),
+      m_acAlpha(0),
       m_acFadeIn(false),
       m_batteryAnimId(-1),
-      m_batteryAlpha(1),
-      m_batteryFadeIn(true),
+      m_batteryAlpha(0),
+      m_batteryFadeIn(false),
       m_isHovered(false),
       m_firstRun(true),
       m_numOfBattery(0),
-      m_acadapter_plugged(false),
+      m_acAdapterPlugged(false),
       m_remainingMSecs(0)
 {
-    kDebug() << "Loading applet battery";
+    //kDebug() << "Loading applet battery";
     setAcceptsHoverEvents(true);
     setHasConfigurationInterface(true);
     setPopupIcon(QIcon());
@@ -100,18 +105,18 @@ Battery::Battery(QObject *parent, const QVariantList &args)
     m_extenderApplet = 0;
     m_theme = new Plasma::Svg(this);
     m_theme->setImagePath("widgets/battery-oxygen");
-    m_theme->setContainsMultipleImages(false);
+    m_theme->setContainsMultipleImages(true);
 }
 
 void Battery::init()
 {
     KConfigGroup cg = config();
     m_showBatteryString = cg.readEntry("showBatteryString", false);
+    m_showRemainingTime = cg.readEntry("showRemainingTime", false);
     m_showMultipleBatteries = cg.readEntry("showMultipleBatteries", !m_isEmbedded);
 
-    showBattery(false);
     m_theme->resize(contentsRect().size());
-    if (m_acadapter_plugged) {
+    if (m_acAdapterPlugged) {
         showAcAdapter(true);
     }
     showBattery(true);
@@ -140,11 +145,11 @@ void Battery::init()
 
     dataUpdated("AC Adapter", dataEngine("powermanagement")->query("AC Adapter"));
 
-    if (!m_isEmbedded) {
+    if (!m_isEmbedded && !extender()->hasItem("powermanagement")) {
         Plasma::ExtenderItem *eItem = new Plasma::ExtenderItem(extender());
         eItem->setName("powermanagement");
-        initBatteryExtender(eItem);
-        extender()->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
+        initExtenderItem(eItem);
+        //extender()->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
     }
 }
 
@@ -156,30 +161,53 @@ void Battery::constraintsEvent(Plasma::Constraints constraints)
     } else {
         setAspectRatioMode(Plasma::KeepAspectRatio);
     }
+    int minWidth;
+    int minHeight;
 
     if (constraints & (Plasma::FormFactorConstraint | Plasma::SizeConstraint)) {
         if (formFactor() == Plasma::Vertical) {
             if (!m_showMultipleBatteries) {
-                setMinimumHeight(qMax(m_textRect.height(), size().width()));
+                minHeight = qMax(m_textRect.height(), size().width());
             } else {
-                setMinimumHeight(qMax(m_textRect.height(), size().width()*m_numOfBattery));
+                minHeight = qMax(m_textRect.height(), size().width()*m_numOfBattery);
             }
             setMinimumWidth(0);
+            setMinimumHeight(minHeight);
             //kDebug() << "Vertical FormFactor";
         } else if (formFactor() == Plasma::Horizontal) {
             if (!m_showMultipleBatteries) {
-                setMinimumWidth(qMax(m_textRect.width(), size().height()));
+                minWidth = qMax(m_textRect.width(), size().height());
             } else {
-                setMinimumWidth(qMax(m_textRect.width(), size().height()*m_numOfBattery));
+                minWidth = qMax(m_textRect.width(), size().height()*m_numOfBattery);
             }
+            setMinimumWidth(minWidth);
             setMinimumHeight(0);
             //kDebug() << "Horizontal FormFactor" << m_textRect.width() << contentsRect().height();
         } else {
             setMaximumSize(QWIDGETSIZE_MAX, QWIDGETSIZE_MAX);
-            setMinimumSize(KIconLoader::SizeSmall, KIconLoader::SizeSmall);
+
+            if (m_showMultipleBatteries) {
+                setMinimumSize(KIconLoader::SizeSmall * m_numOfBattery, KIconLoader::SizeSmall);
+            } else {
+                setMinimumSize(KIconLoader::SizeSmall, KIconLoader::SizeSmall);
+            }
+
+            if (constraints & Plasma::FormFactorConstraint &&
+                    (formFactor() == Plasma::Planar || formFactor() == Plasma::MediaCenter) ) {
+                resize(KIconLoader::SizeEnormous, KIconLoader::SizeEnormous);
+            }
         }
 
-        m_theme->resize(contentsRect().size().toSize());
+        QSize c(contentsRect().size().toSize());
+        if (m_showMultipleBatteries) {
+            if (formFactor() == Plasma::Vertical) {
+                c.setHeight(size().height() / qMax(1, m_numOfBattery));
+            } else if (formFactor() == Plasma::Horizontal) {
+                c.setWidth(size().width() / qMax(1, m_numOfBattery));
+            }
+        }
+        m_theme->resize(c);
+
         m_font.setPointSize(qMax(KGlobalSettings::smallestReadableFont().pointSize(),
                                  qRound(contentsRect().height() / 10)));
         update();
@@ -191,8 +219,8 @@ void Battery::dataUpdated(const QString& source, const Plasma::DataEngine::Data
     if (source.startsWith("Battery")) {
         m_batteries_data[source] = data;
     } else if (source == "AC Adapter") {
-        m_acadapter_plugged = data["Plugged in"].toBool();
-        showAcAdapter(m_acadapter_plugged);
+        m_acAdapterPlugged = data["Plugged in"].toBool();
+        showAcAdapter(m_acAdapterPlugged);
     } else if (source == "PowerDevil") {
         m_availableProfiles = data["availableProfiles"].toStringList();
         m_currentProfile = data["currentProfile"].toString();
@@ -202,7 +230,7 @@ void Battery::dataUpdated(const QString& source, const Plasma::DataEngine::Data
     }
     if (source == "Battery0") {
         m_remainingMSecs  = data["Remaining msec"].toInt();
-        //kDebug() << "Remaining msecs on battery:" << m_remainingMSecs;
+        // kDebug() << "Remaining msecs on battery:" << m_remainingMSecs;
     }
 
     updateStatus();
@@ -233,14 +261,10 @@ void Battery::configAccepted()
     if (m_showMultipleBatteries != ui.showMultipleBatteriesCheckBox->isChecked()) {
         m_showMultipleBatteries = !m_showMultipleBatteries;
         cg.writeEntry("showMultipleBatteries", m_showMultipleBatteries);
-        kDebug() << "Show multiple battery changed: " << m_showMultipleBatteries;
+        //kDebug() << "Show multiple battery changed: " << m_showMultipleBatteries;
         emit sizeHintChanged(Qt::PreferredSize);
     }
 
-    //reconnect sources
-    disconnectSources();
-    connectSources();
-
     emit configNeedsSaving();
 }
 
@@ -277,16 +301,18 @@ Battery::~Battery()
 
 void Battery::suspend()
 {
-    QDBusConnection dbus( QDBusConnection::sessionBus() );
-    QDBusInterface iface( "org.kde.kded", "/modules/powerdevil", "org.kde.PowerDevil", dbus );
-    iface.call( "suspend", Solid::Control::PowerManager::ToRam );
+    hidePopup();
+    QDBusConnection dbus(QDBusConnection::sessionBus());
+    QDBusInterface iface("org.kde.kded", "/modules/powerdevil", "org.kde.PowerDevil", dbus);
+    iface.asyncCall("suspend", Solid::Control::PowerManager::ToRam);
 }
 
 void Battery::hibernate()
 {
-    QDBusConnection dbus( QDBusConnection::sessionBus() );
-    QDBusInterface iface( "org.kde.kded", "/modules/powerdevil", "org.kde.PowerDevil", dbus );
-    iface.call( "suspend", Solid::Control::PowerManager::ToDisk );
+    hidePopup();
+    QDBusConnection dbus(QDBusConnection::sessionBus());
+    QDBusInterface iface("org.kde.kded", "/modules/powerdevil", "org.kde.PowerDevil", dbus);
+    iface.asyncCall("suspend", Solid::Control::PowerManager::ToDisk);
 }
 
 void Battery::brightnessChanged(const int brightness)
@@ -312,13 +338,13 @@ void Battery::setEmbedded(const bool embedded)
     m_isEmbedded = embedded;
 }
 
-void Battery::initBatteryExtender(Plasma::ExtenderItem *item)
+void Battery::initExtenderItem(Plasma::ExtenderItem *item)
 {
     // We only show the extender for applets that are not embedded, as
     // that would create infinitve loops, you really don't want an applet
     // extender when the applet is embedded into another applet, such
     // as the battery applet is also embedded into the battery's extender.
-    if (!m_isEmbedded) {
+    if (!m_isEmbedded && item->name() == "powermanagement") {
         int row = 0;
         int rowHeight = 20;
         int columnWidth = 120;
@@ -369,11 +395,12 @@ void Battery::initBatteryExtender(Plasma::ExtenderItem *item)
 
         Plasma::Label *brightnessLabel = new Plasma::Label(controls);
         brightnessLabel->setText(i18n("Screen Brightness"));
-        brightnessLabel->nativeWidget()->setWordWrap(false);
         m_controlsLayout->addItem(brightnessLabel, row, 0, 1, 3);
         brightnessLabel->nativeWidget()->setWordWrap(false);
         row++;
 
+        m_brightnessLayout = new QGraphicsLinearLayout();
+
         m_brightnessSlider = new Plasma::Slider(controls);
         m_brightnessSlider->setRange(0, 100);
         m_brightnessSlider->setValue(Solid::Control::PowerManager::brightness());
@@ -386,7 +413,7 @@ void Battery::initBatteryExtender(Plasma::ExtenderItem *item)
 
         connect(notifier, SIGNAL(brightnessChanged(float)),
                 this, SLOT(updateSlider(float)));
-        m_controlsLayout->addItem(m_brightnessSlider, row, 1, 1, 1);
+        m_brightnessLayout->addItem(m_brightnessSlider);
 
         Plasma::IconWidget *brightnessIcon = new Plasma::IconWidget(controls);
         brightnessIcon->setIcon("ktip");
@@ -394,8 +421,9 @@ void Battery::initBatteryExtender(Plasma::ExtenderItem *item)
                 this, SLOT(setFullBrightness()));
         brightnessIcon->setDrawBackground(true);
         brightnessIcon->setMinimumSize(rowHeight, rowHeight);
-        m_controlsLayout->addItem(brightnessIcon, row, 2, 1, 1);
-        m_controlsLayout->setRowSpacing(row, 10);
+        brightnessIcon->setMaximumSize(rowHeight, rowHeight);
+        m_brightnessLayout->addItem(brightnessIcon);
+        m_controlsLayout->addItem(m_brightnessLayout, row, 1, 1, 1);
         row++;
 
         m_profileLabel = new Plasma::Label(controls);
@@ -435,6 +463,7 @@ void Battery::initBatteryExtender(Plasma::ExtenderItem *item)
                 suspendButton->setOrientation(Qt::Horizontal);
                 suspendButton->setMaximumHeight(36);
                 suspendButton->setDrawBackground(true);
+                suspendButton->setTextBackgroundColor(QColor());
                 actionsLayout->addItem(suspendButton, 0, 0);
                 connect(suspendButton, SIGNAL(clicked()), this, SLOT(suspend()));
                 actionsLayout->setColumnSpacing(0, 20);
@@ -445,6 +474,7 @@ void Battery::initBatteryExtender(Plasma::ExtenderItem *item)
                 hibernateButton->setOrientation(Qt::Horizontal);
                 hibernateButton->setMaximumHeight(36);
                 hibernateButton->setDrawBackground(true);
+                hibernateButton->setTextBackgroundColor(QColor());
                 actionsLayout->addItem(hibernateButton, 0, 1);
                 connect(hibernateButton, SIGNAL(clicked()), this, SLOT(hibernate()));
             }
@@ -459,9 +489,11 @@ void Battery::initBatteryExtender(Plasma::ExtenderItem *item)
         configButton->setOrientation(Qt::Horizontal);
         configButton->setMaximumHeight(36);
         configButton->setDrawBackground(true);
+        configButton->setTextBackgroundColor(QColor());
         configButton->setIcon("preferences-system-power-management");
         //configButton->nativeWidget()->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
         connect(configButton, SIGNAL(clicked()), this, SLOT(openConfig()));
+        configButton->setEnabled(hasAuthorization("LaunchApp"));
 
         //QGraphicsGridLayout *moreLayout = new QGraphicsGridLayout(m_controlsLayout);
         //moreLayout->setColumnPreferredWidth(0, columnWidth);
@@ -491,32 +523,21 @@ void Battery::updateStatus()
     if (m_numOfBattery && m_batteryLabel) {
         QHashIterator<QString, QHash<QString, QVariant > > battery_data(m_batteries_data);
         int bnum = 0;
-        int hours = m_remainingMSecs/1000/3600;
-        int minutes = qRound(m_remainingMSecs/60000) % 60;
 
         while (battery_data.hasNext()) {
             bnum++;
             battery_data.next();
             QString state = battery_data.value()["State"].toString();
-            if (state == "Discharging" && m_remainingMSecs > 0) {
+            m_remainingMSecs = battery_data.value()["Remaining msec"].toInt();
+            //kDebug() << "time left:" << m_remainingMSecs;
+            if (state == "Discharging" && m_remainingMSecs > 0 && m_showRemainingTime) {
 
                 // FIXME: Somehow, m_extenderApplet is null here, so the label never becomes visible
                 if (m_extenderApplet) {
                     m_extenderApplet->showBatteryLabel(true);
                 }
-
                 // we don't have too much accuracy so only give hours and minutes
-                int msecs = hours * 1000 * 3600 + minutes * 60000;
-                batteryLabelText.append(i18n("Time remaining: <b>%1</b><br />", KGlobal::locale()->prettyFormatDuration(msecs)));
-                kDebug() << "hours:" << hours << "minutes:" << minutes;
-                /* might be useful for the tooltip
-                kDebug() << "hours:" << hours << "minutes:" << minutes;
-                QTime t = QTime(hours, minutes);
-                kDebug() << t;
-                KLocale tmpLocale(*KGlobal::locale());
-                tmpLocale.setTimeFormat("%k:h %Mm remaining");
-                kDebug() << tmpLocale.formatTime(t, false, true); // minutes, hours as duration
-                */
+                batteryLabelText.append(i18n("Time remaining: <b>%1</b><br />", KGlobal::locale()->prettyFormatDuration(m_remainingMSecs)));
             } else {
                 if (m_extenderApplet) {
                     m_extenderApplet->showBatteryLabel(false);
@@ -548,7 +569,7 @@ void Battery::updateStatus()
             }
         }
 
-        if (m_acadapter_plugged) {
+        if (m_acAdapterPlugged) {
             batteryLabelText.append(i18n("<b>AC Adapter:</b> Plugged in"));
         } else {
             batteryLabelText.append(i18n("<b>AC Adapter:</b> Not plugged in"));
@@ -579,6 +600,11 @@ void Battery::updateStatus()
             m_profileLabel->show();
         }
     }
+
+    if (m_brightnessSlider) {
+        m_brightnessSlider->setValue(Solid::Control::PowerManager::brightness());
+        //kDebug() << "Updating brightness:" << Solid::Control::PowerManager::brightness();
+    }
     //kDebug() << "SIZE LABEL" << m_batteryLabel->size() << m_batteryLabel->preferredSize() << m_batteryLabel->preferredSize();
     m_controlsLayout->setColumnMinimumWidth(1,280);
     m_batteryLayout->setColumnMinimumWidth(0,200);
@@ -588,7 +614,7 @@ void Battery::updateStatus()
 
 void Battery::openConfig()
 {
-    kDebug() << "opening powermanagement configuration dialog";
+    //kDebug() << "opening powermanagement configuration dialog";
     QStringList args;
     args << "powerdevilconfig";
     KToolInvocation::kdeinitExec("kcmshell4", args);
@@ -622,16 +648,17 @@ void Battery::showLabel(bool show)
 
 QRectF Battery::scaleRectF(const qreal progress, QRectF rect)
 {
-    if (progress == 1) {
+    if (qFuzzyCompare(progress, qreal(1))) {
         return rect;
     }
+
     // Scale
-    qreal w = rect.width()*progress;
-    qreal h = rect.width()*progress;
+    qreal w = rect.width() * progress;
+    qreal h = rect.width() * progress;
 
     // Position centered
-    rect.setX((rect.width() - w)/2);
-    rect.setY((rect.height() - h)/2);
+    rect.setX(rect.x() + (rect.width() - w) / 2);
+    rect.setY(rect.y() + (rect.height() - h) / 2);
 
     rect.setWidth(w);
     rect.setHeight(h);
@@ -648,14 +675,15 @@ void Battery::showAcAdapter(bool show)
     const int FadeInDuration = 600;
     // As long as the animation is running, we fake it's still plugged in so it gets
     // painted in paintInterface()
-    m_acadapter_plugged = true;
+    m_acAdapterPlugged = true;
 
     if (m_acAnimId != -1) {
         Plasma::Animator::self()->stopCustomAnimation(m_acAnimId);
     }
+
     m_acAnimId = Plasma::Animator::self()->customAnimation(40 / (1000 / FadeInDuration), FadeInDuration,
-                                                      Plasma::Animator::EaseOutCurve, this,
-                                                      "acAnimationUpdate");
+                                                           Plasma::Animator::EaseOutCurve, this,
+                                                           "acAnimationUpdate");
 }
 
 void Battery::showBattery(bool show)
@@ -663,42 +691,50 @@ void Battery::showBattery(bool show)
     if (m_batteryFadeIn == show) {
         return;
     }
+
     m_batteryFadeIn = show;
     const int FadeInDuration = 300;
 
     if (m_batteryAnimId != -1) {
         Plasma::Animator::self()->stopCustomAnimation(m_batteryAnimId);
     }
+
     m_batteryAnimId = Plasma::Animator::self()->customAnimation(40 / (1000 / FadeInDuration), FadeInDuration,
-                                                      Plasma::Animator::EaseOutCurve, this,
-                                                      "batteryAnimationUpdate");
+                                                                Plasma::Animator::EaseOutCurve, this,
+                                                                "batteryAnimationUpdate");
 }
 
 void Battery::animationUpdate(qreal progress)
 {
-    if (progress == 1) {
+    if (qFuzzyCompare(progress, qreal(1))) {
         m_animId = -1;
-    }
-    if (!m_fadeIn) {
-        qreal new_alpha = m_fadeIn ? progress : 1 - progress;
-        m_alpha = qMin(new_alpha, m_alpha);
+        m_alpha = m_fadeIn ? 1 : 0;
+    } else if (m_fadeIn) {
+        m_alpha = progress;
     } else {
-        m_alpha = m_fadeIn ? progress : 1 - progress;
+        m_alpha = qMin(1 - progress, m_alpha);
     }
-    m_alpha = qMax(qreal(0.0), m_alpha);
+
+    m_alpha = qBound(qreal(0), m_alpha, qreal(1));
     update();
 }
 
 void Battery::acAnimationUpdate(qreal progress)
 {
-    if (progress == 1) {
+    if (qFuzzyCompare(progress, qreal(1))) {
         m_acAnimId = -1;
+        m_acAlpha = m_acFadeIn ? 1 : 0;
+    } else if (m_acFadeIn) {
+        m_acAlpha = progress;
+    } else {
+        m_acAlpha = qMin(1 - progress, m_acAlpha);
     }
-    m_acAlpha = m_acFadeIn ? progress : 1 - progress;
+
+    m_acAlpha = qBound(qreal(0), m_acAlpha, qreal(1));
     // During the fadeout animation, we had set it to true (and lie)
     // now the animation has ended, we _really_ set it to not show the adapter
-    if (!m_acFadeIn && (progress == 1)) {
-        m_acadapter_plugged = false;
+    if (!m_acFadeIn && qFuzzyCompare(progress, qreal(1))) {
+        m_acAdapterPlugged = false;
         updateStatus();
     }
     update();
@@ -706,10 +742,16 @@ void Battery::acAnimationUpdate(qreal progress)
 
 void Battery::batteryAnimationUpdate(qreal progress)
 {
-    if (progress == 1) {
+    if (qFuzzyCompare(progress, qreal(1))) {
         m_batteryAnimId = -1;
+        m_batteryAlpha = m_batteryFadeIn ? 1 : 0;
+    } else if (m_batteryFadeIn) {
+        m_batteryAlpha = progress;
+    } else {
+        m_batteryAlpha = qMin(1 - progress, m_batteryAlpha);
     }
-    m_batteryAlpha = m_batteryFadeIn ? progress : 1 - progress;
+
+    m_batteryAlpha = qBound(qreal(0), m_batteryAlpha, qreal(1));
     update();
 }
 
@@ -813,18 +855,11 @@ void Battery::paintBattery(QPainter *p, const QRect &contentsRect, const int bat
         }
     }
 
-    if (m_acadapter_plugged) {
+    if (m_acAdapterPlugged) {
         //QRectF ac_rect = QRectF(contentsRect.topLeft(), QSizeF(contentsRect.width()*m_acAlpha, contentsRect.height()*m_acAlpha));
         m_theme->paint(p, scaleRectF(m_acAlpha, contentsRect), "AcAdapter");
     }
 
-    // For small FormFactors, we're drawing a shadow
-    if (formFactor() == Plasma::Vertical ||
-        formFactor() == Plasma::Horizontal) {
-        if (plugState) {
-            m_theme->paint(p, contentsRect, "Shadow");
-        }
-    }
     if (plugState && m_theme->hasElement("Overlay")) {
         m_theme->paint(p, scaleRectF(m_batteryAlpha, contentsRect), "Overlay");
     }
@@ -839,7 +874,7 @@ void Battery::paintInterface(QPainter *p, const QStyleOptionGraphicsItem *option
 
     if (m_numOfBattery == 0) {
         QRectF ac_contentsRect(contentsRect.topLeft(), QSizeF(qMax(qreal(0.0), contentsRect.width() * m_acAlpha), qMax(qreal(0.0), contentsRect.height() * m_acAlpha)));
-        if (m_acadapter_plugged) {
+        if (m_acAdapterPlugged) {
             m_theme->paint(p, ac_contentsRect, "AcAdapter");
         }
         paintBattery(p, contentsRect, 0, false);
@@ -849,14 +884,25 @@ void Battery::paintInterface(QPainter *p, const QStyleOptionGraphicsItem *option
     if (m_isEmbedded || m_showMultipleBatteries || m_firstRun) {
         // paint each battery with own charge level
         int battery_num = 0;
-        int width = contentsRect.width()/m_numOfBattery;
+        int height = contentsRect.height();
+        int width = contentsRect.width();
+        int xdelta = 0;
+        int ydelta = 0;
+        if (m_showMultipleBatteries) {
+            if (formFactor() == Plasma::Vertical) {
+                height = height / m_numOfBattery;
+                ydelta = height;
+            } else {
+                width = width / m_numOfBattery;
+                xdelta = width;
+            }
+        }
         QHashIterator<QString, QHash<QString, QVariant > > battery_data(m_batteries_data);
         while (battery_data.hasNext()) {
             battery_data.next();
-            QRect corect = QRect(contentsRect.left()+battery_num*width,
-                                 contentsRect.top(),
-                                 width, contentsRect.height());
-
+            QRect corect = QRect(contentsRect.left() + battery_num * xdelta,
+                                 contentsRect.top() + battery_num * ydelta,
+                                 width, height);
             // paint battery with appropriate charge level
             paintBattery(p, corect, battery_data.value()["Percent"].toInt(), battery_data.value()["Plugged in"].toBool());
 
@@ -864,8 +910,24 @@ void Battery::paintInterface(QPainter *p, const QStyleOptionGraphicsItem *option
                 // Show the charge percentage with a box on top of the battery
                 QString batteryLabel;
                 if (battery_data.value()["Plugged in"].toBool()) {
-                    batteryLabel = battery_data.value()["Percent"].toString();
-                    batteryLabel.append("%");
+                    int hours = m_remainingMSecs/1000/3600;
+                    int minutes = qRound(m_remainingMSecs/60000) % 60;
+                    if (!(minutes==0 && hours==0)) {
+                        m_minutes= minutes;
+                        m_hours= hours;
+                    }
+                    QString state = battery_data.value()["State"].toString();
+
+                    if (m_showRemainingTime && (state=="Charging" || state=="Discharging" )) {
+                        m_remainingMSecs = battery_data.value()["Remaining msec"].toInt();
+                        QTime t = QTime(m_hours, m_minutes);
+                        KLocale tmpLocale(*KGlobal::locale());
+                        tmpLocale.setTimeFormat("%k:%M");
+                        batteryLabel = tmpLocale.formatTime(t, false, true); // minutes, hours as duration
+                    } else {
+                        batteryLabel = battery_data.value()["Percent"].toString();
+                        batteryLabel.append("%");
+                    }
                     paintLabel(p, corect, batteryLabel);
                 }
             }
@@ -904,7 +966,7 @@ void Battery::paintInterface(QPainter *p, const QStyleOptionGraphicsItem *option
 
 void Battery::showBatteryLabel(bool show)
 {
-    kDebug() << show;
+    //kDebug() << show;
     if (show != m_showBatteryString) {
         showLabel(show);
         m_showBatteryString = show;
@@ -928,21 +990,6 @@ void Battery::connectSources()
             this,                          SLOT(sourceRemoved(QString)));
 }
 
-void Battery::disconnectSources()
-{
-    const QStringList& battery_sources = dataEngine("powermanagement")->query("Battery")["sources"].toStringList();
-
-    foreach (const QString &battery_source ,battery_sources) {
-        dataEngine("powermanagement")->disconnectSource(battery_source, this);
-    }
-
-    dataEngine("powermanagement")->disconnectSource("AC Adapter", this);
-    dataEngine("powermanagement")->disconnectSource("PowerDevil", this);
-
-    disconnect(SLOT(sourceAdded(QString)));
-    disconnect(SLOT(sourceRemoved(QString)));
-}
-
 void Battery::sourceAdded(const QString& source)
 {
     if (source.startsWith("Battery") && source != "Battery") {
