@@ -86,8 +86,7 @@ class Battery : public Plasma::PopupApplet
 
     private:
         void connectSources();
-        void disconnectSources();
-        void initBatteryExtender(Plasma::ExtenderItem *item);
+        void initExtenderItem(Plasma::ExtenderItem *item);
         void updateStatus();
 
         /* Prevent creating infinite loops by embedding applets inside applets */
@@ -97,6 +96,7 @@ class Battery : public Plasma::PopupApplet
 
         QGraphicsGridLayout *m_controlsLayout;
         QGraphicsGridLayout *m_batteryLayout;
+        QGraphicsLinearLayout *m_brightnessLayout;
         Plasma::Label *m_statusLabel;
         Plasma::Label *m_batteryLabel;
         Plasma::Label *m_profileLabel;
@@ -120,6 +120,10 @@ class Battery : public Plasma::PopupApplet
         bool m_showMultipleBatteries;
         /* Should the battery charge information be shown on top? */
         bool m_showBatteryString;
+        /* Should that info be percentage (false) or time (true)? */
+        bool m_showRemainingTime;
+        int m_minutes;
+        int m_hours;
         QSizeF m_size;
         int m_pixelSize;
         Plasma::Svg* m_theme;
@@ -155,7 +159,7 @@ class Battery : public Plasma::PopupApplet
         int m_boxAlpha;
         int m_boxHoverAlpha;
         int m_numOfBattery;
-        bool m_acadapter_plugged;
+        bool m_acAdapterPlugged;
         int m_remainingMSecs;
 };
 
