@@ -52,7 +52,7 @@ QStandardItem* LeaveModel::createStandardItem(const QString& url)
     } else if (basename == "lock") {
         item->setText(i18n("Lock"));
         item->setIcon(KIcon("system-lock-screen"));
-        item->setData(i18n("Lock the screen"), Kickoff::SubTitleRole);
+        item->setData(i18n("Lock Screen"), Kickoff::SubTitleRole);
     } else if (basename == "switch") {
         item->setText(i18n("Switch User"));
         item->setIcon(KIcon("system-switch-user"));
@@ -60,11 +60,11 @@ QStandardItem* LeaveModel::createStandardItem(const QString& url)
     } else if (basename == "shutdown") {
         item->setText(i18n("Shutdown"));
         item->setIcon(KIcon("system-shutdown"));
-        item->setData(i18n("Turn off the computer"), Kickoff::SubTitleRole);
+        item->setData(i18n("Turn off computer"), Kickoff::SubTitleRole);
     } else if (basename == "restart") {
-        item->setText(i18nc("Restart the computer", "Restart"));
-        item->setIcon(KIcon("system-restart"));
-        item->setData(i18n("Restart the computer"), Kickoff::SubTitleRole);
+        item->setText(i18nc("Restart computer", "Restart"));
+        item->setIcon(KIcon("system-reboot"));
+        item->setData(i18n("Restart Computer"), Kickoff::SubTitleRole);
     } else if (basename == "savesession") {
         item->setText(i18n("Save Session"));
         item->setIcon(KIcon("document-save"));
@@ -79,7 +79,7 @@ QStandardItem* LeaveModel::createStandardItem(const QString& url)
         item->setData(i18n("Pause without logging out"), Kickoff::SubTitleRole);
     } else if (basename == "suspendram") {
         item->setText(i18n("Suspend to RAM"));
-        item->setIcon(KIcon("system-suspend-hibernate"));
+        item->setIcon(KIcon("system-suspend"));
         item->setData(i18n("Pause without logging out"), Kickoff::SubTitleRole);
     } else {
         item->setText(basename);
@@ -95,6 +95,21 @@ LeaveModel::LeaveModel(QObject *parent)
 {
 }
 
+QVariant LeaveModel::headerData(int section, Qt::Orientation orientation, int role) const
+{
+    if (orientation != Qt::Horizontal || section != 0) {
+        return QVariant();
+    }
+
+    switch (role) {
+    case Qt::DisplayRole:
+        return i18n("Leave");
+        break;
+    default:
+        return QVariant();
+    }
+}
+
 void LeaveModel::updateModel()
 {
     clear();
