@@ -1,6 +1,6 @@
 /*
     Copyright 2007 Robert Knight <robertknight@gmail.com>
-    Copyright 2008 Sebastian Sauer <mail@dipe.org>
+    Copyright 2008-2009 Sebastian Sauer <mail@dipe.org>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -22,12 +22,13 @@
 #define MENUVIEW_H
 
 // Qt
-#include <QModelIndex>
+#include <QtCore/QModelIndex>
 
 // KDE
 #include <KMenu>
 
 class QAbstractItemModel;
+class QStandardItem;
 
 namespace Kickoff
 {
@@ -54,14 +55,21 @@ class MenuView : public KMenu
 public:
 
     /** Constructs a new menu with the specified @p parent */
-    MenuView(QWidget *parent = 0);
+    MenuView(QWidget *parent = 0, const QString &title = QString(), const QIcon &icon = QIcon());
     /** Destructor */
     virtual ~MenuView();
 
-    /** Sets the model displayed by this menu. */
-    void setModel(QAbstractItemModel *model);
-    /** Returns the model displayed by this menu. */
-    QAbstractItemModel *model() const;
+    /// Options for a model.
+    enum ModelOptions {
+        None, ///< no options.
+        MergeFirstLevel ///< merge the first both levels of items within the model into one hirachy in the menuview.
+    };
+
+    /** Adds a model to display within this menu. */
+    void addModel(QAbstractItemModel *model, ModelOptions options = None);
+
+    /** Adds a QStandardItem to display within this menu. This menu will take over the ownership of the item. */
+    void addItem(QStandardItem *item);
 
     /** Returns the UrlItemLauncher used to handle launching of urls. */
     UrlItemLauncher *launcher() const;
@@ -104,6 +112,8 @@ public:
     FormatType formatType() const;
     /** Set the format type. */
     void setFormatType(FormatType formattype);
+    /** Set visibility of model title on menu. */
+    void setModelTitleVisible(QAbstractItemModel *model, bool visible);
 
 protected:
 
@@ -126,10 +136,10 @@ protected:
      * The default implementation sets the action's text to the index's Qt::DisplayRole data
      * and the action's icon to the index's Qt::DecorationRole data.
      */
-    virtual void updateAction(QAction *action, const QModelIndex& index);
+    virtual void updateAction(QAbstractItemModel *model, QAction *action, const QModelIndex& index);
 
     // reimplemented
-    virtual bool eventFilter(QObject * watched, QEvent *event);
+    virtual bool eventFilter(QObject *watched, QEvent *event);
 
 public Q_SLOTS:
     /**
@@ -137,6 +147,8 @@ public Q_SLOTS:
      */
     void actionTriggered(QAction* action);
 
+    void contextMenuRequested(const QPoint& pos);
+
 Q_SIGNALS:
     /**
      * Compared to the aboutToShow() this signal will be emitted after the menu
@@ -144,14 +156,16 @@ Q_SIGNALS:
      */
     void afterBeingHidden();
 
+    void customContextMenuRequested(QMenu* menu, const QPoint& pos);
+
 private Q_SLOTS:
-    // new items are about to be inserted into the model
-    void rowsAboutToBeInserted(const QModelIndex& parent, int start, int end);
-    // existing items are about to be removed from the model
+    /// new items have been inserted into the model
+    void rowsInserted(const QModelIndex& parent, int start, int end);
+    /// existing items are about to be removed from the model
     void rowsAboutToBeRemoved(const QModelIndex& parent, int start, int end);
-    // data within an item of the model change
+    /// data within an item of the model change
     void dataChanged(const QModelIndex& topLeft, const QModelIndex& bottomRight);
-    // the model did reset itself and all items are invalid
+    /// the model did reset itself and all items are invalid
     void modelReset();
 
 private:
