@@ -1,4 +1,4 @@
-/*  
+/*
     Copyright 2007 Robert Knight <robertknight@gmail.com>
 
     This library is free software; you can redistribute it and/or
@@ -28,6 +28,7 @@
 #include <QtGui/QPainter>
 #include <QtGui/QPaintEvent>
 #include <QtGui/QScrollBar>
+#include <QtGui/QToolTip>
 
 // KDE
 #include <KDebug>
@@ -37,6 +38,7 @@
 
 // Local
 #include "core/models.h"
+#include "core/kickoffmodel.h"
 #include "ui/itemdelegate.h"
 
 using namespace Kickoff;
@@ -45,14 +47,12 @@ class UrlItemView::Private
 {
 public:
     Private(UrlItemView *parent)
-        : q(parent)
-        , contentsHeight(0)
-        , itemStateProvider(0)
-    {
+            : q(parent)
+            , contentsHeight(0)
+            , itemStateProvider(0) {
     }
 
-    void doLayout()
-    {
+    void doLayout() {
         // clear existing layout information
         itemRects.clear();
         visualOrder.clear();
@@ -69,11 +69,11 @@ public:
         QModelIndex branch = currentRootIndex;
 
         while (true) {
-            if (itemChildOffsets[branch]+row >= q->model()->rowCount(branch) ||
-                branch != currentRootIndex && row > MAX_CHILD_ROWS) {
+            if (itemChildOffsets[branch] + row >= q->model()->rowCount(branch) ||
+                    (branch != currentRootIndex && row > MAX_CHILD_ROWS)) {
 
                 if (branch.isValid()) {
-                    row = branch.row()+1;
+                    row = branch.row() + 1;
                     branch = branch.parent();
                     continue;
                 } else {
@@ -81,7 +81,7 @@ public:
                 }
             }
 
-            QModelIndex child = q->model()->index(row+itemChildOffsets[branch],0,branch);
+            QModelIndex child = q->model()->index(row + itemChildOffsets[branch], 0, branch);
 
             if (q->model()->hasChildren(child)) {
                 QSize childSize = calculateHeaderSize(child);
@@ -93,7 +93,7 @@ public:
                     // don't subtract 1
                     verticalOffset += childSize.height();
                 }
-                horizontalOffset = 0; 
+                horizontalOffset = 0;
                 branch = child;
                 row = 0;
                 visualColumn = 0;
@@ -101,8 +101,8 @@ public:
                 QSize childSize = calculateItemSize(child);
                 //kDebug() << "item rect for" << child.data(Qt::DisplayRole) << "is" << QRect(QPoint(horizontalOffset,verticalOffset), childSize);
 
-                itemRects.insert(child,QRect(QPoint(horizontalOffset,verticalOffset),
-                                             childSize));
+                itemRects.insert(child, QRect(QPoint(horizontalOffset, verticalOffset),
+                                              childSize));
 
                 if (childSize.isValid()) {
                     visualOrder << child;
@@ -113,15 +113,15 @@ public:
                 visualColumn++;
                 row++;
 
-                bool wasLastRow = row+itemChildOffsets[branch] >= q->model()->rowCount(branch);
+                bool wasLastRow = row + itemChildOffsets[branch] >= q->model()->rowCount(branch);
                 bool nextItemIsBranch = false;
-                if (!wasLastRow) { 
-                     QModelIndex nextIndex =q->model()->index(row+itemChildOffsets[branch],0,branch);
-                     nextItemIsBranch = q->model()->hasChildren(nextIndex);
+                if (!wasLastRow) {
+                    QModelIndex nextIndex = q->model()->index(row + itemChildOffsets[branch], 0, branch);
+                    nextItemIsBranch = q->model()->hasChildren(nextIndex);
                 }
 
                 if (visualColumn >= MAX_COLUMNS || wasLastRow || nextItemIsBranch) {
-                    horizontalOffset = 0; 
+                    horizontalOffset = 0;
                     visualColumn = 0;
                 }
 
@@ -138,8 +138,7 @@ public:
 
     void drawHeader(QPainter *painter,
                     const QModelIndex& index,
-                    const QStyleOptionViewItem& option)
-    {
+                    const QStyleOptionViewItem& option) {
         const bool first = isFirstHeader(index);
         const int rightMargin = q->style()->pixelMetric(QStyle::PM_ScrollBarExtent) + 6;
         const int dy = (first ? 4 : ItemDelegate::HEADER_TOP_MARGIN);
@@ -164,25 +163,22 @@ public:
         painter->setPen(QPen(KColorScheme(QPalette::Active).foreground(KColorScheme::InactiveText), 0));
         QString text = index.data(Qt::DisplayRole).value<QString>();
         painter->drawText(option.rect.adjusted(0, dy, -rightMargin, 0),
-                          Qt::AlignVCenter|Qt::AlignRight, text);
+                          Qt::AlignVCenter | Qt::AlignRight, text);
         painter->restore();
     }
 
-    void updateScrollBarRange()
-    {
-        int pageSize = q->height();
-        q->verticalScrollBar()->setRange(0,contentsHeight-pageSize);
+    void updateScrollBarRange() {
+        const int pageSize = q->height();
+        q->verticalScrollBar()->setRange(0, contentsHeight - pageSize);
         q->verticalScrollBar()->setPageStep(pageSize);
         q->verticalScrollBar()->setSingleStep(q->sizeHintForRow(0));
     }
 
-    int contentWidth() const
-    {
+    int contentWidth() const {
         return q->width() - q->style()->pixelMetric(QStyle::PM_ScrollBarExtent) + 2;
     }
 
-    QSize calculateItemSize(const QModelIndex& index) const 
-    {
+    QSize calculateItemSize(const QModelIndex& index) const {
         if (itemStateProvider && !itemStateProvider->isVisible(index)) {
             return QSize();
         } else {
@@ -190,8 +186,7 @@ public:
         }
     }
 
-    bool isFirstHeader(const QModelIndex &index) const
-    {
+    bool isFirstHeader(const QModelIndex &index) const {
         if (index.row() == 0) {
             return q->model()->hasChildren(index);
         }
@@ -210,8 +205,15 @@ public:
         return true;
     }
 
-    QSize calculateHeaderSize(const QModelIndex& index) const
-    {
+    bool insertAbove(const QRect &itemRect, const QPoint &pos) const {
+        return pos.y() < itemRect.top() + (itemRect.height() / 2);
+    }
+
+    bool insertBelow(const QRect &itemRect, const QPoint &pos) const {
+        return pos.y() >= itemRect.top() + (itemRect.height() / 2);
+    }
+
+    QSize calculateHeaderSize(const QModelIndex& index) const {
         const QFontMetrics fm(KGlobalSettings::smallestReadableFont());
         int minHeight = ItemDelegate::HEADER_HEIGHT;
         const bool isFirst = isFirstHeader(index);
@@ -227,14 +229,12 @@ public:
                      + ItemDelegate::HEADER_BOTTOM_MARGIN) ;
     }
 
-    QPoint mapFromViewport(const QPoint& point) const
-    {
-        return point + QPoint(0,q->verticalOffset());
+    QPoint mapFromViewport(const QPoint& point) const {
+        return point + QPoint(0, q->verticalOffset());
     }
 
-    QPoint mapToViewport(const QPoint& point) const
-    {
-        return point - QPoint(0,q->verticalOffset());
+    QPoint mapToViewport(const QPoint& point) const {
+        return point - QPoint(0, q->verticalOffset());
     }
 
     UrlItemView * const q;
@@ -242,10 +242,14 @@ public:
     QPersistentModelIndex hoveredIndex;
     QPersistentModelIndex watchedIndexForDrag;
 
-    QHash<QModelIndex,int> itemChildOffsets;
-    QHash<QModelIndex,QRect> itemRects;
+    QHash<QModelIndex, int> itemChildOffsets;
+    QHash<QModelIndex, QRect> itemRects;
     QList<QModelIndex> visualOrder;
 
+    QRect dropRect;
+    int draggedRow;
+    bool dragging;
+
     int contentsHeight;
     ItemStateProvider *itemStateProvider;
 
@@ -254,7 +258,7 @@ public:
     // TODO Eventually it will be possible to restrict each branch to only showing
     // a given number of children, with Next/Previous arrows to view more children
     //
-    // eg.  
+    // eg.
     //
     // Recent Documents  [1-10 of 100]            Next
     // Recent Applications [10-20 of 30] Previous|Next
@@ -263,11 +267,16 @@ public:
 };
 
 UrlItemView::UrlItemView(QWidget *parent)
-    : QAbstractItemView(parent)
-    , d(new Private(this))
+        : QAbstractItemView(parent)
+        , d(new Private(this))
 {
+    d->dragging = false;
     setIconSize(QSize(KIconLoader::SizeMedium, KIconLoader::SizeMedium));
     setMouseTracking(true);
+    QPalette viewPalette(palette());
+    viewPalette.setColor(QPalette::Window, palette().color(QPalette::Active, QPalette::Base));
+    setPalette(viewPalette);
+    setAutoFillBackground(true);
 }
 
 UrlItemView::~UrlItemView()
@@ -279,7 +288,7 @@ QModelIndex UrlItemView::indexAt(const QPoint& point) const
 {
     // simple linear search through the item rects, this will
     // be inefficient when the viewport is large
-    QHashIterator<QModelIndex,QRect> iter(d->itemRects);
+    QHashIterator<QModelIndex, QRect> iter(d->itemRects);
     while (iter.hasNext()) {
         iter.next();
         if (iter.value().contains(d->mapFromViewport(point))) {
@@ -294,9 +303,9 @@ void UrlItemView::setModel(QAbstractItemModel *model)
     QAbstractItemView::setModel(model);
 
     if (model) {
-        connect(model,SIGNAL(rowsRemoved(QModelIndex,int,int)),this,SLOT(updateLayout()));
-        connect(model,SIGNAL(rowsInserted(QModelIndex,int,int)),this,SLOT(updateLayout()));
-        connect(model,SIGNAL(modelReset()),this,SLOT(updateLayout()));
+        connect(model, SIGNAL(rowsRemoved(QModelIndex, int, int)), this, SLOT(updateLayout()));
+        connect(model, SIGNAL(rowsInserted(QModelIndex, int, int)), this, SLOT(updateLayout()));
+        connect(model, SIGNAL(modelReset()), this, SLOT(updateLayout()));
     }
 
     d->currentRootIndex = QModelIndex();
@@ -316,36 +325,34 @@ void UrlItemView::updateLayout()
 void UrlItemView::scrollTo(const QModelIndex& index, ScrollHint hint)
 {
     QRect itemRect = d->itemRects[index];
-    QRect viewedRect = QRect(d->mapFromViewport(QPoint(0,0)),
+    QRect viewedRect = QRect(d->mapFromViewport(QPoint(0, 0)),
                              size());
-    int topDifference = viewedRect.top()-itemRect.top();
-    int bottomDifference = viewedRect.bottom()-itemRect.bottom();
+    int topDifference = viewedRect.top() - itemRect.top();
+    int bottomDifference = viewedRect.bottom() - itemRect.bottom();
     QScrollBar *scrollBar = verticalScrollBar();
 
     if (!itemRect.isValid())
         return;
 
     switch (hint) {
-        case EnsureVisible:
-            {
-               if (!viewedRect.contains(itemRect)) {
+    case EnsureVisible: {
+        if (!viewedRect.contains(itemRect)) {
 
-                   if (topDifference < 0) {
-                        // scroll view down
-                        scrollBar->setValue(scrollBar->value() - bottomDifference); 
-                   } else {
-                        // scroll view up
-                        scrollBar->setValue(scrollBar->value() - topDifference);
-                   }
-               } 
+            if (topDifference < 0) {
+                // scroll view down
+                scrollBar->setValue(scrollBar->value() - bottomDifference);
+            } else {
+                // scroll view up
+                scrollBar->setValue(scrollBar->value() - topDifference);
             }
-            break;
-        case PositionAtTop:
-            {
-                //d->viewportOffset = itemRect.top();
-            }
-        default:
-            Q_ASSERT(false); // Not implemented
+        }
+    }
+    break;
+    case PositionAtTop: {
+        //d->viewportOffset = itemRect.top();
+    }
+    default:
+        Q_ASSERT(false); // Not implemented
     }
 }
 
@@ -370,55 +377,56 @@ bool UrlItemView::isIndexHidden(const QModelIndex&) const
     return false;
 }
 
-QModelIndex UrlItemView::moveCursor(CursorAction cursorAction,Qt::KeyboardModifiers )
+QModelIndex UrlItemView::moveCursor(CursorAction cursorAction, Qt::KeyboardModifiers)
 {
     QModelIndex index = currentIndex();
 
     int visualIndex = d->visualOrder.indexOf(index);
 
     switch (cursorAction) {
-        case MoveUp:
-                if (!currentIndex().isValid()) {
-                    const QModelIndex root = model()->index(0, 0);
-                    index = model()->index(model()->rowCount(root) - 1, 0, root);
-                } else {
-                    visualIndex = qMax(0,visualIndex-1);
-                }
-            break;
-        case MoveDown:
-                if (!currentIndex().isValid()) {
-                    const QModelIndex root = model()->index(0, 0);
-                    index = model()->index(0, 0, root);
-                } else {
-                    visualIndex = qMin(d->visualOrder.count()-1,visualIndex+1);
-                }
-            break;
-        default:
-                // Do nothing
-            break;
+    case MoveUp:
+        if (!currentIndex().isValid()) {
+            const QModelIndex root = model()->index(0, 0);
+            index = model()->index(model()->rowCount(root) - 1, 0, root);
+        } else {
+            visualIndex = qMax(0, visualIndex - 1);
+        }
+        break;
+    case MoveDown:
+        if (!currentIndex().isValid()) {
+            const QModelIndex root = model()->index(0, 0);
+            index = model()->index(0, 0, root);
+        } else {
+            visualIndex = qMin(d->visualOrder.count() - 1, visualIndex + 1);
+        }
+        break;
+    default:
+        // Do nothing
+        break;
     }
 
     d->hoveredIndex = QModelIndex();
 
-    return currentIndex().isValid() ? d->visualOrder.value(visualIndex,QModelIndex())
-                                    : index;
+    return currentIndex().isValid() ? d->visualOrder.value(visualIndex, QModelIndex())
+           : index;
 }
 
-void UrlItemView::setSelection(const QRect& rect,QItemSelectionModel::SelectionFlags flags)
+void UrlItemView::setSelection(const QRect& rect, QItemSelectionModel::SelectionFlags flags)
 {
     QItemSelection selection;
-    selection.select(indexAt(rect.topLeft()),indexAt(rect.bottomRight()));
-    selectionModel()->select(selection,flags);
+    selection.select(indexAt(rect.topLeft()), indexAt(rect.bottomRight()));
+    selectionModel()->select(selection, flags);
 }
 
 int UrlItemView::verticalOffset() const
 {
-    return verticalScrollBar()->value(); 
+    return verticalScrollBar()->value();
 }
 
-QRegion UrlItemView::visualRegionForSelection(const QItemSelection& selection) const{
+QRegion UrlItemView::visualRegionForSelection(const QItemSelection& selection) const
+{
     QRegion region;
-    foreach(const QModelIndex& index,selection.indexes()) {
+    foreach(const QModelIndex& index, selection.indexes()) {
         region |= visualRect(index);
     }
     return region;
@@ -430,10 +438,29 @@ void UrlItemView::paintEvent(QPaintEvent *event)
         return;
     }
 
+    QPalette viewPalette(palette());
+    viewPalette.setColor(QPalette::Window, palette().color(QPalette::Active, QPalette::Base));
+    setPalette(viewPalette);
+    setAutoFillBackground(true);
+
     QPainter painter(viewport());
     painter.setRenderHint(QPainter::Antialiasing);
 
-    QHashIterator<QModelIndex,QRect> indexIter(d->itemRects);
+    if (d->dragging && dragDropMode() == QAbstractItemView::DragDrop) {
+        const int y = (d->dropRect.top() + d->dropRect.bottom()) / 2;
+
+        painter.save();
+        QLinearGradient gr(d->dropRect.left(), y, d->dropRect.right(), y);
+        gr.setColorAt(0, palette().base().color());
+        gr.setColorAt(.35, palette().windowText().color());
+        gr.setColorAt(.65, palette().windowText().color());
+        gr.setColorAt(1, palette().base().color());
+        painter.setPen(QPen(gr, 1));
+        painter.drawLine(d->dropRect.left(), y, d->dropRect.right(), y);
+        painter.restore();
+    }
+
+    QHashIterator<QModelIndex, QRect> indexIter(d->itemRects);
     while (indexIter.hasNext()) {
         indexIter.next();
         const QRect itemRect = visualRect(indexIter.key());
@@ -460,7 +487,7 @@ void UrlItemView::paintEvent(QPaintEvent *event)
                     option.rect.setLeft(option.rect.left() + ItemDelegate::ITEM_LEFT_MARGIN);
                     option.rect.setRight(option.rect.right() - ItemDelegate::ITEM_RIGHT_MARGIN);
                 }
-                itemDelegate(index)->paint(&painter,option,index);
+                itemDelegate(index)->paint(&painter, option, index);
             }
         }
     }
@@ -471,6 +498,7 @@ void UrlItemView::resizeEvent(QResizeEvent *)
     updateLayout();
 }
 
+
 void UrlItemView::mouseMoveEvent(QMouseEvent *event)
 {
     const QModelIndex itemUnderMouse = indexAt(event->pos());
@@ -481,6 +509,18 @@ void UrlItemView::mouseMoveEvent(QMouseEvent *event)
         d->hoveredIndex = itemUnderMouse;
         setCurrentIndex(d->hoveredIndex);
     }
+
+    Plasma::Delegate *hoveredItemDelegate =
+                    static_cast<Plasma::Delegate*>(itemDelegate(d->hoveredIndex));
+    if (hoveredItemDelegate->showToolTip() == true) {
+        QModelIndex index = d->hoveredIndex;
+        QString titleText = index.data(Qt::DisplayRole).toString();
+        QString subTitleText = index.data(Plasma::Delegate::SubTitleRole).toString();
+        setToolTip(titleText + "\n" + subTitleText);
+    } else {
+        setToolTip("");
+    }
+
     QAbstractItemView::mouseMoveEvent(event);
 }
 
@@ -492,6 +532,8 @@ void UrlItemView::mousePressEvent(QMouseEvent *event)
 
 void UrlItemView::mouseReleaseEvent(QMouseEvent *event)
 {
+    Q_UNUSED(event)
+
     d->watchedIndexForDrag = QModelIndex();
 }
 
@@ -500,9 +542,66 @@ void UrlItemView::setItemStateProvider(ItemStateProvider *provider)
     d->itemStateProvider = provider;
 }
 
-void UrlItemView::startDrag(Qt::DropActions supportedActions) 
+void UrlItemView::dragEnterEvent(QDragEnterEvent *event)
 {
-    kDebug() << "Starting UrlItemView drag with actions" << supportedActions;
+    if (dragDropMode() != QAbstractItemView::DragDrop) {
+        return;
+    }
+
+    d->dragging = true;
+    setDirtyRegion(d->dropRect);
+
+    event->accept();
+}
+
+void UrlItemView::dragLeaveEvent(QDragLeaveEvent *event)
+{
+    if (dragDropMode() != QAbstractItemView::DragDrop) {
+        return;
+    }
+
+    d->dragging = false;
+    setDirtyRegion(d->dropRect);
+
+    event->accept();
+}
+
+void UrlItemView::dragMoveEvent(QDragMoveEvent *event)
+{
+    QAbstractItemView::dragMoveEvent(event);
+
+    const QPoint pos = event->pos();
+    const QModelIndex index = indexAt(pos);
+    setDirtyRegion(d->dropRect);
+
+    // check to see if it's the header
+    if (d->isFirstHeader(index) && index.row() == 0) {
+        event->ignore();
+        return;
+    }
+
+    if (index.isValid()) {
+        const QRect rect = visualRect(index);
+        const int gap = d->contentsHeight;
+
+        if (d->insertAbove(rect, pos)) {
+            d->dropRect = QRect(rect.left(), rect.top() - gap / 2,
+                                rect.width(), gap);
+        } else if (d->insertBelow(rect, pos)) {
+            d->dropRect = QRect(rect.left(), rect.bottom() + 1 - gap / 2,
+                                rect.width(), gap);
+        } else {
+            d->dropRect = rect;
+        }
+    }
+
+    setDirtyRegion(d->dropRect);
+}
+
+
+void UrlItemView::startDrag(Qt::DropActions supportedActions)
+{
+    //kDebug() << "Starting UrlItemView drag with actions" << supportedActions;
 
     if (!d->watchedIndexForDrag.isValid()) {
         return;
@@ -519,19 +618,52 @@ void UrlItemView::startDrag(Qt::DropActions supportedActions)
 
     QModelIndex idx = selectionModel()->selectedIndexes().first();
     QIcon icon = idx.data(Qt::DecorationRole).value<QIcon>();
+    d->draggedRow = idx.row();
     drag->setPixmap(icon.pixmap(IconSize(KIconLoader::Desktop)));
 
-    Qt::DropAction dropAction = drag->exec();
-    QAbstractItemView::startDrag(supportedActions);
+    d->dropRect = QRect();
+    drag->exec();
 }
 
-void UrlItemView::dropEvent(QDropEvent *)
+void UrlItemView::dropEvent(QDropEvent *event)
 {
-    kDebug() << "UrlItemView drop event";
-}
+    if (!d->dragging) {
+        return;
+    }
 
+    // This special code is necessary in order to be able to
+    // inidcate to the model WHERE the item should be dropped,
+    // Since the model cannot tell where the user dropped the
+    // item. The model itself handles the actual moving of the
+    // item.
+    if (dragDropMode() == QAbstractItemView::DragDrop) {
+        int row;
+        QPoint pos = event->pos();
+        QModelIndex parent = indexAt(pos);
+        const QRect rect = visualRect(parent);
+
+        row = parent.row();
+
+        if(d->insertBelow(rect, pos) && d->draggedRow > row) {
+            row++;
+        } else if(d->insertAbove(rect, pos) && d->draggedRow < row) {
+            row--;
+        }
+
+        model()->dropMimeData(event->mimeData(), event->dropAction(), 
+							    row, 0, parent);
+
+        d->dragging = false;
+
+        event->accept();
+    }
+}
 void UrlItemView::leaveEvent(QEvent *event)
 {
+    Q_UNUSED(event)
+
+    //kDebug() << "UrlItemView leave event";
+
     d->hoveredIndex = QModelIndex();
     setCurrentIndex(QModelIndex());
 }
