@@ -22,6 +22,8 @@
 #include <QtCore/QTimer>
 
 #include <KDebug>
+#include <KUrl>
+#include <KLocalizedString>
 
 namespace SystemTray
 {
@@ -33,6 +35,7 @@ public:
     Private() :
         state(Running),
         percentage(0),
+        eta(0),
         timerId(0),
         killable(false),
         suspendable(false),
@@ -45,6 +48,8 @@ public:
     QString message;
     QString error;
     QString speed;
+    QString destination;
+
 
     QMap<QString, qlonglong> totalAmounts;
     QMap<QString, qlonglong> processedAmounts;
@@ -53,6 +58,7 @@ public:
 
     State state;
     uint percentage;
+    uint eta;
     int timerId;
 
     bool killable : 1;
@@ -144,6 +150,43 @@ void Job::setSpeed(const QString &speed)
     }
 }
 
+QString Job::completedMessage() const
+{
+    KUrl location(d->destination);
+    if (location.isValid()) {
+        if (totalAmounts().value("files") > 1) {
+            location.setFileName(QString());
+        }
+
+        QString destinationString;
+        if (location.isLocalFile()) {
+            destinationString = location.toLocalFile();
+        } else {
+            destinationString = location.prettyUrl();
+        }
+
+        QString destinationLink = QString("<a href=\"%1\">%1</a>").arg(destinationString);
+        if (totalAmounts().value("files") > 1) {
+            return i18np("%1 file, to: %2", "%1 files, to: %2", totalAmounts().value("files"),
+                         destinationLink);
+        } else {
+            return destinationLink;
+        }
+    } else {
+        return QString("%1: %2").arg(labels().value(0).first).arg(labels().value(0).second);
+    }
+}
+
+ulong Job::eta() const
+{
+    return d->eta;
+}
+
+void Job::setEta(ulong eta)
+{
+    d->eta = eta;
+}
+
 QMap<QString, qlonglong> Job::totalAmounts() const
 {
     return d->totalAmounts;
@@ -189,6 +232,9 @@ QList<QPair<QString, QString> > Job::labels() const
 void Job::setLabels(QList<QPair<QString, QString> > labels)
 {
     d->labels = labels;
+    if (d->labels.count() > 1 && d->destination.isEmpty()) {
+        d->destination = d->labels.value(1).second;
+    }
     scheduleChangedSignal();
 }
 
