@@ -242,9 +242,6 @@ void AbstractTaskItem::toolTipAboutToShow()
 {
     if (m_applet->showToolTip()) {
         updateToolTip();
-        connect(Plasma::ToolTipManager::self(),
-                SIGNAL(windowPreviewActivated(WId,Qt::MouseButtons,Qt::KeyboardModifiers,QPoint)),
-                this, SLOT(activateWindow(WId,Qt::MouseButtons)));
     } else {
         Plasma::ToolTipManager::self()->clearContent(this);
     }
@@ -253,9 +250,6 @@ void AbstractTaskItem::toolTipAboutToShow()
 void AbstractTaskItem::toolTipHidden()
 {
     Plasma::ToolTipManager::self()->clearContent(this);
-    disconnect(Plasma::ToolTipManager::self(),
-               SIGNAL(windowPreviewActivated(WId,Qt::MouseButtons,Qt::KeyboardModifiers,QPoint)),
-               this, SLOT(activateWindow(WId,Qt::MouseButtons)));
 }
 
 void AbstractTaskItem::activateWindow(WId id, Qt::MouseButtons buttons)
@@ -890,12 +884,14 @@ QRectF AbstractTaskItem::iconRect(const QRectF &b) const
 
     QSize iconSize = m_icon.actualSize(bounds.size().toSize());
 
-    if (iconSize.width() > KIconLoader::SizeSmall && iconSize.width() < KIconLoader::SizeSmallMedium) {
-        iconSize = QSize(KIconLoader::SizeSmall, KIconLoader::SizeSmall);
-    } else if (iconSize.width() > KIconLoader::SizeSmallMedium && iconSize.width() < KIconLoader::SizeMedium) {
-        iconSize = QSize(KIconLoader::SizeSmallMedium, KIconLoader::SizeSmallMedium);
-    } else if (iconSize.width() > KIconLoader::SizeMedium && iconSize.width() < KIconLoader::SizeLarge) {
-        iconSize = QSize(KIconLoader::SizeMedium, KIconLoader::SizeMedium);
+    if (iconSize.width() == iconSize.height()) {
+        if (iconSize.width() > KIconLoader::SizeSmall && iconSize.width() < KIconLoader::SizeSmallMedium) {
+            iconSize = QSize(KIconLoader::SizeSmall, KIconLoader::SizeSmall);
+        } else if (iconSize.width() > KIconLoader::SizeSmallMedium && iconSize.width() < KIconLoader::SizeMedium) {
+            iconSize = QSize(KIconLoader::SizeSmallMedium, KIconLoader::SizeSmallMedium);
+        } else if (iconSize.width() > KIconLoader::SizeMedium && iconSize.width() < KIconLoader::SizeLarge) {
+            iconSize = QSize(KIconLoader::SizeMedium, KIconLoader::SizeMedium);
+        }
     }
 
     return QStyle::alignedRect(QApplication::layoutDirection(), Qt::AlignLeft | Qt::AlignVCenter,
