@@ -242,7 +242,7 @@ void Panel::adjustLastSpace()
 
     if (formFactor() == Plasma::Vertical) {
         foreach (Applet *applet, applets()) {
-            if (applet->sizePolicy().verticalPolicy() == QSizePolicy::Expanding || applet->sizePolicy().verticalPolicy() == QSizePolicy::MinimumExpanding) {
+            if (applet->sizePolicy().verticalPolicy() && QSizePolicy::ExpandFlag) {
                 useSpacer = false;
                 break;
             }
@@ -553,14 +553,13 @@ void Panel::updateBorders(const QRect &geom, bool themeChange)
             break;
         case TopEdge:
             bottomHeight = qMin(bottomHeight, qMax(qreal(1), size().height() - KIconLoader::SizeMedium));
-            break;
+            break;  
         case BottomEdge:
             topHeight = qMin(topHeight, qMax(qreal(1), size().height() - KIconLoader::SizeMedium));
             break;
         default:
             break;
         }
-
         qreal oldLeft = leftWidth;
         qreal oldTop = topHeight;
         qreal oldRight = rightWidth;
@@ -571,13 +570,6 @@ void Panel::updateBorders(const QRect &geom, bool themeChange)
         }
 
         layout()->setContentsMargins(leftWidth, topHeight, rightWidth, bottomHeight);
-        if (formFactor() == Plasma::Vertical) {
-            setPreferredHeight(preferredHeight() - (oldBottom - bottomHeight));
-        } else if (QApplication::layoutDirection() == Qt::LeftToRight) {
-            setPreferredWidth(preferredWidth() - (oldRight - rightWidth));
-        } else {
-            setPreferredWidth(preferredWidth() - (oldLeft - leftWidth));
-        }
 
         layout()->invalidate();
         resize(preferredSize());
