@@ -1,5 +1,6 @@
 /*
  *   Copyright 2007 Aaron Seigo <aseigo@kde.org>
+ *   Copyright 2008 Alex Merry <alex.merry@kdemail.net>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -20,23 +21,22 @@
 #include "timeengine.h"
 
 #include <QDate>
+#include <QDBusConnection>
 #include <QStringList>
 #include <QTime>
-#include <QTimer>
 
-#include <KDebug>
 #include <KLocale>
 #include <KSystemTimeZones>
 #include <KDateTime>
 
-#include "plasma/datacontainer.h"
+#include "timesource.h"
 
 //timezone is defined in msvc
 #ifdef timezone
 #undef timezone
 #endif
 
-TimeEngine::TimeEngine(QObject* parent, const QVariantList& args)
+TimeEngine::TimeEngine(QObject *parent, const QVariantList &args)
     : Plasma::DataEngine(parent, args)
 {
     Q_UNUSED(args)
@@ -51,44 +51,50 @@ TimeEngine::~TimeEngine()
 {
 }
 
+void TimeEngine::init()
+{
+    //QDBusInterface *ktimezoned = new QDBusInterface("org.kde.kded", "/modules/ktimezoned", "org.kde.KTimeZoned");
+    QDBusConnection dbus = QDBusConnection::sessionBus();
+    dbus.connect(QString(), QString(), "org.kde.KTimeZoned", "configChanged", this, SLOT(tzConfigChanged()));
+}
+
+void TimeEngine::tzConfigChanged()
+{
+    TimeSource *s = qobject_cast<TimeSource *>(containerForSource("Local"));
+
+    if (s) {
+        s->setTimeZone("Local");
+    }
+
+    updateAllSources();
+}
+
+QStringList TimeEngine::sources() const
+{
+    QStringList timezones(KSystemTimeZones::zones().keys());
+    timezones << "Local";
+    return timezones;
+}
+
 bool TimeEngine::sourceRequestEvent(const QString &name)
 {
-    //kDebug() << "TimeEngine::sourceRequested " << name;
-    return updateSourceEvent(name);
+    addSource(new TimeSource(name));
+    return true;
 }
 
 bool TimeEngine::updateSourceEvent(const QString &tz)
 {
-    //kDebug() << "TimeEngine::updateTime()";
+    TimeSource *s = qobject_cast<TimeSource *>(containerForSource(tz));
 
-    QString timezone;
-
-    static const QString localName = I18N_NOOP("Local");
-    if (tz == localName) {
-        setData(localName, I18N_NOOP("Time"), QTime::currentTime());
-        setData(localName, I18N_NOOP("Date"), QDate::currentDate());
-        // this is relatively cheap - KSTZ::local() is cached
-        timezone = KSystemTimeZones::local().name();
-    } else {
-        KTimeZone newTz = KSystemTimeZones::zone(tz);
-        if (!newTz.isValid()) {
-            return false;
-        }
-
-        KDateTime dt = KDateTime::currentDateTime(newTz);
-        setData(tz, I18N_NOOP("Time"), dt.time());
-        setData(tz, I18N_NOOP("Date"), dt.date());
-        timezone = tz;
+    if (s) {
+        s->updateTime();
+        scheduleSourcesUpdated();
+        return true;
     }
 
-    QString trTimezone = i18n(timezone.toUtf8());
-    setData(tz, I18N_NOOP("Timezone"), trTimezone);
-    QStringList tzParts = trTimezone.split("/");
-
-    setData(tz, I18N_NOOP("Timezone Continent"), tzParts.value(0));
-    setData(tz, I18N_NOOP("Timezone City"), tzParts.value(1));
-
-    return true;
+    return false;
 }
 
+K_EXPORT_PLASMA_DATAENGINE(time, TimeEngine)
+
 #include "timeengine.moc"
