@@ -28,12 +28,14 @@
 #include <KServiceTypeTrader>
 
 ServiceRunner::ServiceRunner(QObject *parent, const QVariantList &args)
-    : Plasma::AbstractRunner( parent )
+    : Plasma::AbstractRunner(parent, args)
 {
     Q_UNUSED(args)
 
     setObjectName("Application");
     setPriority(AbstractRunner::HighestPriority);
+
+    addSyntax(Plasma::RunnerSyntax(":q:", i18n("Finds applications whose name or description match :q:")));
 }
 
 ServiceRunner::~ServiceRunner()
@@ -47,7 +49,6 @@ void ServiceRunner::match(Plasma::RunnerContext &context)
         return;
     }
 
-    QMutexLocker lock(bigLock());
     // Search for applications which are executable and case-insensitively match the search term
     // See http://techbase.kde.org/Development/Tutorials/Services/Traders#The_KTrader_Query_Language
     // if the following is unclear to you.
@@ -59,18 +60,23 @@ void ServiceRunner::match(Plasma::RunnerContext &context)
     QHash<QString, bool> seen;
     if (!services.isEmpty()) {
         //kDebug() << service->name() << "is an exact match!" << service->storageId() << service->exec();
-        KService::Ptr service = services.at(0);
-        if (!service->noDisplay()) {
-            Plasma::QueryMatch match(this);
-            match.setType(Plasma::QueryMatch::ExactMatch);
-            setupAction(service, match);
-            match.setRelevance(1);
-            matches << match;
-            seen[service->storageId()] = true;
-            seen[service->exec()] = true;
+        foreach (const KService::Ptr &service, services) {
+            if (!service->noDisplay()) {
+                Plasma::QueryMatch match(this);
+                match.setType(Plasma::QueryMatch::ExactMatch);
+                setupMatch(service, match);
+                match.setRelevance(1);
+                matches << match;
+                seen[service->storageId()] = true;
+                seen[service->exec()] = true;
+            }
         }
     }
 
+    if (!context.isValid()) {
+        return;
+    }
+
     // Search for applications which are executable and the term case-insensitive matches any of
     // * a substring of one of the keywords
     // * a substring of the GenericName field
@@ -83,6 +89,10 @@ void ServiceRunner::match(Plasma::RunnerContext &context)
 
     //kDebug() << "got " << services.count() << " services from " << query;
     foreach (const KService::Ptr &service, services) {
+        if (!context.isValid()) {
+            return;
+        }
+
         if (service->noDisplay()) {
             continue;
         }
@@ -100,20 +110,20 @@ void ServiceRunner::match(Plasma::RunnerContext &context)
 
         Plasma::QueryMatch match(this);
         match.setType(Plasma::QueryMatch::PossibleMatch);
-        setupAction(service, match);
+        setupMatch(service, match);
         qreal relevance(0.6);
 
         if (service->name().contains(term, Qt::CaseInsensitive)) {
             relevance = 0.8;
 
             if (service->name().startsWith(term, Qt::CaseInsensitive)) {
-                relevance += 0.5;
+                relevance += 0.1;
             }
         } else if (service->genericName().contains(term, Qt::CaseInsensitive)) {
             relevance = 0.7;
 
             if (service->genericName().startsWith(term, Qt::CaseInsensitive)) {
-                relevance += 0.5;
+                relevance += 0.1;
             }
         }
 
@@ -150,14 +160,13 @@ void ServiceRunner::match(Plasma::RunnerContext &context)
 void ServiceRunner::run(const Plasma::RunnerContext &context, const Plasma::QueryMatch &match)
 {
     Q_UNUSED(context);
-    QMutexLocker lock(bigLock());
     KService::Ptr service = KService::serviceByStorageId(match.data().toString());
     if (service) {
         KRun::run(*service, KUrl::List(), 0);
     }
 }
 
-void ServiceRunner::setupAction(const KService::Ptr &service, Plasma::QueryMatch &match)
+void ServiceRunner::setupMatch(const KService::Ptr &service, Plasma::QueryMatch &match)
 {
     const QString name = service->name();
 
