@@ -11,7 +11,7 @@
 #ifndef BACKGROUNDDIALOG_H
 #define BACKGROUNDDIALOG_H
 
-#include <KDialog>
+#include <KConfigDialog>
 #include "ui_BackgroundDialog.h"
 
 namespace Plasma {
@@ -22,13 +22,31 @@ namespace Plasma {
 class ThemeModel;
 class WallpaperPreview;
 class QStandardItemModel;
+class ScreenPreviewWidget;
 
-class BackgroundDialog : public KDialog, public Ui::BackgroundDialog
+// WallpaperWidget is passed the wallpaper
+// in createConfigurationInterface so it can notify
+// of changes (used to enable the apply button)
+class WallpaperWidget :public QWidget
+{
+    Q_OBJECT
+public:
+      WallpaperWidget(QWidget *parent) :QWidget(parent) {}
+
+signals:
+    void modified(bool isModified);
+
+public slots:
+    void settingsChanged(bool isModified);
+};
+
+class BackgroundDialog : public KConfigDialog, public Ui::BackgroundDialog
 {
     Q_OBJECT
 public:
     BackgroundDialog(const QSize &res, Plasma::Containment *containment,
-                     Plasma::View *view, QWidget *parent = 0);
+                     Plasma::View *view, QWidget* parent, const QString &id,
+                     KConfigSkeleton *s);
     ~BackgroundDialog();
 
     void reloadConfig();
@@ -36,6 +54,9 @@ public:
 public slots:
     void saveConfig();
 
+protected:
+    virtual bool hasChanged();
+
 private:
     KConfigGroup wallpaperConfig(const QString &plugin);
 
@@ -43,6 +64,7 @@ private slots:
     void getNewThemes();
     void changeBackgroundMode(int mode);
     void cleanup();
+    void settingsModified(bool modified = true);
 
 private:
     ThemeModel* m_themeModel;
@@ -50,7 +72,9 @@ private:
     Plasma::Wallpaper* m_wallpaper;
     Plasma::View* m_view;
     Plasma::Containment* m_containment;
-    WallpaperPreview* m_preview;
+    ScreenPreviewWidget* m_preview;
+    KPageWidgetItem *m_appearanceItem;
+    bool m_modified;
 };
 
 #endif // BACKGROUNDDIALOG_H
