@@ -36,6 +36,7 @@
 #include <kworkspace/kworkspace.h>
 #include <KApplication>
 
+#include <QPointer>
 #include <QWidget>
 #include <QTimer>
 
@@ -102,26 +103,26 @@ public:
             , battery(0)
             , currentConfig(0)
             , status(PowerDevilDaemon::NoAction)
-            , ckSessionInterface(0) {};
+            , ckSessionInterface(0) {}
 
-    Solid::Control::PowerManager::Notifier * notifier;
+    Solid::Control::PowerManager::Notifier *notifier;
     QPointer<Solid::Battery> battery;
 
-    OrgFreedesktopScreenSaverInterface * screenSaverIface;
-    OrgKdeKSMServerInterfaceInterface * ksmServerIface;
+    OrgFreedesktopScreenSaverInterface *screenSaverIface;
+    OrgKdeKSMServerInterfaceInterface *ksmServerIface;
 //  Now we send a signal to trigger the configuration of Kscreensaver (Bug #177123) and we don't need the interface anymore
 //     OrgKdeScreensaverInterface * kscreenSaverIface;
 
     KComponentData applicationData;
     KSharedConfig::Ptr profilesConfig;
-    KConfigGroup * currentConfig;
-    PollSystemLoader * pollLoader;
-    SuspensionLockHandler * lockHandler;
+    KConfigGroup *currentConfig;
+    PollSystemLoader *pollLoader;
+    SuspensionLockHandler *lockHandler;
 
     QString currentProfile;
     QStringList availableProfiles;
 
-    KNotification *notification;
+    QPointer<KNotification> notification;
     QTimer *notificationTimer;
 
     PowerDevilDaemon::IdleStatus status;
@@ -163,8 +164,7 @@ PowerDevilDaemon::PowerDevilDaemon(QObject *parent, const QList<QVariant>&)
 
     if (conn.interface()->isServiceRegistered("org.freedesktop.PowerManagement") ||
             conn.interface()->isServiceRegistered("com.novell.powersave") ||
-            conn.interface()->isServiceRegistered("org.freedesktop.Policy.Power") ||
-            conn.interface()->isServiceRegistered("org.kde.powerdevilsystem")) {
+            conn.interface()->isServiceRegistered("org.freedesktop.Policy.Power")) {
         kError() << "PowerDevil not initialized, another power manager has been detected";
         return;
     }
@@ -222,9 +222,8 @@ PowerDevilDaemon::PowerDevilDaemon(QObject *parent, const QList<QVariant>&)
     new PowerDevilAdaptor(this);
     new PowerManagementConnector(this);
 
-    // This gets registered to avoid double copies.
-    QDBusConnection::sessionBus().registerService("org.kde.powerdevilsystem");
     conn.interface()->registerService("org.freedesktop.Policy.Power");
+    QDBusConnection::sessionBus().registerService("org.kde.powerdevil");
     // All systems up Houston, let's go!
     refreshStatus();
 }
@@ -289,7 +288,7 @@ bool PowerDevilDaemon::recacheBatteryPointer(bool force)
         if (!connect(d->battery, SIGNAL(chargePercentChanged(int, const QString &)), this,
                      SLOT(batteryChargePercentChanged(int, const QString &)))) {
 
-            emitCriticalNotification("powerdevilerror", i18n("Could not connect to battery interface!\n"
+            emitCriticalNotification("powerdevilerror", i18n("Could not connect to battery interface.\n"
                                      "Please check your system configuration"));
             return false;
         }
@@ -409,12 +408,21 @@ void PowerDevilDaemon::acAdapterStateChanged(int state, bool forced)
 {
     if (state == Solid::Control::PowerManager::Plugged && !forced) {
         setACPlugged(true);
-        emitNotification("pluggedin", i18n("The power adaptor has been plugged in"));
+
+        // If the AC Adaptor has been plugged in, let's clear some pending suspend actions
+        if (!d->lockHandler->canStartNotification()) {
+            cleanUpTimer();
+            d->lockHandler->releaseNotificationLock();
+            emitNotification("pluggedin", i18n("The power adaptor has been plugged in. Any pending "
+                                               "suspend actions have been canceled."));
+        } else {
+            emitNotification("pluggedin", i18n("The power adaptor has been plugged in."));
+        }
     }
 
     if (state == Solid::Control::PowerManager::Unplugged && !forced) {
         setACPlugged(false);
-        emitNotification("unplugged", i18n("The power adaptor has been unplugged"));
+        emitNotification("unplugged", i18n("The power adaptor has been unplugged."));
     }
 
     if (!forced) {
@@ -570,10 +578,10 @@ void PowerDevilDaemon::batteryChargePercentChanged(int percent, const QString &u
         switch (PowerDevilSettings::batLowAction()) {
         case Shutdown:
             if (PowerDevilSettings::waitBeforeSuspending()) {
-                emitWarningNotification("criticalbattery", i18n("Your battery has reached "
-                                        "critical level, the PC will be halted in %1 seconds. "
-                                        "Click here to block the process.",
-                                        PowerDevilSettings::waitBeforeSuspendingTime()),
+                emitWarningNotification("criticalbattery",
+                                        i18np("Your battery level is critical, the computer will be halted in 1 second.",
+                                              "Your battery level is critical, the computer will be halted in %1 seconds.",
+                                              PowerDevilSettings::waitBeforeSuspendingTime()),
                                         SLOT(shutdown()));
             } else {
                 shutdown();
@@ -581,10 +589,10 @@ void PowerDevilDaemon::batteryChargePercentChanged(int percent, const QString &u
             break;
         case S2Disk:
             if (PowerDevilSettings::waitBeforeSuspending()) {
-                emitWarningNotification("criticalbattery", i18n("Your battery has reached "
-                                        "critical level, the PC will be suspended to disk in "
-                                        "%1 seconds. Click here to block the process.",
-                                        PowerDevilSettings::waitBeforeSuspendingTime()),
+                emitWarningNotification("criticalbattery",
+                                        i18np("Your battery level is critical, the computer will be suspended to disk in 1 second.",
+                                              "Your battery level is critical, the computer will be suspended to disk in %1 seconds.",
+                                              PowerDevilSettings::waitBeforeSuspendingTime()),
                                         SLOT(suspendToDisk()));
             } else {
                 suspendToDisk();
@@ -592,10 +600,10 @@ void PowerDevilDaemon::batteryChargePercentChanged(int percent, const QString &u
             break;
         case S2Ram:
             if (PowerDevilSettings::waitBeforeSuspending()) {
-                emitWarningNotification("criticalbattery", i18n("Your battery has reached "
-                                        "critical level, the PC will be suspended to RAM in "
-                                        "%1 seconds. Click here to block the process",
-                                        PowerDevilSettings::waitBeforeSuspendingTime()),
+                emitWarningNotification("criticalbattery",
+                                        i18np("Your battery level is critical, the computer will be suspended to RAM in 1 second.",
+                                              "Your battery level is critical, the computer will be suspended to RAM in %1 seconds.",
+                                              PowerDevilSettings::waitBeforeSuspendingTime()),
                                         SLOT(suspendToRam()));
             } else {
                 suspendToRam();
@@ -603,25 +611,24 @@ void PowerDevilDaemon::batteryChargePercentChanged(int percent, const QString &u
             break;
         case Standby:
             if (PowerDevilSettings::waitBeforeSuspending()) {
-                emitWarningNotification("criticalbattery", i18n("Your battery has reached "
-                                        "critical level, the PC is going Standby in %1 seconds. "
-                                        "Click here to block the process.",
-                                        PowerDevilSettings::waitBeforeSuspendingTime()),
+                emitWarningNotification("criticalbattery",
+                                        i18np("Your battery level is critical, the computer will be put into standby in 1 second.",
+                                              "Your battery level is critical, the computer will be put into standby in %1 seconds.",
+                                              PowerDevilSettings::waitBeforeSuspendingTime()),
                                         SLOT(standby()));
             } else {
                 standby();
             }
             break;
         default:
-            emitWarningNotification("criticalbattery", i18n("Your battery has reached "
-                                    "critical level, save your work as soon as possible!"));
+            emitWarningNotification("criticalbattery", i18n("Your battery level is critical: save your work as soon as possible."));
             break;
         }
     } else if (charge == PowerDevilSettings::batteryWarningLevel()) {
-        emitWarningNotification("warningbattery", i18n("Your battery has reached warning level"));
+        emitWarningNotification("warningbattery", i18n("Your battery has reached the warning level."));
         refreshStatus();
     } else if (charge == PowerDevilSettings::batteryLowLevel()) {
-        emitWarningNotification("lowbattery", i18n("Your battery has reached low level"));
+        emitWarningNotification("lowbattery", i18n("Your battery has reached a low level."));
         refreshStatus();
     }
 }
@@ -657,6 +664,9 @@ void PowerDevilDaemon::buttonPressed(int but)
         case Lock:
             lockScreen();
             break;
+        case TurnOffScreen:
+            turnOffScreen();
+            break;
         default:
             break;
         }
@@ -681,6 +691,9 @@ void PowerDevilDaemon::buttonPressed(int but)
         case ShutdownDialog:
             shutdownDialog();
             break;
+        case TurnOffScreen:
+            turnOffScreen();
+            break;
         default:
             break;
         }
@@ -705,6 +718,9 @@ void PowerDevilDaemon::buttonPressed(int but)
         case ShutdownDialog:
             shutdownDialog();
             break;
+        case TurnOffScreen:
+            turnOffScreen();
+            break;
         default:
             break;
         }
@@ -742,9 +758,9 @@ void PowerDevilDaemon::shutdownNotification(bool automated)
     }
 
     if (PowerDevilSettings::waitBeforeSuspending()) {
-        emitNotification("doingjob", i18n("The computer will be halted in %1 seconds. Click "
-                                          "here to block the process.",
-                                          PowerDevilSettings::waitBeforeSuspendingTime()),
+        emitNotification("doingjob", i18np("The computer will be halted in 1 second.",
+                                           "The computer will be halted in %1 seconds.",
+                                           PowerDevilSettings::waitBeforeSuspendingTime()),
                          SLOT(shutdown()));
     } else {
         shutdown();
@@ -762,9 +778,9 @@ void PowerDevilDaemon::suspendToDiskNotification(bool automated)
     }
 
     if (PowerDevilSettings::waitBeforeSuspending()) {
-        emitNotification("doingjob", i18n("The computer will be suspended to disk in %1 "
-                                          "seconds. Click here to block the process.",
-                                          PowerDevilSettings::waitBeforeSuspendingTime()),
+        emitNotification("doingjob", i18np("The computer will be suspended to disk in 1 second.",
+                                           "The computer will be suspended to disk in %1 seconds.",
+                                           PowerDevilSettings::waitBeforeSuspendingTime()),
                          SLOT(suspendToDisk()));
     } else {
         suspendToDisk();
@@ -782,9 +798,9 @@ void PowerDevilDaemon::suspendToRamNotification(bool automated)
     }
 
     if (PowerDevilSettings::waitBeforeSuspending()) {
-        emitNotification("doingjob", i18n("The computer will be suspended to RAM in %1 "
-                                          "seconds. Click here to block the process.",
-                                          PowerDevilSettings::waitBeforeSuspendingTime()),
+        emitNotification("doingjob", i18np("The computer will be suspended to RAM in 1 second.",
+                                           "The computer will be suspended to RAM in %1 seconds.",
+                                           PowerDevilSettings::waitBeforeSuspendingTime()),
                          SLOT(suspendToRam()));
     } else {
         suspendToRam();
@@ -802,9 +818,9 @@ void PowerDevilDaemon::standbyNotification(bool automated)
     }
 
     if (PowerDevilSettings::waitBeforeSuspending()) {
-        emitNotification("doingjob", i18n("The computer will be put into standby in %1 "
-                                          "seconds. Click here to block the process.",
-                                          PowerDevilSettings::waitBeforeSuspendingTime()),
+        emitNotification("doingjob", i18np("The computer will be put into standby in 1 second.",
+                                           "The computer will be put into standby in %1 seconds.",
+                                           PowerDevilSettings::waitBeforeSuspendingTime()),
                          SLOT(standby()));
     } else {
         standby();
@@ -829,7 +845,7 @@ void PowerDevilDaemon::shutdown(bool automated)
 
 void PowerDevilDaemon::shutdownDialog()
 {
-    d->ksmServerIface->logout((int)KWorkSpace::ShutdownConfirmYes, (int)KWorkSpace::ShutdownTypeNone,
+    d->ksmServerIface->logout((int)KWorkSpace::ShutdownConfirmYes, (int)KWorkSpace::ShutdownTypeDefault,
                               (int)KWorkSpace::ShutdownModeDefault);
 }
 
@@ -1015,6 +1031,10 @@ void PowerDevilDaemon::poll(int idle)
             POLLER_CALL(d->pollLoader->poller(), catchIdleEvent());
             lockScreen();
             break;
+        case TurnOffScreen:
+            POLLER_CALL(d->pollLoader->poller(), catchIdleEvent());
+            turnOffScreen();
+            break;
         default:
             break;
         }
@@ -1103,12 +1123,13 @@ void PowerDevilDaemon::emitCriticalNotification(const QString &evid, const QStri
     } else {
         d->notification = KNotification::event(evid, message, KIcon(iconname).pixmap(20, 20),
                                                0, KNotification::Persistent, d->applicationData);
+        d->notification->setActions(QStringList() << i18nc("Interrupts the suspension/shutdown process", "Abort Action"));
 
         connect(d->notificationTimer, SIGNAL(timeout()), slot);
         connect(d->notificationTimer, SIGNAL(timeout()), SLOT(cleanUpTimer()));
 
-        d->lockHandler->connect(d->notification, SIGNAL(closed()), d->lockHandler, SLOT(releaseNotificationLock()));
-        connect(d->notification, SIGNAL(closed()), SLOT(cleanUpTimer()));
+        d->lockHandler->connect(d->notification, SIGNAL(activated(unsigned int)), d->lockHandler, SLOT(releaseNotificationLock()));
+        connect(d->notification, SIGNAL(activated(unsigned int)), SLOT(cleanUpTimer()));
 
         d->notificationTimer->start(PowerDevilSettings::waitBeforeSuspendingTime() * 1000);
     }
@@ -1130,12 +1151,13 @@ void PowerDevilDaemon::emitWarningNotification(const QString &evid, const QStrin
     } else {
         d->notification = KNotification::event(evid, message, KIcon(iconname).pixmap(20, 20),
                                                0, KNotification::Persistent, d->applicationData);
+        d->notification->setActions(QStringList() << i18nc("Interrupts the suspension/shutdown process", "Abort Action"));
 
         connect(d->notificationTimer, SIGNAL(timeout()), slot);
         connect(d->notificationTimer, SIGNAL(timeout()), SLOT(cleanUpTimer()));
 
-        d->lockHandler->connect(d->notification, SIGNAL(closed()), d->lockHandler, SLOT(releaseNotificationLock()));
-        connect(d->notification, SIGNAL(closed()), SLOT(cleanUpTimer()));
+        d->lockHandler->connect(d->notification, SIGNAL(activated(unsigned int)), d->lockHandler, SLOT(releaseNotificationLock()));
+        connect(d->notification, SIGNAL(activated(unsigned int)), SLOT(cleanUpTimer()));
 
         d->notificationTimer->start(PowerDevilSettings::waitBeforeSuspendingTime() * 1000);
     }
@@ -1157,12 +1179,13 @@ void PowerDevilDaemon::emitNotification(const QString &evid, const QString &mess
     } else {
         d->notification = KNotification::event(evid, message, KIcon(iconname).pixmap(20, 20),
                                                0, KNotification::Persistent, d->applicationData);
+        d->notification->setActions(QStringList() << i18nc("Interrupts the suspension/shutdown process", "Abort Action"));
 
         connect(d->notificationTimer, SIGNAL(timeout()), slot);
         connect(d->notificationTimer, SIGNAL(timeout()), SLOT(cleanUpTimer()));
 
-        d->lockHandler->connect(d->notification, SIGNAL(closed()), d->lockHandler, SLOT(releaseNotificationLock()));
-        connect(d->notification, SIGNAL(closed()), SLOT(cleanUpTimer()));
+        d->lockHandler->connect(d->notification, SIGNAL(activated(unsigned int)), d->lockHandler, SLOT(releaseNotificationLock()));
+        connect(d->notification, SIGNAL(activated(unsigned int)), SLOT(cleanUpTimer()));
 
         d->notificationTimer->start(PowerDevilSettings::waitBeforeSuspendingTime() * 1000);
     }
@@ -1173,10 +1196,10 @@ void PowerDevilDaemon::cleanUpTimer()
     kDebug() << "Disconnecting signals";
 
     d->notificationTimer->disconnect();
-    d->notification->disconnect();
     d->notificationTimer->stop();
 
     if (d->notification) {
+        d->notification->disconnect();
         d->notification->deleteLater();
     }
 }
@@ -1204,7 +1227,7 @@ KConfigGroup * PowerDevilDaemon::getCurrentProfile(bool forcereload)
 
     if (!d->currentConfig->isValid() || !d->currentConfig->entryMap().size()) {
         emitCriticalNotification("powerdevilerror", i18n("The profile \"%1\" has been selected, "
-                                 "but it does not exist!\nPlease check your PowerDevil configuration.",
+                                 "but it does not exist.\nPlease check your PowerDevil configuration.",
                                  d->currentProfile));
         reloadProfile();
         delete d->currentConfig;
@@ -1349,7 +1372,7 @@ QVariantMap PowerDevilDaemon::getSupportedSuspendMethods()
     }
 
     if (methods & Solid::Control::PowerManager::ToRam) {
-        retlist[i18n("Suspend to Ram")] = (int) S2Ram;
+        retlist[i18n("Suspend to RAM")] = (int) S2Ram;
     }
 
     if (methods & Solid::Control::PowerManager::Standby) {
@@ -1421,6 +1444,14 @@ void PowerDevilDaemon::setBrightness(int value)
 
 void PowerDevilDaemon::turnOffScreen()
 {
+    if (!checkIfCurrentSessionActive()) {
+        return;
+    }
+
+    if (PowerDevilSettings::configLockScreen()) {
+        lockScreen();
+    }
+
 #ifdef HAVE_DPMS
 
     CARD16 dummy;
@@ -1475,18 +1506,12 @@ void PowerDevilDaemon::profileFirstLoad()
 
 bool PowerDevilDaemon::toggleCompositing(bool enabled)
 {
-    KSharedConfigPtr KWinConfig = KSharedConfig::openConfig("kwinrc");
-    KConfigGroup config(KWinConfig, "Compositing");
-    bool state = config.readEntry("Enabled", false);
+    QDBusInterface kwiniface("org.kde.kwin", "/KWin", "org.kde.KWin", QDBusConnection::sessionBus());
 
-    if (state != enabled) {
-        config.writeEntry("Enabled", enabled);
-
-        QDBusMessage message = QDBusMessage::createSignal("/KWin",
-                               "org.kde.KWin",
-                               "reloadConfig");
-        QDBusConnection::sessionBus().send(message);
+    QDBusReply<bool> state = kwiniface.call("compositingActive");
 
+    if (state.value() != enabled) {
+        kwiniface.call("toggleCompositing");
         return true;
     } else {
         return false;
